@@ -33,7 +33,6 @@
 #include "ompi/constants.h"
 #include "opal/class/opal_object.h"
 #include "opal/class/opal_hash_table.h"
-#include "orte/mca/gpr/gpr_types.h"
 
 #define ATTR_HASH_SIZE 10
 
@@ -45,14 +44,14 @@
 #define OMPI_KEYVAL_F77_MPI1       0x0004
 
 
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
+BEGIN_C_DECLS
+
 enum ompi_attribute_type_t {
-    UNUSED_ATTR = 0, /**< Make the compilers happy when we have to construct an attribute */
+    UNUSED_ATTR = 0, /**< Make the compilers happy when we have to construct
+                      * an attribute */
     COMM_ATTR,       /**< The attribute belongs to a comm object. Starts
-		      * with 1 so that we can have it initialized to 0
-		      * using memset in the constructor */
+                      * with 1 so that we can have it initialized to 0
+                      * using memset in the constructor */
     TYPE_ATTR,       /**< The attribute belongs to datatype object */
     WIN_ATTR         /**< The attribute belongs to a win object */
 };
@@ -68,7 +67,7 @@ typedef void (ompi_mpi1_fortran_copy_attr_function)(MPI_Fint *oldobj,
                                                     MPI_Fint *extra_state, 
                                                     MPI_Fint *attr_in,
                                                     MPI_Fint *attr_out,
-                                                    ompi_fortran_logical_t *flag, 
+                                                    ompi_fortran_logical_t *flag,
                                                     MPI_Fint *ierr);
 typedef void (ompi_mpi1_fortran_delete_attr_function)(MPI_Fint *obj, 
                                                       MPI_Fint *keyval,
@@ -85,7 +84,7 @@ typedef void (ompi_mpi2_fortran_copy_attr_function)(MPI_Fint *oldobj,
                                                     void *extra_state, 
                                                     void *attr_in, 
                                                     void *attr_out,
-                                                    ompi_fortran_logical_t *flag, 
+                                                    ompi_fortran_logical_t *flag,
                                                     MPI_Fint *ierr);
 typedef void (ompi_mpi2_fortran_delete_attr_function)(MPI_Fint *obj, 
                                                       MPI_Fint *keyval,
@@ -99,11 +98,14 @@ typedef void (ompi_mpi2_fortran_delete_attr_function)(MPI_Fint *obj,
  * on the main communicator.
  */
 typedef int (MPI_Comm_internal_copy_attr_function)(MPI_Comm, int, void *,
-                                                   void *, void *, int *, MPI_Comm);
+                                                   void *, void *, int *, 
+                                                   MPI_Comm);
 typedef int (MPI_Type_internal_copy_attr_function)(MPI_Datatype, int, void *,
-                                                   void *, void *, int *, MPI_Datatype);
+                                                   void *, void *, int *, 
+                                                   MPI_Datatype);
 typedef int (MPI_Win_internal_copy_attr_function)(MPI_Win, int, void *,
-                                                  void *, void *, int *, MPI_Win);
+                                                  void *, void *, int *, 
+                                                  MPI_Win);
 
 typedef void (ompi_attribute_keyval_destructor_fn_t)(int);
 
@@ -151,22 +153,22 @@ typedef union ompi_attribute_fortran_ptr_t ompi_attribute_fortran_ptr_t;
 struct ompi_attribute_keyval_t {
     opal_object_t super;
     ompi_attribute_type_t attr_type; /**< One of COMM/WIN/DTYPE. This
-				       will be used to cast the
-				       copy/delete attribute functions
-				       properly and error checking */
+                                       will be used to cast the
+                                       copy/delete attribute functions
+                                       properly and error checking */
     int attr_flag; /**< flag field: contains "OMPI_KEYVAL_PREDEFINED",
-		      "OMPI_KEYVAL_F77"  */
+                      "OMPI_KEYVAL_F77"  */
     ompi_attribute_fn_ptr_union_t copy_attr_fn; /**< Copy function for the 
-					     attribute */
+                                             attribute */
     ompi_attribute_fn_ptr_union_t delete_attr_fn; /**< Delete function for the
-					       attribute */
+                                               attribute */
     void *extra_state; /**< Extra state of the attribute */
     int key; /**< Keep a track of which key this item belongs to, so that
-		the key can be deleted when this object is destroyed */
+                the key can be deleted when this object is destroyed */
 
-    /** If non-null, call this function when the OBJ destructor for this
-        object is invoked */
-    ompi_attribute_keyval_destructor_fn_t *extra_destructor;
+    /** Extra state for bindings to hang data on.  If non-NULL, will be
+        freed by the C base when the keyval is destroyed. */
+    void *bindings_extra_state;
 };
 
 typedef struct ompi_attribute_keyval_t ompi_attribute_keyval_t;
@@ -222,6 +224,10 @@ int ompi_attr_finalize(void);
  * @param extra_state    Extra state to hang off/do some special things (IN)
  * @param flags          Flags for the key -- flags contain OMPI_KEYVAL_F77,
  *                       OMPI_KEYVAL_PREDEFINED
+ * @param bindings_extra_state Extra state that, if non-NULL, will
+ *                       automatically be free()'ed by the C base when
+ *                       the keyval is destroyed.
+ *
  * NOTE: I have taken the assumption that user cannot modify/delete
  * any predefined keys or the attributes attached. To accomplish this,
  * all MPI* calls will have OMPI_KEYVAL_PREDEFINED set as 0. MPI
@@ -240,11 +246,11 @@ int ompi_attr_finalize(void);
  *
  */
 
-int ompi_attr_create_keyval(ompi_attribute_type_t type, 
-                            ompi_attribute_fn_ptr_union_t copy_attr_fn, 
-                            ompi_attribute_fn_ptr_union_t delete_attr_fn,
-                            int *key, void *extra_state, int flags,
-                            ompi_attribute_keyval_destructor_fn_t *destructor);
+OMPI_DECLSPEC int ompi_attr_create_keyval(ompi_attribute_type_t type, 
+                                          ompi_attribute_fn_ptr_union_t copy_attr_fn, 
+                                          ompi_attribute_fn_ptr_union_t delete_attr_fn,
+                                          int *key, void *extra_state, int flags,
+                                          void *bindings_extra_state);
 
 /**
  * Free an attribute keyval
@@ -325,10 +331,10 @@ int ompi_attr_set_c(ompi_attribute_type_t type, void *object,
  * So yes, this is more code, but it's clearer and less error-prone
  * (read: better) this way.
  */
-int ompi_attr_set_fortran_mpi1(ompi_attribute_type_t type, void *object, 
-                               opal_hash_table_t **attr_hash,
-                               int key, MPI_Fint attribute, 
-                               bool predefined, bool need_lock);
+OMPI_DECLSPEC int ompi_attr_set_fortran_mpi1(ompi_attribute_type_t type, void *object, 
+                                             opal_hash_table_t **attr_hash,
+                                             int key, MPI_Fint attribute, 
+                                             bool predefined, bool need_lock);
 
 /**
  * Set an attribute on the comm/win/datatype in a form valid for
@@ -363,10 +369,10 @@ int ompi_attr_set_fortran_mpi1(ompi_attribute_type_t type, void *object,
  * So yes, this is more code, but it's clearer and less error-prone
  * (read: better) this way.
  */
-int ompi_attr_set_fortran_mpi2(ompi_attribute_type_t type, void *object, 
-                               opal_hash_table_t **attr_hash,
-                               int key, MPI_Aint attribute, 
-                               bool predefined, bool need_lock);
+OMPI_DECLSPEC int ompi_attr_set_fortran_mpi2(ompi_attribute_type_t type, void *object, 
+                                             opal_hash_table_t **attr_hash,
+                                             int key, MPI_Aint attribute, 
+                                             bool predefined, bool need_lock);
 
 /**
  * Get an attribute on the comm/win/datatype in a form valid for C.
@@ -417,8 +423,8 @@ int ompi_attr_get_c(opal_hash_table_t *attr_hash, int key,
  * (read: better) this way.
  */
 
-int ompi_attr_get_fortran_mpi1(opal_hash_table_t *attr_hash, int key, 
-                               MPI_Fint *attribute, int *flag);
+    OMPI_DECLSPEC int ompi_attr_get_fortran_mpi1(opal_hash_table_t *attr_hash, int key, 
+                                                 MPI_Fint *attribute, int *flag);
 
 
 /**
@@ -444,8 +450,8 @@ int ompi_attr_get_fortran_mpi1(opal_hash_table_t *attr_hash, int key,
  * (read: better) this way.
  */
 
-int ompi_attr_get_fortran_mpi2(opal_hash_table_t *attr_hash, int key, 
-                               MPI_Aint *attribute, int *flag);
+OMPI_DECLSPEC int ompi_attr_get_fortran_mpi2(opal_hash_table_t *attr_hash, int key, 
+                                             MPI_Aint *attribute, int *flag);
 
 
 /**
@@ -484,8 +490,8 @@ int ompi_attr_delete(ompi_attribute_type_t type, void *object,
  */
 
 int ompi_attr_copy_all(ompi_attribute_type_t type, void *old_object, 
-		      void *new_object, opal_hash_table_t *oldattr_hash,
-		      opal_hash_table_t *newkeyhash);
+                      void *new_object, opal_hash_table_t *oldattr_hash,
+                      opal_hash_table_t *newkeyhash);
 
 
 /** 
@@ -499,7 +505,7 @@ int ompi_attr_copy_all(ompi_attribute_type_t type, void *old_object,
  */
 
 int ompi_attr_delete_all(ompi_attribute_type_t type, void *object, 
-			opal_hash_table_t *attr_hash);
+                        opal_hash_table_t *attr_hash);
 
 
 /**
@@ -520,21 +526,7 @@ int ompi_attr_create_predefined(void);
  */
 int ompi_attr_free_predefined(void);
 
-struct orte_gpr_notify_data_t;
-/**
- * \internal
- * Callback function to get data from registry and create predefined attributes
- *
- * @returns Nothing
- */
-void ompi_attr_create_predefined_callback(
-	orte_gpr_notify_data_t *data,
-	void *cbdata);
-
-
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
 
+END_C_DECLS
 
 #endif /* OMPI_ATTRIBUTE_H */
