@@ -3,13 +3,15 @@
  * Copyright (c) 2004-2006 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2008 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2006 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2006 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -28,23 +30,15 @@
 #ifndef DATATYPE_H_HAS_BEEN_INCLUDED
 #define DATATYPE_H_HAS_BEEN_INCLUDED
 
-#include "ompi_config.h"
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#include <string.h>
-#include "ompi/constants.h"
 #include "opal/class/opal_object.h"
-#include "opal/class/opal_hash_table.h"
-#include "ompi/class/ompi_pointer_array.h"
+#include "opal/class/opal_pointer_array.h"
 #include "mpi.h"
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
-OMPI_DECLSPEC extern ompi_pointer_array_t *ompi_datatype_f_to_c_table;
+
+BEGIN_C_DECLS
+
+struct opal_hash_table_t;
+
+extern struct opal_pointer_array_t ompi_datatype_f_to_c_table;
 
 /* if there are more basic datatypes than the number of bytes in the int type
  * the bdt_used field of the data description struct should be changed to long.
@@ -59,7 +53,7 @@ OMPI_DECLSPEC extern ompi_pointer_array_t *ompi_datatype_f_to_c_table;
 #define DT_FLAG_USER_LB       0x0010  /**< has a user defined LB */
 #define DT_FLAG_USER_UB       0x0020  /**< has a user defined UB */
 #define DT_FLAG_PREDEFINED    0x0040  /**< cannot be removed: initial and predefined datatypes */
-#define DT_FLAG_NO_GAPS       0x0080  /**< we are inside a loop */
+#define DT_FLAG_NO_GAPS       0x0080  /**< no gaps around the datatype */
 #define DT_FLAG_DATA          0x0100  /**< data or control structure */
 #define DT_FLAG_ONE_SIDED     0x0200  /**< datatype can be used for one sided operations */
 #define DT_FLAG_UNAVAILABLE   0x0400  /**< datatypes unavailable on the build (OS or compiler dependant) */
@@ -84,9 +78,16 @@ OMPI_DECLSPEC extern ompi_pointer_array_t *ompi_datatype_f_to_c_table;
 
 typedef union dt_elem_desc dt_elem_desc_t;
 
+/**
+ * The number of supported entries in the data-type definition and the
+ * associated type.
+ */
+#define MAX_DT_COMPONENT_COUNT UINT_MAX
+typedef uint32_t opal_ddt_count_t;
+
 typedef struct dt_type_desc {
-    uint32_t          length;  /* the maximum number of elements in the description array */
-    uint32_t          used;    /* the number of used elements in the description array */
+    opal_ddt_count_t  length;  /* the maximum number of elements in the description array */
+    opal_ddt_count_t  used;    /* the number of used elements in the description array */
     dt_elem_desc_t*   desc;
 } dt_type_desc_t;
 
@@ -94,20 +95,20 @@ typedef struct dt_type_desc {
  */
 typedef struct ompi_datatype_t {
     opal_object_t      super;    /**< basic superclass */
-    unsigned long      size;     /**< total size in bytes of the memory used by the data if
+    size_t             size;     /**< total size in bytes of the memory used by the data if
                                   * the data is put on a contiguous buffer */
     uint32_t           align;    /**< data should be aligned to */
-    long               true_lb;
-    long               true_ub;  /**< the true ub of the data without user defined lb and ub */
-    long               lb;       /**< lower bound in memory */
-    long               ub;       /**< upper bound in memory */
+    ptrdiff_t          true_lb;
+    ptrdiff_t          true_ub;  /**< the true ub of the data without user defined lb and ub */
+    ptrdiff_t          lb;       /**< lower bound in memory */
+    ptrdiff_t          ub;       /**< upper bound in memory */
     uint16_t           flags;    /**< the flags */
     uint16_t           id;       /**< data id, normally the index in the data array. */
     uint32_t           nbElems;  /**< total number of elements inside the datatype */
     uint64_t           bdt_used; /**< which basic datatypes are used in the data description */
 
     /* Attribute fields */
-    opal_hash_table_t *d_keyhash;
+    struct opal_hash_table_t *d_keyhash;
     int32_t            d_f_to_c_index;
     char               name[MPI_MAX_OBJECT_NAME];
     dt_type_desc_t     desc;     /**< the data description */
@@ -121,82 +122,73 @@ typedef struct ompi_datatype_t {
     uint32_t           btypes[DT_MAX_PREDEFINED];
 } ompi_datatype_t;
 
-OBJ_CLASS_DECLARATION( ompi_datatype_t );
+OMPI_DECLSPEC OBJ_CLASS_DECLARATION( ompi_datatype_t );
 
 int ompi_ddt_register_params(void);
-int32_t ompi_ddt_init( void );
-int32_t ompi_ddt_finalize( void );
+OMPI_DECLSPEC int32_t ompi_ddt_init( void );
+OMPI_DECLSPEC int32_t ompi_ddt_finalize( void );
 ompi_datatype_t* ompi_ddt_create( int32_t expectedSize );
-int32_t ompi_ddt_commit( ompi_datatype_t** );
-int32_t ompi_ddt_destroy( ompi_datatype_t** );
+OMPI_DECLSPEC int32_t ompi_ddt_commit( ompi_datatype_t** );
+OMPI_DECLSPEC int32_t ompi_ddt_destroy( ompi_datatype_t** );
 static inline int32_t ompi_ddt_is_committed( const ompi_datatype_t* type )
 { return ((type->flags & DT_FLAG_COMMITED) == DT_FLAG_COMMITED); }
 static inline int32_t ompi_ddt_is_overlapped( const ompi_datatype_t* type )
 { return ((type->flags & DT_FLAG_OVERLAP) == DT_FLAG_OVERLAP); }
-static inline int32_t ompi_ddt_is_acceptable_for_one_sided( const ompi_datatype_t* type )
-{ return ((type->flags & DT_FLAG_ONE_SIDED) == DT_FLAG_ONE_SIDED); }
+static inline int32_t ompi_ddt_is_acceptable_for_one_sided( const ompi_datatype_t* type __opal_attribute_unused__ )
+{ return true; }
 static inline int32_t ompi_ddt_is_valid( const ompi_datatype_t* type )
 { return !((type->flags & DT_FLAG_UNAVAILABLE) == DT_FLAG_UNAVAILABLE); }
 static inline int32_t ompi_ddt_is_predefined( const ompi_datatype_t* type )
 { return (type->flags & DT_FLAG_PREDEFINED); }
 
-void ompi_ddt_dump( const ompi_datatype_t* pData );
+OMPI_DECLSPEC void ompi_ddt_dump( const ompi_datatype_t* pData );
 /* data creation functions */
 OMPI_DECLSPEC int32_t ompi_ddt_duplicate( const ompi_datatype_t* oldType, ompi_datatype_t** newType );
 OMPI_DECLSPEC int32_t ompi_ddt_create_contiguous( int count, const ompi_datatype_t* oldType, ompi_datatype_t** newType );
-OMPI_DECLSPEC int32_t ompi_ddt_create_vector( int count, int bLength, long stride,
+OMPI_DECLSPEC int32_t ompi_ddt_create_vector( int count, int bLength, int stride,
                                               const ompi_datatype_t* oldType, ompi_datatype_t** newType );
-OMPI_DECLSPEC int32_t ompi_ddt_create_hvector( int count, int bLength, long stride,
+OMPI_DECLSPEC int32_t ompi_ddt_create_hvector( int count, int bLength, MPI_Aint stride,
                                                const ompi_datatype_t* oldType, ompi_datatype_t** newType );
 OMPI_DECLSPEC int32_t ompi_ddt_create_indexed( int count, const int* pBlockLength, const int* pDisp,
                                                const ompi_datatype_t* oldType, ompi_datatype_t** newType );
-OMPI_DECLSPEC int32_t ompi_ddt_create_hindexed( int count, const int* pBlockLength, const long* pDisp,
+OMPI_DECLSPEC int32_t ompi_ddt_create_hindexed( int count, const int* pBlockLength, const MPI_Aint* pDisp,
                                                 const ompi_datatype_t* oldType, ompi_datatype_t** newType );
 OMPI_DECLSPEC int32_t ompi_ddt_create_indexed_block( int count, int bLength, const int* pDisp,
                                                      const ompi_datatype_t* oldType, ompi_datatype_t** newType );
-OMPI_DECLSPEC int32_t ompi_ddt_create_struct( int count, const int* pBlockLength, const long* pDisp,
+OMPI_DECLSPEC int32_t ompi_ddt_create_struct( int count, const int* pBlockLength, const MPI_Aint* pDisp,
                                               ompi_datatype_t* const* pTypes, ompi_datatype_t** newType );
-OMPI_DECLSPEC int32_t ompi_ddt_create_resized( const ompi_datatype_t* oldType, long lb, long extent, ompi_datatype_t** newType );
-OMPI_DECLSPEC int32_t ompi_ddt_create_subarray( int ndims, const int* pSizes,
-                                                const int* pSubSizes, const int* pStarts,
-                                                int order, const ompi_datatype_t* oldType, ompi_datatype_t** newType );
-OMPI_DECLSPEC int32_t ompi_ddt_create_darray( int size, int rank, int ndims,
-                                              const int* pGSizes, const int *pDistrib,
-                                              const int* pDArgs, const int* pPSizes,
-                                              int order, const ompi_datatype_t* oldType,
-                                              ompi_datatype_t** newType );
+OMPI_DECLSPEC int32_t ompi_ddt_create_resized( const ompi_datatype_t* oldType, MPI_Aint lb, MPI_Aint extent, ompi_datatype_t** newType );
 
 OMPI_DECLSPEC int32_t ompi_ddt_add( ompi_datatype_t* pdtBase, const ompi_datatype_t* pdtAdd, uint32_t count,
-                                    long disp, long extent );
+                                    ptrdiff_t disp, ptrdiff_t extent );
 
-static inline int32_t ompi_ddt_type_lb( const ompi_datatype_t* pData, long* disp )
+static inline int32_t ompi_ddt_type_lb( const ompi_datatype_t* pData, ptrdiff_t* disp )
 { *disp = pData->lb; return 0; }
-static inline int32_t ompi_ddt_type_ub( const ompi_datatype_t* pData, long* disp )
+static inline int32_t ompi_ddt_type_ub( const ompi_datatype_t* pData, ptrdiff_t* disp )
 { *disp = pData->ub; return 0; }
-static inline int32_t ompi_ddt_type_size ( const ompi_datatype_t* pData, int32_t *size )
+static inline int32_t ompi_ddt_type_size ( const ompi_datatype_t* pData, size_t *size )
 { *size = pData->size; return 0; }
-static inline int32_t ompi_ddt_type_extent( const ompi_datatype_t* pData, long* extent )
+static inline int32_t ompi_ddt_type_extent( const ompi_datatype_t* pData, ptrdiff_t* extent )
 { *extent = pData->ub - pData->lb; return 0; }
 
-static inline int32_t ompi_ddt_get_extent( const ompi_datatype_t* pData, long* lb, long* extent)
+static inline int32_t ompi_ddt_get_extent( const ompi_datatype_t* pData, ptrdiff_t* lb, ptrdiff_t* extent)
 { *lb = pData->lb; *extent = pData->ub - pData->lb; return 0; }
-static inline int32_t ompi_ddt_get_true_extent( const ompi_datatype_t* pData, long* true_lb, long* true_extent)
+static inline int32_t ompi_ddt_get_true_extent( const ompi_datatype_t* pData, ptrdiff_t* true_lb, ptrdiff_t* true_extent)
 { *true_lb = pData->true_lb; *true_extent = (pData->true_ub - pData->true_lb); return 0; }
-static inline int32_t ompi_ddt_get_size( const ompi_datatype_t* pData, unsigned long* size )
-{ *size = pData->size; return 0; }
 /*
  * This function return true (1) if the datatype representation depending on the count
  * is contiguous in the memory. And false (0) otherwise.
  */
 static inline int32_t ompi_ddt_is_contiguous_memory_layout( const ompi_datatype_t* datatype, int32_t count )
 {
-    if( !(datatype->flags & DT_FLAG_CONTIGUOUS) ) return 0;
-    if( (count == 1) || (datatype->flags & DT_FLAG_NO_GAPS) ) return 1;
-    assert( (long)datatype->size != (datatype->ub - datatype->lb) );
-    return 0;
+   if( !(datatype->flags & DT_FLAG_CONTIGUOUS) ) return 0;
+   if( (count == 1) || (datatype->flags & DT_FLAG_NO_GAPS) ) return 1;
+   assert( (ptrdiff_t)datatype->size != (datatype->ub - datatype->lb) );
+   return 0;
 }
 
-OMPI_DECLSPEC int32_t ompi_ddt_get_element_count( const ompi_datatype_t* pData, int32_t iSize );
+OMPI_DECLSPEC int32_t ompi_ddt_get_element_count( const ompi_datatype_t* pData, size_t iSize );
+OMPI_DECLSPEC int32_t ompi_ddt_set_element_count( const ompi_datatype_t* pData, uint32_t count, size_t* length );
 OMPI_DECLSPEC int32_t ompi_ddt_copy_content_same_ddt( const ompi_datatype_t* pData, int32_t count,
                                                       char* pDestBuf, char* pSrcBuf );
 
@@ -213,11 +205,11 @@ OMPI_DECLSPEC int32_t ompi_ddt_sndrcv( void *sbuf, int32_t scount, const ompi_da
  */
 OMPI_DECLSPEC int32_t ompi_ddt_get_args( const ompi_datatype_t* pData, int32_t which,
                                          int32_t * ci, int32_t * i,
-                                         int32_t * ca, long* a,
+                                         int32_t * ca, MPI_Aint* a,
                                          int32_t * cd, ompi_datatype_t** d, int32_t * type);
 OMPI_DECLSPEC int32_t ompi_ddt_set_args( ompi_datatype_t* pData,
                                          int32_t ci, int32_t ** i,
-                                         int32_t ca, long* a,
+                                         int32_t ca, MPI_Aint* a,
                                          int32_t cd, ompi_datatype_t** d,int32_t type);
 OMPI_DECLSPEC int32_t ompi_ddt_copy_args( const ompi_datatype_t* source_data,
                                           ompi_datatype_t* dest_data );
@@ -242,7 +234,5 @@ OMPI_DECLSPEC ompi_datatype_t*
 ompi_ddt_create_from_packed_description( void** packed_buffer,
                                          struct ompi_proc_t* remote_processor );
 
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
+END_C_DECLS
 #endif  /* DATATYPE_H_HAS_BEEN_INCLUDED */
