@@ -1,14 +1,17 @@
+/* -*- Mode: C; c-basic-offset:4 ; -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2007 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -23,15 +26,15 @@
 #include "ompi_config.h"
 
 #include "mpi.h"
+#include "opal/prefetch.h"
 #include "opal/class/opal_object.h"
-#include "ompi/class/ompi_pointer_array.h"
+#include "opal/class/opal_pointer_array.h"
 #include "ompi/runtime/mpiruntime.h"
 #include "ompi/errhandler/errhandler_predefined.h"
 #include "ompi/errhandler/errcode-internal.h"
 
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
+BEGIN_C_DECLS
+
 /*
  * These must correspond to the fortran handle indices
  */
@@ -53,6 +56,16 @@ typedef void (ompi_errhandler_fortran_handler_fn_t)(MPI_Fint *,
  */
 typedef void (ompi_errhandler_generic_handler_fn_t)(void *, int *, ...);
 
+/**
+ * Enum to denote what language the error handler was created from
+ */
+enum ompi_errhandler_lang_t {
+    OMPI_ERRHANDLER_LANG_C,
+    OMPI_ERRHANDLER_LANG_CXX,
+    OMPI_ERRHANDLER_LANG_FORTRAN
+};
+typedef enum ompi_errhandler_lang_t ompi_errhandler_lang_t;
+
 
 /**
  * Enum used to describe what kind MPI object an error handler is used for
@@ -66,6 +79,18 @@ enum ompi_errhandler_type_t {
 typedef enum ompi_errhandler_type_t ompi_errhandler_type_t;
 
 
+/*
+ * Need to forward declare this for use in ompi_errhandle_cxx_dispatch_fn_t.
+ */
+struct ompi_errhandler_t;
+
+/**
+ * C++ invocation function signature
+ */
+typedef void (ompi_errhandler_cxx_dispatch_fn_t)(struct ompi_errhandler_t *errhandler,
+                                                 void *handle, int *err_code,
+                                                 const char *message);
+
 /**
  * Back-end type for MPI_Errorhandler.
  */
@@ -77,15 +102,26 @@ struct ompi_errhandler_t {
     
     ompi_errhandler_type_t eh_mpi_object_type;
 
-    /* Flags about the error handler */
-    bool eh_fortran_function;
+    /* What language was the error handler created in */
+    ompi_errhandler_lang_t eh_lang;
 
-    /* Function pointers */
+    /* Function pointers.  Note that we *have* to have all 4 types
+       (vs., for example, a union) because the predefined errhandlers
+       can be invoked on any MPI object type, so we need callbacks for
+       all of three. */
     MPI_Comm_errhandler_fn *eh_comm_fn;
-    MPI_File_errhandler_fn *eh_file_fn;
+    ompi_file_errhandler_fn *eh_file_fn;
     MPI_Win_errhandler_fn *eh_win_fn;
     ompi_errhandler_fortran_handler_fn_t *eh_fort_fn;
 
+    /* Have separate callback for C++ errhandlers.  This pointer is
+       initialized to NULL and will be set explicitly by the C++
+       bindings for Create_errhandler.  This function is invoked
+       when eh_lang==OMPI_ERRHANDLER_LANG_CXX so that the user's
+       callback function can be invoked with the right language
+       semantics. */
+    ompi_errhandler_cxx_dispatch_fn_t *eh_cxx_dispatch_fn;
+
     /* index in Fortran <-> C translation array */
     int eh_f_to_c_index;
 };
@@ -107,10 +143,23 @@ OMPI_DECLSPEC extern ompi_errhandler_t ompi_mpi_errors_are_fatal;
  */
 OMPI_DECLSPEC extern ompi_errhandler_t ompi_mpi_errors_return;
 
+/**
+ * Global variable for MPI::ERRORS_THROW_EXCEPTIONS.  Will abort if
+ * MPI_INIT wasn't called as MPI::INIT
+ */
+OMPI_DECLSPEC extern ompi_errhandler_t ompi_mpi_errors_throw_exceptions;
+
 /**
  * Table for Fortran <-> C errhandler handle conversion
  */
-OMPI_DECLSPEC extern ompi_pointer_array_t *ompi_errhandler_f_to_c_table;
+OMPI_DECLSPEC extern opal_pointer_array_t ompi_errhandler_f_to_c_table;
+
+
+/**
+ * Forward declaration so that we don't have to include
+ * request/request.h here.
+ */
+struct ompi_request_t;
 
 
 /**
@@ -122,7 +171,7 @@ OMPI_DECLSPEC extern ompi_pointer_array_t *ompi_errhandler_f_to_c_table;
  * we're before MPI_Init() or after MPI_Finalize()).
  */
 #define OMPI_ERR_INIT_FINALIZE(name) \
-  if (!ompi_mpi_initialized || ompi_mpi_finalized) { \
+  if( OPAL_UNLIKELY(!ompi_mpi_initialized || ompi_mpi_finalized) ) { \
     ompi_mpi_errors_are_fatal_comm_handler(NULL, NULL, name); \
   }
 
@@ -162,8 +211,9 @@ OMPI_DECLSPEC extern ompi_pointer_array_t *ompi_errhandler_f_to_c_table;
  * OMPI_SUCCESS.
  */
 #define OMPI_ERRHANDLER_CHECK(rc, mpi_object, err_code, message) \
-  if (rc != OMPI_SUCCESS) { \
+  if( OPAL_UNLIKELY(rc != OMPI_SUCCESS) ) { \
     int __mpi_err_code = (err_code < 0 ? (ompi_errcode_get_mpi_code(err_code)) : err_code); \
+    OPAL_CR_EXIT_LIBRARY() \
     ompi_errhandler_invoke((mpi_object)->error_handler, \
 			   (mpi_object), \
                            (int) (mpi_object)->errhandler_type, \
@@ -188,6 +238,7 @@ OMPI_DECLSPEC extern ompi_pointer_array_t *ompi_errhandler_f_to_c_table;
  * MPI_SUCCESS.
  */
 #define OMPI_ERRHANDLER_RETURN(rc, mpi_object, err_code, message) \
+  OPAL_CR_EXIT_LIBRARY() \
   if (rc != OMPI_SUCCESS) { \
     int __mpi_err_code = (err_code < 0 ? (ompi_errcode_get_mpi_code(err_code)) : err_code); \
     ompi_errhandler_invoke((mpi_object)->error_handler, \
@@ -252,10 +303,20 @@ OMPI_DECLSPEC extern ompi_pointer_array_t *ompi_errhandler_f_to_c_table;
    * If this function returns, it returns the err_code.  Note that it
    * may not return (e.g., for MPI_ERRORS_ARE_FATAL).
    */
-  int ompi_errhandler_invoke(ompi_errhandler_t *errhandler, void *mpi_object, 
-			     int type, int err_code, const char *message);
+  OMPI_DECLSPEC int ompi_errhandler_invoke(ompi_errhandler_t *errhandler, void *mpi_object, 
+                                           int type, int err_code, const char *message);
 
 
+  /**
+   * Invoke an MPI exception on the first request found in the array
+   * that has a non-MPI_SUCCESS value for MPI_ERROR in its status.  It
+   * is safe to invoke this function if none of the requests have an
+   * outstanding error; MPI_SUCCESS will be returned.
+   */
+  int ompi_errhandler_request_invoke(int count, 
+                                     struct ompi_request_t **requests,
+                                     const char *message);
+
   /**
    * Create a ompi_errhandler_t
    *
@@ -277,14 +338,10 @@ OMPI_DECLSPEC extern ompi_pointer_array_t *ompi_errhandler_f_to_c_table;
    * arbitrarily.  Note that (void*) is not sufficient because at
    * least theoretically, a sizeof(void*) may not necessarily be the
    * same as sizeof(void(*)).
-   *
-   * NOTE: It *always* sets the "fortran" flag to false.  Fortran
-   * wrappers for MPI_*_CREATE_ERRHANDLER are expected to reset this
-   * flag to true manually.
    */
-  ompi_errhandler_t *ompi_errhandler_create(ompi_errhandler_type_t object_type,
-					    ompi_errhandler_generic_handler_fn_t *func);
-
+  OMPI_DECLSPEC ompi_errhandler_t *ompi_errhandler_create(ompi_errhandler_type_t object_type,
+					    ompi_errhandler_generic_handler_fn_t *func,
+                                            ompi_errhandler_lang_t language);
 
 /**
  * Check to see if an errhandler is intrinsic.
@@ -305,8 +362,7 @@ static inline bool ompi_errhandler_is_intrinsic(ompi_errhandler_t *errhandler)
 
     return false;
 }
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
+
+END_C_DECLS
 
 #endif /* OMPI_ERRHANDLER_H */
