@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2007 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -16,13 +16,17 @@
  * $HEADER$
  */
 
+#include "ompi_config.h"
 #include "ompi/mca/btl/base/base.h" 
 #include "ompi/mca/bml/base/base.h" 
 
-int mca_bml_base_close( void ) {
+int mca_bml_base_close( void )
+{
     int rc; 
+
     if(OMPI_SUCCESS != (rc = mca_btl_base_close()))
         return rc;
+
     return OMPI_SUCCESS; 
 }
 
