@@ -1,9 +1,10 @@
 /*
- * Copyright (c) 2006      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2008 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2008      Mellanox Technologies. All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  *
  * @file
@@ -25,20 +26,23 @@ typedef struct ompi_btl_openib_ini_values_t {
 
     uint32_t use_eager_rdma;
     bool use_eager_rdma_set;
+
+    char *receive_queues;
+
+    int32_t max_inline_data;
+    bool max_inline_data_set;
 } ompi_btl_openib_ini_values_t;
 
 
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
+BEGIN_C_DECLS
 
     /**
-     * Read in the INI files containing HCA params
+     * Read in the INI files containing device params
      */
     int ompi_btl_openib_ini_init(void);
 
     /**
-     * Query the read-in params for a given HCA
+     * Query the read-in params for a given device
      */
     int ompi_btl_openib_ini_query(uint32_t vendor_id,
                                   uint32_t vendor_part_id,
@@ -49,7 +53,12 @@ extern "C" {
      */
     int ompi_btl_openib_ini_finalize(void);
 
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
+    /**
+     * string to int convertors with dec/hex autodetection
+     */
+    int ompi_btl_openib_ini_intify(char *string);
+    int ompi_btl_openib_ini_intify_list(char *str, uint32_t **values, int *len);
+
+END_C_DECLS
+
 #endif
