@@ -1,14 +1,15 @@
-/*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ /*
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2008 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2008      UT-Battelle, LLC. All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -22,9 +23,9 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <errno.h>
+#include <inttypes.h>
 
 #include "ompi/constants.h"
-#include "opal/util/output.h"
 #include "ompi/mca/pml/pml.h"
 #include "ompi/mca/btl/btl.h"
 #include "ompi/datatype/convertor.h"
@@ -45,8 +46,9 @@ mca_btl_portals_module_t mca_btl_portals_module = {
         0,   /* max size of first frag */
         0,   /* min send size */
         0,   /* max send size */
-        0,   /* min rdma size */
-        0,   /* max rdma size */
+        0,   /* btl_rdma_pipeline_send_length */
+        0,   /* btl_rdma_pipeline_frag_size */
+        0,   /* btl_min_rdma_pipeline_size */
         0,   /* exclusivity - higher than sm, lower than self */
         0,   /* latency */
         0,   /* bandwidth */
@@ -54,7 +56,7 @@ mca_btl_portals_module_t mca_btl_portals_module = {
 
         mca_btl_portals_add_procs,
         mca_btl_portals_del_procs,
-        mca_btl_portals_register,
+        NULL,
         mca_btl_portals_finalize,
 
         mca_btl_portals_alloc,
@@ -62,11 +64,13 @@ mca_btl_portals_module_t mca_btl_portals_module = {
         mca_btl_portals_prepare_src,
         mca_btl_portals_prepare_dst,
         mca_btl_portals_send,
+        NULL, /* mca_btl_portals_sendi, */
         mca_btl_portals_put,
         mca_btl_portals_get,
         mca_btl_base_dump,
         NULL, /* mpool */
-        NULL /* register error */
+        NULL, /* register error */
+        NULL
     },
 };
 
@@ -84,15 +88,16 @@ mca_btl_portals_add_procs(struct mca_btl_base_module_t* btl_base,
     size_t i;
     unsigned long distance;
     bool need_activate = false;
+    bool accel;
 
     assert(&mca_btl_portals_module == (mca_btl_portals_module_t*) btl_base);
     opal_output_verbose(50, mca_btl_portals_component.portals_output,
-                        "Adding %d procs (%d)", nprocs,
-                        mca_btl_portals_module.portals_num_procs);
+                        "Adding %d procs (%d)", (int) nprocs,
+                        (int) mca_btl_portals_module.portals_num_procs);
 
     /* if we havne't already, get our network handle */
     if (mca_btl_portals_module.portals_ni_h == PTL_INVALID_HANDLE) {
-        ret = ompi_common_portals_ni_initialize(&mca_btl_portals_module.portals_ni_h);
+        ret = ompi_common_portals_ni_initialize(&mca_btl_portals_module.portals_ni_h, &accel);
         if (OMPI_SUCCESS != ret) return ret;
     }
 
@@ -117,15 +122,18 @@ mca_btl_portals_add_procs(struct mca_btl_base_module_t* btl_base,
         if (NULL == peers[i]) return OMPI_ERROR;
         *((mca_btl_base_endpoint_t*) peers[i]) = portals_procs[i];
 
-        /* make sure we can reach the process - this is supposed to be
-           a cheap-ish operation */
-        ret = PtlNIDist(mca_btl_portals_module.portals_ni_h,
-                        portals_procs[i],
-                        &distance);
-        if (ret != PTL_OK) {
-            opal_output_verbose(10, mca_btl_portals_component.portals_output,
-                                "Could not find distance to process %d", i);
-            continue;
+        /* accelerated doesn't support PtlNIDist() */
+        if (accel == false) {
+            /* make sure we can reach the process - this is supposed to be
+               a cheap-ish operation */
+            ret = PtlNIDist(mca_btl_portals_module.portals_ni_h,
+                            portals_procs[i],
+                            &distance);
+            if (ret != PTL_OK) {
+                opal_output_verbose(10, mca_btl_portals_component.portals_output,
+                                    "Could not find distance to process %d", (int) i);
+                continue;
+            }
         }
 
         OPAL_THREAD_ADD32(&mca_btl_portals_module.portals_num_procs, 1);
@@ -185,8 +193,8 @@ mca_btl_portals_del_procs(struct mca_btl_base_module_t *btl_base,
 
     assert(&mca_btl_portals_module == (mca_btl_portals_module_t*) btl_base);
     opal_output_verbose(50, mca_btl_portals_component.portals_output,
-                        "Removing %d procs (%d)", nprocs,
-                        mca_btl_portals_module.portals_num_procs);
+                        "Removing %d procs (%d)", (int) nprocs,
+                        (int) mca_btl_portals_module.portals_num_procs);
 
     for (i = 0 ; i < nprocs ; ++i) {
         free(peers[i]);
@@ -218,36 +226,22 @@ mca_btl_portals_del_procs(struct mca_btl_base_module_t *btl_base,
 }
 
 
-int
-mca_btl_portals_register(struct mca_btl_base_module_t* btl_base, 
-                          mca_btl_base_tag_t tag, 
-                          mca_btl_base_module_recv_cb_fn_t cbfunc, 
-                          void* cbdata)
-{
-    assert(&mca_btl_portals_module == (mca_btl_portals_module_t*) btl_base);
-
-    mca_btl_portals_module.portals_reg[tag].cbfunc = cbfunc; 
-    mca_btl_portals_module.portals_reg[tag].cbdata = cbdata; 
-
-    return OMPI_SUCCESS;
-}
-
-
 mca_btl_base_descriptor_t*
 mca_btl_portals_alloc(struct mca_btl_base_module_t* btl_base,
-                       size_t size)
+                      struct mca_btl_base_endpoint_t* endpoint,
+                      uint8_t order,
+                      size_t size,
+                      uint32_t flags)
 {
     int rc;
     mca_btl_portals_frag_t* frag;
 
     assert(&mca_btl_portals_module == (mca_btl_portals_module_t*) btl_base);
-
+    
     if (size <= mca_btl_portals_module.super.btl_eager_limit) { 
         OMPI_BTL_PORTALS_FRAG_ALLOC_EAGER(&mca_btl_portals_module, frag, rc); 
         if (OMPI_SUCCESS != rc) return NULL;
-        frag->segments[0].seg_len = 
-            size <= mca_btl_portals_module.super.btl_eager_limit ? 
-            size : mca_btl_portals_module.super.btl_eager_limit ; 
+        frag->segments[0].seg_len = size;
     } else { 
         OMPI_BTL_PORTALS_FRAG_ALLOC_MAX(&mca_btl_portals_module, frag, rc); 
         if (OMPI_SUCCESS != rc) return NULL;
@@ -257,7 +251,8 @@ mca_btl_portals_alloc(struct mca_btl_base_module_t* btl_base,
     }
     
     frag->base.des_src_cnt = 1;
-    frag->base.des_flags = 0; 
+    frag->base.des_flags = flags | MCA_BTL_DES_SEND_ALWAYS_CALLBACK; 
+    frag->base.order = MCA_BTL_NO_ORDER;
 
     return &frag->base;
 }
@@ -302,8 +297,10 @@ mca_btl_portals_prepare_src(struct mca_btl_base_module_t* btl_base,
                             struct mca_btl_base_endpoint_t* peer,
                             mca_mpool_base_registration_t* registration, 
                             struct ompi_convertor_t* convertor,
+                            uint8_t order,
                             size_t reserve,
-                            size_t* size)
+                            size_t* size,
+                            uint32_t flags)
 {
     mca_btl_portals_frag_t* frag;
     size_t max_data = *size;
@@ -315,7 +312,7 @@ mca_btl_portals_prepare_src(struct mca_btl_base_module_t* btl_base,
 
     if (0 != reserve || 0 != ompi_convertor_need_buffers(convertor)) {
         frag = (mca_btl_portals_frag_t*) 
-            mca_btl_portals_alloc(btl_base, max_data + reserve);
+            mca_btl_portals_alloc(btl_base, peer, MCA_BTL_NO_ORDER, max_data + reserve, flags);
         if (NULL == frag)  {
             return NULL;
         }
@@ -366,8 +363,10 @@ mca_btl_portals_prepare_src(struct mca_btl_base_module_t* btl_base,
 
         /* either a put or get.  figure out which later */
         OPAL_OUTPUT_VERBOSE((90, mca_btl_portals_component.portals_output,
-                             "rdma src posted for frag 0x%x, callback 0x%x, bits %lld",
-                             frag, frag->base.des_cbfunc, frag->segments[0].seg_key.key64));
+                             "rdma src posted for frag 0x%lx, callback 0x%lx, bits %"PRIu64", flags say %d" ,
+                             (unsigned long) frag, 
+                             (unsigned long) frag->base.des_cbfunc,
+                             frag->segments[0].seg_key.key64, flags));
 
         /* create a match entry */
         ret = PtlMEAttach(mca_btl_portals_module.portals_ni_h,
@@ -412,8 +411,8 @@ mca_btl_portals_prepare_src(struct mca_btl_base_module_t* btl_base,
     frag->base.des_src = frag->segments;
     frag->base.des_dst = NULL;
     frag->base.des_dst_cnt = 0;
-    frag->base.des_flags = 0;
-
+    frag->base.des_flags = flags | MCA_BTL_DES_SEND_ALWAYS_CALLBACK; 
+    frag->base.order = MCA_BTL_NO_ORDER;
     return &frag->base;
 }
 
@@ -423,14 +422,15 @@ mca_btl_portals_prepare_dst(struct mca_btl_base_module_t* btl_base,
                             struct mca_btl_base_endpoint_t* peer,
                             mca_mpool_base_registration_t* registration, 
                             struct ompi_convertor_t* convertor,
+                            uint8_t order,
                             size_t reserve,
-                            size_t* size)
+                            size_t* size,
+                            uint32_t flags)
 {
     mca_btl_portals_frag_t* frag;
     ptl_md_t md;
     ptl_handle_me_t me_h;
     int ret;
-    ptrdiff_t lb;
 
     assert(&mca_btl_portals_module == (mca_btl_portals_module_t*) btl_base);
 
@@ -447,20 +447,22 @@ mca_btl_portals_prepare_dst(struct mca_btl_base_module_t* btl_base,
         return NULL;
     }
 
-    ompi_ddt_type_lb(convertor->pDesc, &lb);
     frag->segments[0].seg_len = *size;
-    frag->segments[0].seg_addr.pval = convertor->pBaseBuf + lb + convertor->bConverted;
+    ompi_convertor_get_current_pointer( convertor, (void**)&(frag->segments[0].seg_addr.pval) );
     frag->segments[0].seg_key.key64 = 
         OPAL_THREAD_ADD64(&(mca_btl_portals_module.portals_rdma_key), 1);
     frag->base.des_src = NULL;
     frag->base.des_src_cnt = 0;
     frag->base.des_dst = frag->segments;
     frag->base.des_dst_cnt = 1;
-    frag->base.des_flags = 0;
-
+    frag->base.des_flags = flags | MCA_BTL_DES_SEND_ALWAYS_CALLBACK; 
+    
     OPAL_OUTPUT_VERBOSE((90, mca_btl_portals_component.portals_output,
-                         "rdma dest posted for frag 0x%x, callback 0x%x, bits %lld",
-                         frag, frag->base.des_cbfunc, frag->segments[0].seg_key.key64));
+                         "rdma dest posted for frag 0x%lx, callback 0x%lx, bits %" PRIu64 " flags %d",
+                         (unsigned long) frag,
+                         (unsigned long) frag->base.des_cbfunc,
+                         frag->segments[0].seg_key.key64,
+                         flags));
 
     /* create a match entry */
     ret = PtlMEAttach(mca_btl_portals_module.portals_ni_h,
@@ -500,7 +502,7 @@ mca_btl_portals_prepare_dst(struct mca_btl_base_module_t* btl_base,
         OMPI_BTL_PORTALS_FRAG_RETURN_USER(&mca_btl_portals_module.super, frag);
         return NULL;
     }
-
+    frag->base.order = MCA_BTL_NO_ORDER;
     return &frag->base;
 }
 
@@ -511,12 +513,22 @@ mca_btl_portals_finalize(struct mca_btl_base_module_t *btl_base)
     int ret;
 
     assert(&mca_btl_portals_module == (mca_btl_portals_module_t*) btl_base);
+    OPAL_OUTPUT_VERBOSE((90, mca_btl_portals_component.portals_output,
+                            "in mca_btl_portals_finalize"));
 
+    /* sanity check */
+    assert(mca_btl_portals_module.portals_outstanding_ops  >= 0);
+    
     /* finalize all communication */
     while (mca_btl_portals_module.portals_outstanding_ops > 0) {
+        OPAL_OUTPUT_VERBOSE((90, mca_btl_portals_component.portals_output,
+                            "portals_outstanding_ops: %d", 
+                            mca_btl_portals_module.portals_outstanding_ops));
+        
         mca_btl_portals_component_progress();
     }
-
+    
+    
     if (mca_btl_portals_module.portals_num_procs != 0) {
         int i;
 
@@ -526,14 +538,14 @@ mca_btl_portals_finalize(struct mca_btl_base_module_t *btl_base)
         for (i = 0 ; i < OMPI_BTL_PORTALS_EQ_SIZE ; ++i) {
             int ptl_ret = PtlEQFree(mca_btl_portals_module.portals_eq_handles[i]);
             if (PTL_OK != ptl_ret) {
-#if OMPI_PORTALS_CRAYXT3
+#if (OMPI_PORTALS_CRAYXT3 || OMPI_PORTALS_CRAYXT3_MODEX)
                 if (i != OMPI_BTL_PORTALS_EQ_SEND && PTL_EQ_IN_USE != ptl_ret) {
                     /* The PML isn't great about cleaning up after itself.
                        Ignore related errors. */
 #endif
                     opal_output(mca_btl_portals_component.portals_output,
                             "Error freeing EQ %d: %d", i, ptl_ret);
-#if OMPI_PORTALS_CRAYXT3
+#if (OMPI_PORTALS_CRAYXT3 || OMPI_PORTALS_CRAYXT3_MODEX)
                 }
 #endif
             }
@@ -546,7 +558,7 @@ mca_btl_portals_finalize(struct mca_btl_base_module_t *btl_base)
     OBJ_DESTRUCT(&mca_btl_portals_module.portals_frag_eager);
     OBJ_DESTRUCT(&mca_btl_portals_module.portals_frag_max);
     OBJ_DESTRUCT(&mca_btl_portals_module.portals_frag_user);
-
+    
     ompi_common_portals_ni_finalize();
     ompi_common_portals_finalize();
 
