@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -39,12 +39,11 @@
 extern "C" {
 #endif
 
-
 /**
  * Shared Memory (SELF) BTL module.
  */
 struct mca_btl_self_component_t {
-    mca_btl_base_component_1_0_0_t super;  /**< base BTL component */
+    mca_btl_base_component_2_0_0_t super;  /**< base BTL component */
     int free_list_num;                     /**< initial size of free lists */
     int free_list_max;                     /**< maximum size of free lists */
     int free_list_inc;                     /**< number of elements to alloc when growing free lists */
@@ -52,20 +51,19 @@ struct mca_btl_self_component_t {
     ompi_free_list_t self_frags_eager;     /**< free list of self first */
     ompi_free_list_t self_frags_send;      /**< free list of self second */
     ompi_free_list_t self_frags_rdma;      /**< free list of self second */
-    mca_btl_base_recv_reg_t self_reg[256];
 };
 typedef struct mca_btl_self_component_t mca_btl_self_component_t;
-extern mca_btl_self_component_t mca_btl_self_component;
+OMPI_MODULE_DECLSPEC extern mca_btl_self_component_t mca_btl_self_component;
 
 /**
  * Register shared memory module parameters with the MCA framework
  */
-extern int mca_btl_self_component_open(void);
+int mca_btl_self_component_open(void);
 
 /**
  * Any final cleanup before being unloaded.
  */
-extern int mca_btl_self_component_close(void);
+int mca_btl_self_component_close(void);
 
 /**
  * SELF module initialization.
@@ -75,7 +73,7 @@ extern int mca_btl_self_component_close(void);
  * @param enable_mpi_threads (IN)         Flag indicating whether BTL must support multilple simultaneous invocations from different threads
  *
  */
-extern mca_btl_base_module_t** mca_btl_self_component_init(
+mca_btl_base_module_t** mca_btl_self_component_init(
     int *num_btls, 
     bool enable_progress_threads,
     bool enable_mpi_threads
@@ -91,7 +89,7 @@ extern mca_btl_base_module_t mca_btl_self;
  * @return     OMPI_SUCCESS or error status on failure.
  */
 
-extern int mca_btl_self_finalize(
+int mca_btl_self_finalize(
     struct mca_btl_base_module_t* btl
 );
 
@@ -110,7 +108,7 @@ extern int mca_btl_self_finalize(
  * 
  */
 
-extern int mca_btl_self_add_procs(
+int mca_btl_self_add_procs(
     struct mca_btl_base_module_t* btl,
     size_t nprocs,
     struct ompi_proc_t **procs,
@@ -118,30 +116,7 @@ extern int mca_btl_self_add_procs(
     struct ompi_bitmap_t* reachability
 );
 
-                                                                                                               
-/**
- * PML->BTL notification of change in the process list.
- * PML->BTL Notification that a receive fragment has been matched.
- * Called for message that is send from process with the virtual
- * address of the shared memory segment being the same as that of
- * the receiver.
- * 
- * @param btl (IN)
- * @param proc (IN)  
- * @param peer (OUT)
- * @return     OMPI_SUCCESS or error status on failure.
- * 
- */
 
-extern int mca_btl_self_add_procs_same_base_addr(
-    struct mca_btl_base_module_t* btl,
-    size_t nprocs,
-    struct ompi_proc_t **procs,
-    struct mca_btl_base_endpoint_t** peers,
-    ompi_bitmap_t* reachability
-);
-
-                                                                                                               
 /**
  * PML->BTL notification of change in the process list.
  *
@@ -151,7 +126,7 @@ extern int mca_btl_self_add_procs_same_base_addr(
  * @return             Status indicating if cleanup was successful
  *
  */
-extern int mca_btl_self_del_procs(
+int mca_btl_self_del_procs(
     struct mca_btl_base_module_t* btl,
     size_t nprocs,
     struct ompi_proc_t **procs,
@@ -159,35 +134,18 @@ extern int mca_btl_self_del_procs(
 );
 
 
-/**
- * Register a callback function that is called on receipt
- * of a fragment.
- *
- * @param btl (IN)     BTL module
- * @return             Status indicating if cleanup was successful
- *
- * When the process list changes, the PML notifies the BTL of the
- * change, to provide the opportunity to cleanup or release any
- * resources associated with the peer.
- */
-
-extern int mca_btl_self_register(
-    struct mca_btl_base_module_t* btl,
-    mca_btl_base_tag_t tag,
-    mca_btl_base_module_recv_cb_fn_t cbfunc,
-    void* cbdata
-);
-                                                                                                                   
-
 /**
  * Allocate a segment.
  *
  * @param btl (IN)      BTL module
  * @param size (IN)     Request segment size.
  */
-extern mca_btl_base_descriptor_t* mca_btl_self_alloc(
+mca_btl_base_descriptor_t* mca_btl_self_alloc(
     struct mca_btl_base_module_t* btl,
-    size_t size
+    struct mca_btl_base_endpoint_t* endpoint,
+    uint8_t order,
+    size_t size,
+    uint32_t flags
 );
 
 /**
@@ -196,7 +154,7 @@ extern mca_btl_base_descriptor_t* mca_btl_self_alloc(
  * @param btl (IN)      BTL module
  * @param segment (IN)  Allocated segment.
  */
-extern int mca_btl_self_free(
+int mca_btl_self_free(
     struct mca_btl_base_module_t* btl,
     mca_btl_base_descriptor_t* segment
 );
@@ -212,8 +170,10 @@ struct mca_btl_base_descriptor_t* mca_btl_self_prepare_src(
     struct mca_btl_base_endpoint_t* endpoint,
     struct mca_mpool_base_registration_t* registration,
     struct ompi_convertor_t* convertor,
+    uint8_t order,
     size_t reserve,
-    size_t* size
+    size_t* size,
+    uint32_t flags
 );
                                                                                                             
 /**
@@ -227,8 +187,10 @@ struct mca_btl_base_descriptor_t* mca_btl_self_prepare_dst(
     struct mca_btl_base_endpoint_t* endpoint,
     struct mca_mpool_base_registration_t* registration,
     struct ompi_convertor_t* convertor,
+    uint8_t order,
     size_t reserve,
-    size_t* size
+    size_t* size,
+    uint32_t flags
 );
                                                                                                             
 /**
@@ -237,7 +199,7 @@ struct mca_btl_base_descriptor_t* mca_btl_self_prepare_dst(
  * @param btl (IN)      BTL module
  * @param peer (IN)     BTL peer addressing
  */
-extern int mca_btl_self_send(
+int mca_btl_self_send(
     struct mca_btl_base_module_t* btl,
     struct mca_btl_base_endpoint_t* endpoint,
     struct mca_btl_base_descriptor_t* descriptor,
@@ -251,16 +213,22 @@ extern int mca_btl_self_send(
  * @param peer (IN)     BTL peer addressing
  */
                                                                                                             
-extern int mca_btl_self_rdma(
+int mca_btl_self_rdma(
     struct mca_btl_base_module_t* btl,
     struct mca_btl_base_endpoint_t* endpoint,
     struct mca_btl_base_descriptor_t* descriptor
 );
-                                                                                                            
 
+/**
+ * Fault Tolerance Event Notification Function
+ * @param state Checkpoint Stae
+ * @return OMPI_SUCCESS or failure status
+ */
+int mca_btl_self_ft_event(int state);
 
 #if defined(c_plusplus) || defined(__cplusplus)
 }
 #endif
+
 #endif
 
