@@ -1,18 +1,19 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2008 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2006-2007 Voltaire. All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 #include "ompi_config.h"
@@ -41,33 +42,35 @@
 #include "opal/event/event.h"
 #include "opal/util/if.h"
 #include "opal/util/argv.h"
-#include "opal/util/output.h"
-#include "orte/util/sys_info.h"
+#include "orte/util/show_help.h"
 #include "orte/util/proc_info.h"
+#include "orte/runtime/orte_globals.h"
+
 #include "ompi/mca/pml/pml.h"
 #include "opal/mca/base/mca_base_param.h"
-#include "ompi/mca/pml/base/pml_base_module_exchange.h"
+#include "ompi/runtime/ompi_module_exchange.h"
 #include "ompi/mca/mpool/base/base.h"
 #include "ompi/mca/common/sm/common_sm_mmap.h"
 #include "ompi/mca/btl/base/btl_base_error.h"
+
+#if OPAL_ENABLE_FT    == 1
+#include "opal/runtime/opal_cr.h"
+#endif
+
 #include "btl_sm.h"
 #include "btl_sm_frag.h"
 #include "btl_sm_fifo.h"
 
-
-
 /*
- * Shared Memory (SM) component instance. 
+ * Shared Memory (SM) component instance.
  */
-
 mca_btl_sm_component_t mca_btl_sm_component = {
     {  /* super is being filled in */
         /* First, the mca_base_component_t struct containing meta information
           about the component itself */
         {
-            /* Indicate that we are a pml v1.0.0 component (which also implies a
-               specific MCA version) */
-            MCA_BTL_BASE_VERSION_1_0_1,
+            MCA_BTL_BASE_VERSION_2_0_0,
+
             "sm", /* MCA component name */
             OMPI_MAJOR_VERSION,  /* MCA component major version */
             OMPI_MINOR_VERSION,  /* MCA component minor version */
@@ -75,14 +78,12 @@ mca_btl_sm_component_t mca_btl_sm_component = {
             mca_btl_sm_component_open,  /* component open */
             mca_btl_sm_component_close  /* component close */
         },
-
-        /* Next the MCA v1.0.0 component meta data */
         {
-            /* Whether the component is checkpointable or not */
-            false
+            /* The component is checkpoint ready */
+            MCA_BASE_METADATA_PARAM_CHECKPOINT
         },
 
-        mca_btl_sm_component_init,  
+        mca_btl_sm_component_init,
         mca_btl_sm_component_progress,
     }  /* end super */
 };
@@ -93,7 +94,7 @@ mca_btl_sm_component_t mca_btl_sm_component = {
  */
 
 static inline char* mca_btl_sm_param_register_string(
-    const char* param_name, 
+    const char* param_name,
     const char* default_value)
 {
     char *param_value;
@@ -101,9 +102,9 @@ static inline char* mca_btl_sm_param_register_string(
     mca_base_param_lookup_string(id, &param_value);
     return param_value;
 }
-                                                                                                                            
+
 static inline int mca_btl_sm_param_register_int(
-    const char* param_name, 
+    const char* param_name,
     int default_value)
 {
     int id = mca_base_param_register_int("btl","sm",param_name,NULL,default_value);
@@ -127,24 +128,18 @@ int mca_btl_sm_component_open(void)
         mca_btl_sm_param_register_int("free_list_max", -1);
     mca_btl_sm_component.sm_free_list_inc =
         mca_btl_sm_param_register_int("free_list_inc", 64);
-    mca_btl_sm_component.sm_exclusivity =
-        mca_btl_sm_param_register_int("exclusivity", MCA_BTL_EXCLUSIVITY_HIGH-1);
-    mca_btl_sm_component.sm_latency =
-        mca_btl_sm_param_register_int("latency", 100);
     mca_btl_sm_component.sm_max_procs =
         mca_btl_sm_param_register_int("max_procs", -1);
     mca_btl_sm_component.sm_extra_procs =
         mca_btl_sm_param_register_int("sm_extra_procs", -1);
     mca_btl_sm_component.sm_mpool_name =
         mca_btl_sm_param_register_string("mpool", "sm");
-    mca_btl_sm_component.eager_limit =
-        mca_btl_sm_param_register_int("eager_limit", 4*1024);
-    mca_btl_sm_component.max_frag_size =
-        mca_btl_sm_param_register_int("max_frag_size", 32*1024);
     mca_btl_sm_component.size_of_cb_queue =
         mca_btl_sm_param_register_int("size_of_cb_queue", 128);
     mca_btl_sm_component.cb_lazy_free_freq =
         mca_btl_sm_param_register_int("cb_lazy_free_freq", 120);
+    mca_btl_sm_component.cb_max_num =
+        mca_btl_sm_param_register_int("cb_max_num", -1);
     /* make sure that queue size and lazy free frequency are consistent -
      * want to make sure that slots are freed at a rate they can be
      * reused, w/o allocating extra new circular buffer fifo arrays */
@@ -163,10 +158,27 @@ int mca_btl_sm_component_open(void)
     mca_btl_sm_component.sm_extra_procs =
         mca_btl_sm_param_register_int("sm_extra_procs", 2);
 
+    mca_btl_sm.super.btl_exclusivity = MCA_BTL_EXCLUSIVITY_HIGH-1;
+    mca_btl_sm.super.btl_eager_limit = 4*1024;
+    mca_btl_sm.super.btl_rndv_eager_limit = 4*1024;
+    mca_btl_sm.super.btl_max_send_size = 32*1024;
+    mca_btl_sm.super.btl_rdma_pipeline_send_length = 32*1024;
+    mca_btl_sm.super.btl_rdma_pipeline_frag_size = 32*1024;
+    mca_btl_sm.super.btl_min_rdma_pipeline_size = 0;
+    mca_btl_sm.super.btl_flags = MCA_BTL_FLAGS_SEND;
+    mca_btl_sm.super.btl_bandwidth = 900;
+    mca_btl_sm.super.btl_latency = 100;
+
+    mca_btl_base_param_register(&mca_btl_sm_component.super.btl_version,
+            &mca_btl_sm.super);
+    mca_btl_sm_component.max_frag_size = mca_btl_sm.super.btl_max_send_size;
+    mca_btl_sm_component.eager_limit = mca_btl_sm.super.btl_eager_limit;
+
     /* initialize objects */
     OBJ_CONSTRUCT(&mca_btl_sm_component.sm_lock, opal_mutex_t);
-    OBJ_CONSTRUCT(&mca_btl_sm_component.sm_frags1, ompi_free_list_t);
-    OBJ_CONSTRUCT(&mca_btl_sm_component.sm_frags2, ompi_free_list_t);
+    OBJ_CONSTRUCT(&mca_btl_sm_component.sm_frags_eager, ompi_free_list_t);
+    OBJ_CONSTRUCT(&mca_btl_sm_component.sm_frags_max, ompi_free_list_t);
+    OBJ_CONSTRUCT(&mca_btl_sm_component.pending_send_fl, opal_free_list_t);
     return OMPI_SUCCESS;
 }
 
@@ -185,8 +197,8 @@ int mca_btl_sm_component_close(void)
      * directly into the mmapped file, they will auto-magically dissapear
      * when the file get unmapped.
      */
-    /*OBJ_DESTRUCT(&mca_btl_sm_component.sm_frags1);*/
-    /*OBJ_DESTRUCT(&mca_btl_sm_component.sm_frags2);*/
+    /*OBJ_DESTRUCT(&mca_btl_sm_component.sm_frags_eager);*/
+    /*OBJ_DESTRUCT(&mca_btl_sm_component.sm_frags_max);*/
 
     /* unmap the shared memory control structure */
     if(mca_btl_sm_component.mmap_file != NULL) {
@@ -198,12 +210,22 @@ int mca_btl_sm_component_close(void)
                     errno);
             goto CLEANUP;
         }
-    
+
         /* unlink file, so that it will be deleted when all references
          * to it are gone - no error checking, since we want all procs
          * to call this, so that in an abnormal termination scenario,
          * this file will still get cleaned up */
+#if OPAL_ENABLE_FT    == 1
+        /* Only unlink the file if we are *not* restarting
+         * If we are restarting the file will be unlinked at a later time.
+         */
+        if(OPAL_CR_STATUS_RESTART_PRE  != opal_cr_checkpointing_state &&
+           OPAL_CR_STATUS_RESTART_POST != opal_cr_checkpointing_state ) {
+            unlink(mca_btl_sm_component.mmap_file->map_path);
+        }
+#else
         unlink(mca_btl_sm_component.mmap_file->map_path);
+#endif
         OBJ_RELEASE(mca_btl_sm_component.mmap_file);
     }
 
@@ -212,7 +234,7 @@ int mca_btl_sm_component_close(void)
     if(mca_btl_sm_component.sm_fifo_fd > 0) {
         /* write a done message down the pipe */
         unsigned char cmd = DONE;
-        if( write(mca_btl_sm_component.sm_fifo_fd,&cmd,sizeof(cmd)) != 
+        if( write(mca_btl_sm_component.sm_fifo_fd,&cmd,sizeof(cmd)) !=
                 sizeof(cmd)){
             opal_output(0, "mca_btl_sm_component_close: write fifo failed: errno=%d\n",
                     errno);
@@ -230,19 +252,17 @@ CLEANUP:
     return return_value;
 }
 
-
 /*
  *  SM component initialization
  */
 mca_btl_base_module_t** mca_btl_sm_component_init(
-    int *num_ptls, 
+    int *num_btls,
     bool enable_progress_threads,
     bool enable_mpi_threads)
 {
-    mca_btl_base_module_t **ptls = NULL;
-    int i;
+    mca_btl_base_module_t **btls = NULL;
 
-    *num_ptls = 0;
+    *num_btls = 0;
 
     /* lookup/create shared memory pool only when used */
     mca_btl_sm_component.sm_mpool = NULL;
@@ -250,9 +270,9 @@ mca_btl_base_module_t** mca_btl_sm_component_init(
 
 #if OMPI_ENABLE_PROGRESS_THREADS == 1
     /* create a named pipe to receive events  */
-    sprintf( mca_btl_sm_component.sm_fifo_path, 
+    sprintf( mca_btl_sm_component.sm_fifo_path,
              "%s"OPAL_PATH_SEP"sm_fifo.%lu", orte_process_info.job_session_dir,
-             (unsigned long)orte_process_info.my_name->vpid );
+             (unsigned long)ORTE_PROC_MY_NAME->vpid );
     if(mkfifo(mca_btl_sm_component.sm_fifo_path, 0660) < 0) {
         opal_output(0, "mca_btl_sm_component_init: mkfifo failed with errno=%d\n",errno);
         return NULL;
@@ -269,47 +289,25 @@ mca_btl_base_module_t** mca_btl_sm_component_init(
     opal_thread_start(&mca_btl_sm_component.sm_fifo_thread);
 #endif
 
-    /* allocate the Shared Memory PTL */
-    *num_ptls = 2;
-    ptls = (mca_btl_base_module_t**)malloc((*num_ptls)*sizeof(mca_btl_base_module_t*));
-    if (NULL == ptls) {
+    /* allocate the Shared Memory BTL */
+    *num_btls = 1;
+    btls = (mca_btl_base_module_t**)malloc(sizeof(mca_btl_base_module_t*));
+    if (NULL == btls) {
         return NULL;
     }
 
-    /* get pointer to the ptls */
-    ptls[0] = (mca_btl_base_module_t *)(&(mca_btl_sm[0]));
-    ptls[1] = (mca_btl_base_module_t *)(&(mca_btl_sm[1]));
-
-    /* set scheduling parameters */
-    for( i=0 ; i < 2 ; i++ ) {
-        mca_btl_sm[i].super.btl_eager_limit=mca_btl_sm_component.eager_limit;
-        mca_btl_sm[i].super.btl_min_send_size=mca_btl_sm_component.max_frag_size;
-        mca_btl_sm[i].super.btl_max_send_size=mca_btl_sm_component.max_frag_size;
-        mca_btl_sm[i].super.btl_min_rdma_size=mca_btl_sm_component.max_frag_size;
-        mca_btl_sm[i].super.btl_max_rdma_size=mca_btl_sm_component.max_frag_size;
-        /* The order in which the SM modules are initialized is important as only
-         * the first one (the one using the mca_btl_sm_add_procs_same_base_addr)
-         * will setup all the memory for the internal structures (sm_proc_connect).
-         * Therefore, the order in which the two SM module will be after the 
-         * selection is important. We have to make sure they get sorted in the
-         * correct order. The simplest way is to force the exclusivity of the
-         * second module to something lower than the exclusivity of the first one.
-         */
-        mca_btl_sm[i].super.btl_exclusivity = mca_btl_sm_component.sm_exclusivity - i;
-        mca_btl_sm[i].super.btl_latency     = mca_btl_sm_component.sm_latency; /* lowest latency */
-        mca_btl_sm[i].super.btl_bandwidth   = 900; /* not really used now since exclusivity is set to the highest value */
-    }
+    /* get pointer to the btls */
+    btls[0] = (mca_btl_base_module_t *)(&(mca_btl_sm));
 
     /* initialize some PTL data */
     /* start with no SM procs */
     mca_btl_sm_component.num_smp_procs = 0;
-    mca_btl_sm_component.my_smp_rank   = 0xFFFFFFFF;  /* not defined */
+    mca_btl_sm_component.my_smp_rank   = -1;  /* not defined */
 
-    /* set flag indicating ptl not inited */
-    mca_btl_sm[0].btl_inited=false;
-    mca_btl_sm[1].btl_inited=false;
+    /* set flag indicating btl not inited */
+    mca_btl_sm.btl_inited=false;
 
-    return ptls;
+    return btls;
 }
 
 
@@ -329,195 +327,148 @@ void mca_btl_sm_component_event_thread(opal_object_t* thread)
         if( DONE == cmd ){
             /* return when done message received */
             return;
-        } 
+        }
         mca_btl_sm_component_progress();
     }
 }
 #endif
 
+void
+btl_sm_add_pending(struct mca_btl_base_endpoint_t *ep, void *data, bool resend)
+{
+    int rc;
+    btl_sm_pending_send_item_t *si;
+    opal_free_list_item_t *i;
+    OPAL_FREE_LIST_GET(&mca_btl_sm_component.pending_send_fl, i, rc);
+
+    /* don't handle error for now */
+    assert(i != NULL && rc == OMPI_SUCCESS);
+
+    si = (btl_sm_pending_send_item_t*)i;
+    si->data = data;
+
+    /* if data was on pending send list then prepend it to the list to
+     * minimize reordering */
+    if(resend)
+        opal_list_prepend(&ep->pending_sends, (opal_list_item_t*)si);
+    else
+        opal_list_append(&ep->pending_sends, (opal_list_item_t*)si);
+}
+
+static int process_pending_send(struct mca_btl_base_endpoint_t *ep) 
+{ 
+    btl_sm_pending_send_item_t *si; 
+    int rc; 
+
+    si = (btl_sm_pending_send_item_t*)opal_list_remove_first(&ep->pending_sends); 
+    if(NULL == si) return OMPI_ERROR; 
+    
+    OPAL_FREE_LIST_RETURN(&mca_btl_sm_component.pending_send_fl, (opal_list_item_t*)si);
+
+    MCA_BTL_SM_FIFO_WRITE(ep, ep->my_smp_rank, ep->peer_smp_rank, si->data,
+                          true, rc);
+
+    return rc; 
+} 
 
 int mca_btl_sm_component_progress(void)
 {
     /* local variables */
-    unsigned int peer_smp_rank ;
     mca_btl_sm_frag_t *frag;
+    mca_btl_sm_frag_t Frag;
     ompi_fifo_t *fifo = NULL;
-    int my_smp_rank=mca_btl_sm_component.my_smp_rank;
-    int proc;
-    int rc = 0, btl = 0;
-
-    /* send progress is made by the PML */
-
-    /* 
-     * receive progress 
-     */
+    mca_btl_sm_hdr_t *hdr;
+    int my_smp_rank = mca_btl_sm_component.my_smp_rank;
+    int peer_smp_rank, rc = 0;
 
     /* poll each fifo */
+    for(peer_smp_rank = 0; peer_smp_rank < mca_btl_sm_component.num_smp_procs;
+        peer_smp_rank++) {
+        if(peer_smp_rank == mca_btl_sm_component.my_smp_rank)
+            continue;
 
-    /* loop over fifo's - procs with same base shared memory 
-     * virtual address as this process */
-    for( proc=0 ; proc < mca_btl_sm_component.num_smp_procs_same_base_addr
-            ; proc++ ) 
-    {
-        peer_smp_rank= mca_btl_sm_component.list_smp_procs_same_base_addr[proc];
-        fifo=&(mca_btl_sm_component.fifo[my_smp_rank][peer_smp_rank]);
-
+        fifo = &(mca_btl_sm_component.fifo[my_smp_rank][peer_smp_rank]);
+      recheck_peer:
         /* if fifo is not yet setup - continue - not data has been sent*/
         if(OMPI_CB_FREE == fifo->tail){
             continue;
         }
 
         /* aquire thread lock */
-        if( opal_using_threads() ) {
+        if(opal_using_threads()) {
             opal_atomic_lock(fifo->tail_lock);
         }
 
-        /* get pointer - pass in offset to change queue pointer
-         * addressing from that of the sender.  In this case, we know
-         * that we have the same base address as the sender, so no
-         * translation is necessary when accessing the fifo.  Hence,
-         * we use the _same_base_addr varient. */
-        frag = (mca_btl_sm_frag_t *)ompi_fifo_read_from_tail(fifo);
+        hdr = (mca_btl_sm_hdr_t *)ompi_fifo_read_from_tail(fifo);
 
         /* release thread lock */
-        if( opal_using_threads() ) {
+        if(opal_using_threads()) {
             opal_atomic_unlock(fifo->tail_lock);
         }
 
-        if( OMPI_CB_FREE == frag ) {
+        if(OMPI_CB_FREE == hdr) {
             continue;
         }
 
+        rc++;
         /* dispatch fragment by type */
-        switch(frag->type) {
-            case MCA_BTL_SM_FRAG_ACK:
-            {
-                /* completion callback */
-                frag->base.des_cbfunc(&mca_btl_sm[0].super, frag->endpoint, &frag->base, frag->rc);
-                break;
-            }
+        switch(((uintptr_t)hdr) & MCA_BTL_SM_FRAG_TYPE_MASK) {
             case MCA_BTL_SM_FRAG_SEND:
             {
+                mca_btl_active_message_callback_t* reg;
+                /* change the address from address relative to the shared
+                 * memory address, to a true virtual address */
+                hdr = (mca_btl_sm_hdr_t *)((char *)hdr +
+                        mca_btl_sm_component.sm_offset[peer_smp_rank]);
                 /* recv upcall */
-                mca_btl_sm_recv_reg_t* reg = mca_btl_sm[0].sm_reg + frag->tag;
-                reg->cbfunc(&mca_btl_sm[0].super,frag->tag,&frag->base,reg->cbdata);
-                frag->type = MCA_BTL_SM_FRAG_ACK;
-                MCA_BTL_SM_FIFO_WRITE( mca_btl_sm_component.sm_peers[peer_smp_rank],
-                                       my_smp_rank, peer_smp_rank, frag, rc );
-                if(OMPI_SUCCESS != rc)
-                    goto err;
+                reg = mca_btl_base_active_message_trigger + hdr->tag;
+                Frag.segment.seg_addr.pval = ((char*)hdr) +
+                    sizeof(mca_btl_sm_hdr_t);
+                Frag.segment.seg_len = hdr->len;
+                Frag.base.des_dst_cnt = 1;
+                Frag.base.des_dst = &(Frag.segment);
+                reg->cbfunc(&mca_btl_sm.super, hdr->tag, &(Frag.base),
+                            reg->cbdata);
+                MCA_BTL_SM_FIFO_WRITE(
+                        mca_btl_sm_component.sm_peers[peer_smp_rank],
+                        my_smp_rank, peer_smp_rank, hdr->frag, false, rc);
+		goto recheck_peer;
                 break;
             }
-            default:
-            {
-                /* unknown */
-                frag->rc = OMPI_ERROR;
-                frag->type = MCA_BTL_SM_FRAG_ACK;
-                MCA_BTL_SM_FIFO_WRITE( mca_btl_sm_component.sm_peers[peer_smp_rank],
-                                       my_smp_rank, peer_smp_rank, frag, rc );
-                if(OMPI_SUCCESS != rc)
-                    goto err;
-                break;
-            }
-        }
-        rc++;
-    }  /* end peer_local_smp_rank loop */
-
-
-    btl = 1;
-    /* loop over fifo's - procs with different base shared memory 
-     * virtual address as this process */
-    for( proc=0 ; proc < mca_btl_sm_component.num_smp_procs_different_base_addr
-            ; proc++ ) 
-    {
-        peer_smp_rank= mca_btl_sm_component.list_smp_procs_different_base_addr[proc];
-        fifo=&(mca_btl_sm_component.fifo[my_smp_rank][peer_smp_rank]);
-
-        /* if fifo is not yet setup - continue - not data has been sent*/
-        if(OMPI_CB_FREE == fifo->tail){
-            continue;
-        }
-
-        /* aquire thread lock */
-        if( opal_using_threads() ) {
-            opal_atomic_lock(fifo->tail_lock);
-        }
-
-        /* get pointer - pass in offset to change queue pointer
-         * addressing from that of the sender.  In this case, we do
-         * *not* have the same base address as the sender, so we must
-         * translate every access into the fifo to be relevant to our
-         * memory space.  Hence, we do *not* use the _same_base_addr
-         * variant. */
-        frag = (mca_btl_sm_frag_t *)ompi_fifo_read_from_tail( fifo );
-        
-        if( OMPI_CB_FREE == frag ) {
-            /* release thread lock */
-            if( opal_using_threads() ) {
-                opal_atomic_unlock(fifo->tail_lock);
-            }
-            continue;
-        }
-
-        /* release thread lock */
-        if( opal_using_threads() ) {
-            opal_atomic_unlock(fifo->tail_lock);
-        }
-
-        /* change the address from address relative to the shared
-         * memory address, to a true virtual address */
-        frag = (mca_btl_sm_frag_t *)( (char *)frag +
-                mca_btl_sm_component.sm_offset[peer_smp_rank]);
-
-        /* dispatch fragment by type */
-        switch(frag->type) {
             case MCA_BTL_SM_FRAG_ACK:
             {
-                /* completion callback */
-                frag->base.des_src = 
-                    ( mca_btl_base_segment_t* )((unsigned char*)frag->base.des_dst + mca_btl_sm_component.sm_offset[peer_smp_rank]);
-                frag->base.des_src->seg_addr.pval =
-                    ((unsigned char*)frag->base.des_src->seg_addr.pval +
-                     mca_btl_sm_component.sm_offset[peer_smp_rank]);
-                frag->base.des_dst = frag->base.des_src;
-                frag->base.des_cbfunc(&mca_btl_sm[1].super, frag->endpoint, &frag->base, frag->rc);
-                break;
-            }
-            case MCA_BTL_SM_FRAG_SEND:
-            {
-                /* recv upcall */
-                mca_btl_sm_recv_reg_t* reg = mca_btl_sm[1].sm_reg + frag->tag;
-                frag->base.des_dst = (mca_btl_base_segment_t*)
-                    ((unsigned char*)frag->base.des_src + mca_btl_sm_component.sm_offset[peer_smp_rank]);
-                frag->base.des_dst->seg_addr.pval = 
-                    ((unsigned char*)frag->base.des_dst->seg_addr.pval +
-                    mca_btl_sm_component.sm_offset[peer_smp_rank]);
-                frag->base.des_src = frag->base.des_dst;
-                reg->cbfunc(&mca_btl_sm[1].super,frag->tag,&frag->base,reg->cbdata);
-                frag->type = MCA_BTL_SM_FRAG_ACK;
-                MCA_BTL_SM_FIFO_WRITE( mca_btl_sm_component.sm_peers[peer_smp_rank],
-                                       my_smp_rank, peer_smp_rank, frag, rc );
-                if(OMPI_SUCCESS != rc)
-                    goto err;
-                break;
+                int status = (uintptr_t)hdr & MCA_BTL_SM_FRAG_STATUS_MASK;
+                int btl_ownership;
+                struct mca_btl_base_endpoint_t* endpoint;
+
+                frag = (mca_btl_sm_frag_t *)((char*)((uintptr_t)hdr &
+                            (~(MCA_BTL_SM_FRAG_TYPE_MASK |
+                            MCA_BTL_SM_FRAG_STATUS_MASK))));
+                endpoint = frag->endpoint;
+                btl_ownership = (frag->base.des_flags & MCA_BTL_DES_FLAGS_BTL_OWNERSHIP);
+                if( MCA_BTL_DES_SEND_ALWAYS_CALLBACK & frag->base.des_flags ) {
+                    /* completion callback */
+                    frag->base.des_cbfunc(&mca_btl_sm.super, frag->endpoint,
+                                          &frag->base, status?OMPI_ERROR:OMPI_SUCCESS);
+                }
+                if( btl_ownership ) {
+                    MCA_BTL_SM_FRAG_RETURN(frag);
+                }
+                if(opal_list_get_size(&endpoint->pending_sends)) {
+                   if( OMPI_ERR_RESOURCE_BUSY == process_pending_send(endpoint) )
+                       break;
+                }
+                goto recheck_peer;
             }
             default:
-            {
                 /* unknown */
-                frag->rc = OMPI_ERROR;
-                frag->type = MCA_BTL_SM_FRAG_ACK;
-                MCA_BTL_SM_FIFO_WRITE( mca_btl_sm_component.sm_peers[peer_smp_rank],
-                                       my_smp_rank, peer_smp_rank, frag, rc );
-                if(OMPI_SUCCESS != rc)
-                    goto err;
+                hdr = (mca_btl_sm_hdr_t*)((uintptr_t)hdr->frag |
+                        MCA_BTL_SM_FRAG_STATUS_MASK);
+                MCA_BTL_SM_FIFO_WRITE(
+                        mca_btl_sm_component.sm_peers[peer_smp_rank],
+                        my_smp_rank, peer_smp_rank, hdr, false, rc);
                 break;
-            }
         }
-        rc++;
-    }  /* end peer_local_smp_rank loop */
-    return rc;
-err:
-    BTL_ERROR(("SM faild to send message due to shortage of shared memory.\n"));
-    mca_btl_sm[btl].error_cb(&mca_btl_sm[btl].super, MCA_BTL_ERROR_FLAGS_FATAL);
+    }
     return rc;
 }
