@@ -2,23 +2,20 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2008 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2007-2008 Sun Microsystems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
  * 
  * $HEADER$
  *
- * In windows, many of the socket functions return an EWOULDBLOCK
- * instead of \ things like EAGAIN, EINPROGRESS, etc. It has been
- * verified that this will \ not conflict with other error codes that
- * are returned by these functions \ under UNIX/Linux environments 
  */
 
 #include "ompi_config.h"
@@ -53,6 +50,9 @@
 
 #include "ompi/types.h"
 #include "ompi/mca/btl/base/btl_base_error.h"
+#include "opal/util/if.h"
+#include "opal/util/net.h"
+
 #include "btl_tcp.h"
 #include "btl_tcp_endpoint.h" 
 #include "btl_tcp_proc.h"
@@ -64,7 +64,6 @@
  * Initialize state of the endpoint instance.
  *
  */
-
 static void mca_btl_tcp_endpoint_construct(mca_btl_tcp_endpoint_t* endpoint)
 {
     endpoint->endpoint_btl = NULL;
@@ -92,8 +91,6 @@ static void mca_btl_tcp_endpoint_construct(mca_btl_tcp_endpoint_t* endpoint)
  * Destroy a endpoint
  *
  */
-
-
 static void mca_btl_tcp_endpoint_destruct(mca_btl_tcp_endpoint_t* endpoint)
 {
     mca_btl_tcp_proc_remove(endpoint->endpoint_proc, endpoint);
@@ -132,23 +129,49 @@ static void mca_btl_tcp_endpoint_dump(mca_btl_base_endpoint_t* btl_endpoint, con
     char src[64];
     char dst[64];
     int sndbuf,rcvbuf,nodelay,flags;
+#if OPAL_WANT_IPV6
+    struct sockaddr_storage inaddr;
+#else
     struct sockaddr_in inaddr;
+#endif
     opal_socklen_t obtlen;
-    opal_socklen_t addrlen = sizeof(struct sockaddr_in);
+    opal_socklen_t addrlen = sizeof(inaddr);
 
     getsockname(btl_endpoint->endpoint_sd, (struct sockaddr*)&inaddr, &addrlen);
+#if OPAL_WANT_IPV6
+    {
+        char *address;
+        address = (char *) opal_net_get_hostname((struct sockaddr*) &inaddr);
+        if (NULL != address) {
+            sprintf(src, "%s", address);
+        }
+    }
+#else
     sprintf(src, "%s", inet_ntoa(inaddr.sin_addr));
+#endif
     getpeername(btl_endpoint->endpoint_sd, (struct sockaddr*)&inaddr, &addrlen);
+#if OPAL_WANT_IPV6
+    {
+        char *address;
+        address = (char *) opal_net_get_hostname ((struct sockaddr*) &inaddr);
+        if (NULL != address) {
+            sprintf(dst, "%s", address);
+        }
+    }
+#else
     sprintf(dst, "%s", inet_ntoa(inaddr.sin_addr));
+#endif
 
     if((flags = fcntl(btl_endpoint->endpoint_sd, F_GETFL, 0)) < 0) {
-        BTL_ERROR(("fcntl(F_GETFL) failed with errno=%d", opal_socket_errno));
+        BTL_ERROR(("fcntl(F_GETFL) failed: %s (%d)", 
+                   strerror(opal_socket_errno), opal_socket_errno));
     }
 
 #if defined(SO_SNDBUF)
     obtlen = sizeof(sndbuf);
     if(getsockopt(btl_endpoint->endpoint_sd, SOL_SOCKET, SO_SNDBUF, (char *)&sndbuf, &obtlen) < 0) {
-        BTL_ERROR(("SO_SNDBUF option: errno %d", opal_socket_errno));
+        BTL_ERROR(("SO_SNDBUF option: %s (%d)",
+                   strerror(opal_socket_errno), opal_socket_errno));
     }
 #else
     sndbuf = -1;
@@ -156,7 +179,8 @@ static void mca_btl_tcp_endpoint_dump(mca_btl_base_endpoint_t* btl_endpoint, con
 #if defined(SO_RCVBUF)
     obtlen = sizeof(rcvbuf);
     if(getsockopt(btl_endpoint->endpoint_sd, SOL_SOCKET, SO_RCVBUF, (char *)&rcvbuf, &obtlen) < 0) {
-        BTL_ERROR(("SO_RCVBUF option: errno %d", opal_socket_errno));
+        BTL_ERROR(("SO_RCVBUF option: %s (%d)", 
+                   strerror(opal_socket_errno), opal_socket_errno));
     }
 #else
     rcvbuf = -1;
@@ -164,13 +188,14 @@ static void mca_btl_tcp_endpoint_dump(mca_btl_base_endpoint_t* btl_endpoint, con
 #if defined(TCP_NODELAY)
     obtlen = sizeof(nodelay);
     if(getsockopt(btl_endpoint->endpoint_sd, IPPROTO_TCP, TCP_NODELAY, (char *)&nodelay, &obtlen) < 0) {
-        BTL_ERROR(("TCP_NODELAY option: errno %d", opal_socket_errno));
+        BTL_ERROR(("TCP_NODELAY option: %s (%d)", 
+                   strerror(opal_socket_errno), opal_socket_errno));
     }
 #else
     nodelay = 0;
 #endif
 
-    BTL_DEBUG(("%s: %s - %s nodelay %d sndbuf %d rcvbuf %d flags %08x", 
+    BTL_VERBOSE(("%s: %s - %s nodelay %d sndbuf %d rcvbuf %d flags %08x", 
         msg, src, dst, nodelay, sndbuf, rcvbuf, flags));
 }
 #endif
@@ -179,7 +204,7 @@ static void mca_btl_tcp_endpoint_dump(mca_btl_base_endpoint_t* btl_endpoint, con
  * Initialize events to be used by the endpoint instance for TCP select/poll callbacks.
  */
 
-static inline void mca_btl_tcp_endpoint_event_init(mca_btl_base_endpoint_t* btl_endpoint, int sd)
+static inline void mca_btl_tcp_endpoint_event_init(mca_btl_base_endpoint_t* btl_endpoint)
 {
 #if MCA_BTL_TCP_ENDPOINT_CACHE
     btl_endpoint->endpoint_cache     = (char*)malloc(mca_btl_tcp_component.tcp_endpoint_cache);
@@ -187,15 +212,21 @@ static inline void mca_btl_tcp_endpoint_event_init(mca_btl_base_endpoint_t* btl_
 #endif  /* MCA_BTL_TCP_ENDPOINT_CACHE */
 
     opal_event_set( &btl_endpoint->endpoint_recv_event, 
-		    btl_endpoint->endpoint_sd, 
-		    OPAL_EV_READ|OPAL_EV_PERSIST, 
-		    mca_btl_tcp_endpoint_recv_handler,
-		    btl_endpoint );
+                    btl_endpoint->endpoint_sd, 
+                    OPAL_EV_READ|OPAL_EV_PERSIST, 
+                    mca_btl_tcp_endpoint_recv_handler,
+                    btl_endpoint );
+    /**
+     * The send event should be non persistent until the endpoint is
+     * completely connected. This means, when the event is created it
+     * will be fired only once, and when the endpoint is marked as 
+     * CONNECTED the event should be recreated with the correct flags.
+     */
     opal_event_set( &btl_endpoint->endpoint_send_event, 
-		    btl_endpoint->endpoint_sd, 
-		    OPAL_EV_WRITE|OPAL_EV_PERSIST, 
-		    mca_btl_tcp_endpoint_send_handler,
-		    btl_endpoint);
+                    btl_endpoint->endpoint_sd, 
+                    OPAL_EV_WRITE, 
+                    mca_btl_tcp_endpoint_send_handler,
+                    btl_endpoint);
 }
 
 
@@ -207,12 +238,14 @@ static inline void mca_btl_tcp_endpoint_event_init(mca_btl_base_endpoint_t* btl_
 int mca_btl_tcp_endpoint_send(mca_btl_base_endpoint_t* btl_endpoint, mca_btl_tcp_frag_t* frag)
 {
     int rc = OMPI_SUCCESS;
+
     OPAL_THREAD_LOCK(&btl_endpoint->endpoint_send_lock);
     switch(btl_endpoint->endpoint_state) {
     case MCA_BTL_TCP_CONNECTING:
     case MCA_BTL_TCP_CONNECT_ACK:
     case MCA_BTL_TCP_CLOSED:
         opal_list_append(&btl_endpoint->endpoint_frags, (opal_list_item_t*)frag);
+        frag->base.des_flags |= MCA_BTL_DES_SEND_ALWAYS_CALLBACK;
         if(btl_endpoint->endpoint_state == MCA_BTL_TCP_CLOSED)
             rc = mca_btl_tcp_endpoint_start_connect(btl_endpoint);
         break;
@@ -223,20 +256,26 @@ int mca_btl_tcp_endpoint_send(mca_btl_base_endpoint_t* btl_endpoint, mca_btl_tcp
         if (btl_endpoint->endpoint_send_frag == NULL) {
             if(frag->base.des_flags & MCA_BTL_DES_FLAGS_PRIORITY &&
                mca_btl_tcp_frag_send(frag, btl_endpoint->endpoint_sd)) {
+                int btl_ownership = (frag->base.des_flags & MCA_BTL_DES_FLAGS_BTL_OWNERSHIP);
+
                 OPAL_THREAD_UNLOCK(&btl_endpoint->endpoint_send_lock);
-                frag->base.des_cbfunc(&frag->btl->super, frag->endpoint, &frag->base, frag->rc);
-                return OMPI_SUCCESS;
+                if( frag->base.des_flags & MCA_BTL_DES_SEND_ALWAYS_CALLBACK ) {
+                    frag->base.des_cbfunc(&frag->btl->super, frag->endpoint, &frag->base, frag->rc);
+                }
+                if( btl_ownership ) {
+                    MCA_BTL_TCP_FRAG_RETURN(frag);
+                }
+                return 1;
             } else {
                 btl_endpoint->endpoint_send_frag = frag;
                 opal_event_add(&btl_endpoint->endpoint_send_event, 0);
+                frag->base.des_flags |= MCA_BTL_DES_SEND_ALWAYS_CALLBACK;
             }
         } else {
+            frag->base.des_flags |= MCA_BTL_DES_SEND_ALWAYS_CALLBACK;
             opal_list_append(&btl_endpoint->endpoint_frags, (opal_list_item_t*)frag);
         }
         break;
-    case MCA_BTL_TCP_SHUTDOWN:
-        rc = OMPI_ERROR;
-        break;
     }
     OPAL_THREAD_UNLOCK(&btl_endpoint->endpoint_send_lock);
     return rc;
@@ -255,7 +294,8 @@ static int mca_btl_tcp_endpoint_send_blocking(mca_btl_base_endpoint_t* btl_endpo
         int retval = send(btl_endpoint->endpoint_sd, (const char *)ptr+cnt, size-cnt, 0);
         if(retval < 0) {
             if(opal_socket_errno != EINTR && opal_socket_errno != EAGAIN && opal_socket_errno != EWOULDBLOCK) {
-                BTL_ERROR(("send() failed with errno=%d",opal_socket_errno));
+                BTL_ERROR(("send() failed: %s (%d)",
+                           strerror(opal_socket_errno), opal_socket_errno));
                 mca_btl_tcp_endpoint_close(btl_endpoint);
                 return -1;
             }
@@ -295,42 +335,46 @@ static int mca_btl_tcp_endpoint_send_connect_ack(mca_btl_base_endpoint_t* btl_en
  * otherwise, reject the connection and continue with the current connection 
  */
 
-bool mca_btl_tcp_endpoint_accept(mca_btl_base_endpoint_t* btl_endpoint, struct sockaddr_in* addr, int sd)
+bool mca_btl_tcp_endpoint_accept(mca_btl_base_endpoint_t* btl_endpoint,
+                                 struct sockaddr* addr, int sd)
 {
     mca_btl_tcp_addr_t* btl_addr;
     mca_btl_tcp_proc_t* this_proc = mca_btl_tcp_proc_local();
-    orte_ns_cmp_bitmask_t mask = ORTE_NS_CMP_ALL;
+    mca_btl_tcp_proc_t *endpoint_proc = btl_endpoint->endpoint_proc;
     int cmpval;
 
     OPAL_THREAD_LOCK(&btl_endpoint->endpoint_recv_lock);
     OPAL_THREAD_LOCK(&btl_endpoint->endpoint_send_lock);
-    if((btl_addr = btl_endpoint->endpoint_addr) != NULL  &&
-        btl_addr->addr_inet.s_addr == addr->sin_addr.s_addr) {
-        mca_btl_tcp_proc_t *endpoint_proc = btl_endpoint->endpoint_proc;
-        cmpval = orte_ns.compare_fields(mask, 
-                                 &endpoint_proc->proc_ompi->proc_name,
-                                 &this_proc->proc_ompi->proc_name);
-        if((btl_endpoint->endpoint_sd < 0) ||
-           (btl_endpoint->endpoint_state != MCA_BTL_TCP_CONNECTED &&
-            cmpval < 0)) {
+
+    if(NULL == (btl_addr = btl_endpoint->endpoint_addr)) {
+        OPAL_THREAD_UNLOCK(&btl_endpoint->endpoint_send_lock);
+        OPAL_THREAD_UNLOCK(&btl_endpoint->endpoint_recv_lock);
+        return false;
+    }
+
+    cmpval = orte_util_compare_name_fields(ORTE_NS_CMP_ALL, 
+                                    &endpoint_proc->proc_ompi->proc_name,
+                                    &this_proc->proc_ompi->proc_name);
+    if((btl_endpoint->endpoint_sd < 0) ||
+       (btl_endpoint->endpoint_state != MCA_BTL_TCP_CONNECTED &&
+        cmpval < 0)) {
+        mca_btl_tcp_endpoint_close(btl_endpoint);
+        btl_endpoint->endpoint_sd = sd;
+        if(mca_btl_tcp_endpoint_send_connect_ack(btl_endpoint) != OMPI_SUCCESS) {
             mca_btl_tcp_endpoint_close(btl_endpoint);
-            btl_endpoint->endpoint_sd = sd;
-            if(mca_btl_tcp_endpoint_send_connect_ack(btl_endpoint) != OMPI_SUCCESS) {
-                 mca_btl_tcp_endpoint_close(btl_endpoint);
-                 OPAL_THREAD_UNLOCK(&btl_endpoint->endpoint_send_lock);
-                 OPAL_THREAD_UNLOCK(&btl_endpoint->endpoint_recv_lock);
-                 return false;
-            }
-            mca_btl_tcp_endpoint_event_init(btl_endpoint, sd);
-            opal_event_add(&btl_endpoint->endpoint_recv_event, 0);
-            mca_btl_tcp_endpoint_connected(btl_endpoint);
-#if OMPI_ENABLE_DEBUG && WANT_PEER_DUMP
-            mca_btl_tcp_endpoint_dump(btl_endpoint, "accepted");
-#endif
             OPAL_THREAD_UNLOCK(&btl_endpoint->endpoint_send_lock);
             OPAL_THREAD_UNLOCK(&btl_endpoint->endpoint_recv_lock);
-            return true;
+            return false;
         }
+        mca_btl_tcp_endpoint_event_init(btl_endpoint);
+        opal_event_add(&btl_endpoint->endpoint_recv_event, 0);
+        mca_btl_tcp_endpoint_connected(btl_endpoint);
+#if OMPI_ENABLE_DEBUG && WANT_PEER_DUMP
+        mca_btl_tcp_endpoint_dump(btl_endpoint, "accepted");
+#endif
+        OPAL_THREAD_UNLOCK(&btl_endpoint->endpoint_send_lock);
+        OPAL_THREAD_UNLOCK(&btl_endpoint->endpoint_recv_lock);
+        return true;
     }
     OPAL_THREAD_UNLOCK(&btl_endpoint->endpoint_send_lock);
     OPAL_THREAD_UNLOCK(&btl_endpoint->endpoint_recv_lock);
@@ -343,39 +387,28 @@ bool mca_btl_tcp_endpoint_accept(mca_btl_base_endpoint_t* btl_endpoint, struct s
  * and update the endpoint state to reflect the connection has
  * been closed.
  */
-
 void mca_btl_tcp_endpoint_close(mca_btl_base_endpoint_t* btl_endpoint)
 {
-    if(btl_endpoint->endpoint_sd >= 0) {
-        opal_event_del(&btl_endpoint->endpoint_recv_event);
-        opal_event_del(&btl_endpoint->endpoint_send_event);
-        CLOSE_THE_SOCKET(btl_endpoint->endpoint_sd);
-        btl_endpoint->endpoint_sd = -1;
-#if MCA_BTL_TCP_ENDPOINT_CACHE
-        free( btl_endpoint->endpoint_cache );
-        btl_endpoint->endpoint_cache        = NULL;
-        btl_endpoint->endpoint_cache_pos    = NULL;
-        btl_endpoint->endpoint_cache_length = 0;
-#endif  /* MCA_BTL_TCP_ENDPOINT_CACHE */
-    }
+    if(btl_endpoint->endpoint_sd < 0)
+        return;
     btl_endpoint->endpoint_state = MCA_BTL_TCP_CLOSED;
     btl_endpoint->endpoint_retries++;
+    opal_event_del(&btl_endpoint->endpoint_recv_event);
+    opal_event_del(&btl_endpoint->endpoint_send_event);
+    CLOSE_THE_SOCKET(btl_endpoint->endpoint_sd);
+    btl_endpoint->endpoint_sd = -1;
+#if MCA_BTL_TCP_ENDPOINT_CACHE
+    free( btl_endpoint->endpoint_cache );
+    btl_endpoint->endpoint_cache        = NULL;
+    btl_endpoint->endpoint_cache_pos    = NULL;
+    btl_endpoint->endpoint_cache_length = 0;
+#endif  /* MCA_BTL_TCP_ENDPOINT_CACHE */
 }
 
-void mca_btl_tcp_endpoint_shutdown(mca_btl_base_endpoint_t* btl_endpoint)
-{
-    OPAL_THREAD_LOCK(&btl_endpoint->endpoint_recv_lock);
-    OPAL_THREAD_LOCK(&btl_endpoint->endpoint_send_lock);
-    mca_btl_tcp_endpoint_close(btl_endpoint);
-    btl_endpoint->endpoint_state = MCA_BTL_TCP_SHUTDOWN;
-    OPAL_THREAD_UNLOCK(&btl_endpoint->endpoint_send_lock);
-    OPAL_THREAD_UNLOCK(&btl_endpoint->endpoint_recv_lock);
-}
-
-
 /*
  *  Setup endpoint state to reflect that connection has been established,
- *  and start any pending sends.
+ *  and start any pending sends. This function should be called with the
+ *  send lock locked.
  */
 
 static void mca_btl_tcp_endpoint_connected(mca_btl_base_endpoint_t* btl_endpoint)
@@ -383,6 +416,14 @@ static void mca_btl_tcp_endpoint_connected(mca_btl_base_endpoint_t* btl_endpoint
     /* setup socket options */
     btl_endpoint->endpoint_state = MCA_BTL_TCP_CONNECTED;
     btl_endpoint->endpoint_retries = 0;
+
+    /* Create the send event in a persistent manner. */
+    opal_event_set( &btl_endpoint->endpoint_send_event, 
+                    btl_endpoint->endpoint_sd, 
+                    OPAL_EV_WRITE | OPAL_EV_PERSIST,
+                    mca_btl_tcp_endpoint_send_handler,
+                    btl_endpoint );
+
     if(opal_list_get_size(&btl_endpoint->endpoint_frags) > 0) {
         if(NULL == btl_endpoint->endpoint_send_frag)
             btl_endpoint->endpoint_send_frag = (mca_btl_tcp_frag_t*)
@@ -412,7 +453,8 @@ static int mca_btl_tcp_endpoint_recv_blocking(mca_btl_base_endpoint_t* btl_endpo
         /* socket is non-blocking so handle errors */
         if(retval < 0) {
             if(opal_socket_errno != EINTR && opal_socket_errno != EAGAIN && opal_socket_errno != EWOULDBLOCK) {
-                BTL_ERROR(("recv() failed with errno=%d",opal_socket_errno));
+                BTL_ERROR(("recv(%d) failed: %s (%d)",
+                           btl_endpoint->endpoint_sd, strerror(opal_socket_errno), opal_socket_errno));
                 mca_btl_tcp_endpoint_close(btl_endpoint);
                 return -1;
             }
@@ -424,13 +466,11 @@ static int mca_btl_tcp_endpoint_recv_blocking(mca_btl_base_endpoint_t* btl_endpo
 }
 
 
-
 /*
  *  Receive the endpoints globally unique process identification from a newly
  *  connected socket and verify the expected response. If so, move the
  *  socket to a connected state.
  */
-
 static int mca_btl_tcp_endpoint_recv_connect_ack(mca_btl_base_endpoint_t* btl_endpoint)
 {
     orte_process_name_t guid;
@@ -440,20 +480,14 @@ static int mca_btl_tcp_endpoint_recv_connect_ack(mca_btl_base_endpoint_t* btl_en
         return OMPI_ERR_UNREACH;
     }
     ORTE_PROCESS_NAME_NTOH(guid);
-
     /* compare this to the expected values */
-    if(memcmp(&btl_proc->proc_name, &guid, sizeof(orte_process_name_t)) != 0) {
-        BTL_ERROR(("received unexpected process identifier [%lu,%lu,%lu]", 
-            ORTE_NAME_ARGS(&guid)));
+    if (OPAL_EQUAL != orte_util_compare_name_fields(ORTE_NS_CMP_ALL, &btl_proc->proc_name, &guid)) {
+        BTL_ERROR(("received unexpected process identifier %s", 
+                   ORTE_NAME_PRINT(&guid)));
         mca_btl_tcp_endpoint_close(btl_endpoint);
         return OMPI_ERR_UNREACH;
     }
 
-    /* connected */
-    mca_btl_tcp_endpoint_connected(btl_endpoint);
-#if OMPI_ENABLE_DEBUG && WANT_PEER_DUMP
-    mca_btl_tcp_endpoint_dump(btl_endpoint, "connected");
-#endif
     return OMPI_SUCCESS;
 }
 
@@ -462,21 +496,24 @@ void mca_btl_tcp_set_socket_options(int sd)
 {
     int optval;
 #if defined(TCP_NODELAY)
-    optval = 1;
+    optval = mca_btl_tcp_component.tcp_use_nodelay;
     if(setsockopt(sd, IPPROTO_TCP, TCP_NODELAY, (char *)&optval, sizeof(optval)) < 0) {
-        BTL_ERROR(("setsockopt(TCP_NODELAY) failed with errno=%d", opal_socket_errno));
+        BTL_ERROR(("setsockopt(TCP_NODELAY) failed: %s (%d)", 
+                   strerror(opal_socket_errno), opal_socket_errno));
     }
 #endif
 #if defined(SO_SNDBUF)
     if(mca_btl_tcp_component.tcp_sndbuf > 0 &&
        setsockopt(sd, SOL_SOCKET, SO_SNDBUF, (char *)&mca_btl_tcp_component.tcp_sndbuf, sizeof(int)) < 0) {
-        BTL_ERROR(("setsockopt(SO_SNDBUF) failed with errno %d", opal_socket_errno));
+        BTL_ERROR(("setsockopt(SO_SNDBUF) failed: %s (%d)", 
+                   strerror(opal_socket_errno), opal_socket_errno));
     }
 #endif
 #if defined(SO_RCVBUF)
     if(mca_btl_tcp_component.tcp_rcvbuf > 0 &&
        setsockopt(sd, SOL_SOCKET, SO_RCVBUF, (char *)&mca_btl_tcp_component.tcp_rcvbuf, sizeof(int)) < 0) {
-        BTL_ERROR(("setsockopt(SO_RCVBUF) failed with errno %d", opal_socket_errno));
+        BTL_ERROR(("setsockopt(SO_RCVBUF) failed: %s (%d)", 
+                   strerror(opal_socket_errno), opal_socket_errno));
     }
 #endif
 }
@@ -490,13 +527,22 @@ void mca_btl_tcp_set_socket_options(int sd)
  *  our globally unique process identifier to the endpoint and wait for
  *  the endpoints response.
  */
-
 static int mca_btl_tcp_endpoint_start_connect(mca_btl_base_endpoint_t* btl_endpoint)
 {
     int rc,flags;
-    struct sockaddr_in endpoint_addr;
-
-    btl_endpoint->endpoint_sd = socket(AF_INET, SOCK_STREAM, 0);
+    struct sockaddr_storage endpoint_addr;
+    /* By default consider a IPv4 connection */
+    uint16_t af_family = AF_INET;
+    opal_socklen_t addrlen = sizeof(struct sockaddr_in);
+    
+#if OPAL_WANT_IPV6
+    if (AF_INET6 == btl_endpoint->endpoint_addr->addr_family) {
+        af_family = AF_INET6;
+        addrlen = sizeof (struct sockaddr_in6);
+    }
+#endif
+    
+    btl_endpoint->endpoint_sd = socket(af_family, SOCK_STREAM, 0);
     if (btl_endpoint->endpoint_sd < 0) {
         btl_endpoint->endpoint_retries++;
         return OMPI_ERR_UNREACH;
@@ -506,28 +552,42 @@ static int mca_btl_tcp_endpoint_start_connect(mca_btl_base_endpoint_t* btl_endpo
     mca_btl_tcp_set_socket_options(btl_endpoint->endpoint_sd);
 
     /* setup event callbacks */
-    mca_btl_tcp_endpoint_event_init(btl_endpoint, btl_endpoint->endpoint_sd);
+    mca_btl_tcp_endpoint_event_init(btl_endpoint);
 
     /* setup the socket as non-blocking */
     if((flags = fcntl(btl_endpoint->endpoint_sd, F_GETFL, 0)) < 0) {
-        BTL_ERROR(("fcntl(F_GETFL) failed with errno=%d", opal_socket_errno));
+        BTL_ERROR(("fcntl(F_GETFL) failed: %s (%d)", 
+                   strerror(opal_socket_errno), opal_socket_errno));
     } else {
         flags |= O_NONBLOCK;
         if(fcntl(btl_endpoint->endpoint_sd, F_SETFL, flags) < 0)
-            BTL_ERROR(("fcntl(F_SETFL) failed with errno=%d", opal_socket_errno));
+            BTL_ERROR(("fcntl(F_SETFL) failed: %s (%d)", 
+                       strerror(opal_socket_errno), opal_socket_errno));
     }
 
     /* start the connect - will likely fail with EINPROGRESS */
-    endpoint_addr.sin_family = AF_INET;
-    endpoint_addr.sin_addr = btl_endpoint->endpoint_addr->addr_inet;
-    endpoint_addr.sin_port = btl_endpoint->endpoint_addr->addr_port;
-    if(connect(btl_endpoint->endpoint_sd, (struct sockaddr*)&endpoint_addr, sizeof(endpoint_addr)) < 0) {
+    mca_btl_tcp_proc_tosocks(btl_endpoint->endpoint_addr, &endpoint_addr);
+
+    opal_output_verbose(20, mca_btl_base_output, 
+                        "btl: tcp: attempting to connect() to address %s on port %d",
+                        opal_net_get_hostname((struct sockaddr*) &endpoint_addr),
+                        btl_endpoint->endpoint_addr->addr_port);
+
+    if(connect(btl_endpoint->endpoint_sd, (struct sockaddr*)&endpoint_addr, addrlen) < 0) {
         /* non-blocking so wait for completion */
         if(opal_socket_errno == EINPROGRESS || opal_socket_errno == EWOULDBLOCK) {
             btl_endpoint->endpoint_state = MCA_BTL_TCP_CONNECTING;
             opal_event_add(&btl_endpoint->endpoint_send_event, 0);
             return OMPI_SUCCESS;
         }
+        {
+            char *address;
+            address = opal_net_get_hostname((struct sockaddr*) &endpoint_addr);
+            BTL_PEER_ERROR( btl_endpoint->endpoint_proc->proc_ompi,
+                          ( "Unable to connect to the peer %s on port %d: %s\n",
+                            address,
+                           btl_endpoint->endpoint_addr->addr_port, strerror(opal_socket_errno) ) );
+        }
         mca_btl_tcp_endpoint_close(btl_endpoint);
         btl_endpoint->endpoint_retries++;
         return OMPI_ERR_UNREACH;
@@ -549,7 +609,6 @@ static int mca_btl_tcp_endpoint_start_connect(mca_btl_base_endpoint_t* btl_endpo
  * later. Otherwise, send this processes identifier to the endpoint on the 
  * newly connected socket.
  */
-
 static void mca_btl_tcp_endpoint_complete_connect(mca_btl_base_endpoint_t* btl_endpoint)
 {
     int so_error = 0;
@@ -560,7 +619,7 @@ static void mca_btl_tcp_endpoint_complete_connect(mca_btl_base_endpoint_t* btl_e
 
     /* check connect completion status */
     if(getsockopt(btl_endpoint->endpoint_sd, SOL_SOCKET, SO_ERROR, (char *)&so_error, &so_length) < 0) {
-        BTL_ERROR(("getsockopt() failed with errno=%d", opal_socket_errno));
+        BTL_ERROR(("getsockopt() failed: %s (%d)", strerror(opal_socket_errno), opal_socket_errno));
         mca_btl_tcp_endpoint_close(btl_endpoint);
         return;
     }
@@ -569,7 +628,7 @@ static void mca_btl_tcp_endpoint_complete_connect(mca_btl_base_endpoint_t* btl_e
         return;
     }
     if(so_error != 0) {
-        BTL_ERROR(("connect() failed with errno=%d", so_error));
+        BTL_ERROR(("connect() failed: %s (%d)", strerror(so_error), so_error));
         mca_btl_tcp_endpoint_close(btl_endpoint);
         return;
     }
@@ -591,13 +650,30 @@ static void mca_btl_tcp_endpoint_complete_connect(mca_btl_base_endpoint_t* btl_e
 static void mca_btl_tcp_endpoint_recv_handler(int sd, short flags, void* user)
 {
     mca_btl_base_endpoint_t* btl_endpoint = (mca_btl_base_endpoint_t *)user;
+
+    /* Make sure we don't have a race between a thread that remove the
+     * recv event, and one event already scheduled.
+     */
+    if( sd != btl_endpoint->endpoint_sd )
+        return;
+
     OPAL_THREAD_LOCK(&btl_endpoint->endpoint_recv_lock);
     switch(btl_endpoint->endpoint_state) {
     case MCA_BTL_TCP_CONNECT_ACK:
         {
-            mca_btl_tcp_endpoint_recv_connect_ack(btl_endpoint);
+            int rc = OMPI_ERROR;
+            rc = mca_btl_tcp_endpoint_recv_connect_ack(btl_endpoint);
+            if( OMPI_SUCCESS == rc ) {
+                /* we are now connected. Start sending the data */
+                OPAL_THREAD_LOCK(&btl_endpoint->endpoint_send_lock);
+                mca_btl_tcp_endpoint_connected(btl_endpoint);
+                OPAL_THREAD_UNLOCK(&btl_endpoint->endpoint_send_lock);
+#if OMPI_ENABLE_DEBUG && WANT_PEER_DUMP
+                mca_btl_tcp_endpoint_dump(btl_endpoint, "connected");
+#endif
+            }
             OPAL_THREAD_UNLOCK(&btl_endpoint->endpoint_recv_lock);
-            break;
+            return;
         }
     case MCA_BTL_TCP_CONNECTED:
         {
@@ -625,32 +701,39 @@ static void mca_btl_tcp_endpoint_recv_handler(int sd, short flags, void* user)
         data_still_pending_on_endpoint:
 #endif  /* MCA_BTL_TCP_ENDPOINT_CACHE */
             /* check for completion of non-blocking recv on the current fragment */
-            if(mca_btl_tcp_frag_recv(frag, sd) == false) {
+            if(mca_btl_tcp_frag_recv(frag, btl_endpoint->endpoint_sd) == false) {
                 btl_endpoint->endpoint_recv_frag = frag;
             } else {
                 btl_endpoint->endpoint_recv_frag = NULL;
-		if( MCA_BTL_TCP_HDR_TYPE_SEND == frag->hdr.type ) {
-		  mca_btl_base_recv_reg_t* reg = frag->btl->tcp_reg + frag->hdr.base.tag;
-		  reg->cbfunc(&frag->btl->super, frag->hdr.base.tag, &frag->base, reg->cbdata);
+                if( MCA_BTL_TCP_HDR_TYPE_SEND == frag->hdr.type ) {
+                    mca_btl_active_message_callback_t* reg;
+                    reg = mca_btl_base_active_message_trigger + frag->hdr.base.tag;
+                    reg->cbfunc(&frag->btl->super, frag->hdr.base.tag, &frag->base, reg->cbdata);
                 }
 #if MCA_BTL_TCP_ENDPOINT_CACHE
                 if( 0 != btl_endpoint->endpoint_cache_length ) {
-		    /* If the cache still contain some data we can reuse the same fragment
-		     * until we flush it completly.
-		     */
+                    /* If the cache still contain some data we can reuse the same fragment
+                     * until we flush it completly.
+                     */
                     MCA_BTL_TCP_FRAG_INIT_DST(frag, btl_endpoint);
                     goto data_still_pending_on_endpoint;
                 }
 #endif  /* MCA_BTL_TCP_ENDPOINT_CACHE */
                 MCA_BTL_TCP_FRAG_RETURN(frag);
             }
-            OPAL_THREAD_UNLOCK(&btl_endpoint->endpoint_recv_lock);
 #if MCA_BTL_TCP_ENDPOINT_CACHE
             assert( 0 == btl_endpoint->endpoint_cache_length );
 #endif  /* MCA_BTL_TCP_ENDPOINT_CACHE */
+            OPAL_THREAD_UNLOCK(&btl_endpoint->endpoint_recv_lock);
             break;
         }
-    case MCA_BTL_TCP_SHUTDOWN:
+    case MCA_BTL_TCP_CLOSED:
+        /* This is a thread-safety issue. As multiple threads are allowed
+         * to generate events (in the lib event) we endup with several
+         * threads executing the receive callback, when we reach the end
+         * of the MPI_Finalize. The first one will close the connections,
+         * and all others will complain.
+         */
         OPAL_THREAD_UNLOCK(&btl_endpoint->endpoint_recv_lock);
         break;
     default:
@@ -676,10 +759,11 @@ static void mca_btl_tcp_endpoint_send_handler(int sd, short flags, void* user)
         mca_btl_tcp_endpoint_complete_connect(btl_endpoint);
         break;
     case MCA_BTL_TCP_CONNECTED:
-        {
         /* complete the current send */
-        do {
+        while (NULL != btl_endpoint->endpoint_send_frag) {
             mca_btl_tcp_frag_t* frag = btl_endpoint->endpoint_send_frag;
+            int btl_ownership = (frag->base.des_flags & MCA_BTL_DES_FLAGS_BTL_OWNERSHIP);
+
             if(mca_btl_tcp_frag_send(frag, btl_endpoint->endpoint_sd) == false) {
                 break;
             }
@@ -689,20 +773,22 @@ static void mca_btl_tcp_endpoint_send_handler(int sd, short flags, void* user)
 
             /* if required - update request status and release fragment */
             OPAL_THREAD_UNLOCK(&btl_endpoint->endpoint_send_lock);
+            assert( frag->base.des_flags & MCA_BTL_DES_SEND_ALWAYS_CALLBACK );
             frag->base.des_cbfunc(&frag->btl->super, frag->endpoint, &frag->base, frag->rc);
+            if( btl_ownership ) {
+                MCA_BTL_TCP_FRAG_RETURN(frag);
+            }
             OPAL_THREAD_LOCK(&btl_endpoint->endpoint_send_lock);
 
-        } while (NULL != btl_endpoint->endpoint_send_frag);
+        }
 
         /* if nothing else to do unregister for send event notifications */
         if(NULL == btl_endpoint->endpoint_send_frag) {
             opal_event_del(&btl_endpoint->endpoint_send_event);
         }
         break;
-        }
     default:
-        BTL_ERROR(("invalid connection state (%d)",
-            btl_endpoint->endpoint_state));
+        BTL_ERROR(("invalid connection state (%d)", btl_endpoint->endpoint_state));
         opal_event_del(&btl_endpoint->endpoint_send_event);
         break;
     }
