@@ -1,14 +1,17 @@
+/* -*- Mode: C; c-basic-offset:4 ; -*- */
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2007 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2007-2008 University of Houston. All rights reserved.
+ * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -35,81 +38,25 @@
 #include "ompi/mca/bml/bml.h"
 #include "ompi/mca/bml/base/base.h"
 #include "ompi/mca/pml/pml.h"
+#include "ompi/mca/pml/base/base.h"
 #include "ompi/mca/btl/btl.h"
 
-
 /* Local functions and data */
-#define HIER_MAXPROTOCOL 7
+#define HIER_MAXPROTOCOL 6
+#define HIER_MAX_PROTNAMELEN 7
 static int mca_coll_hierarch_max_protocol=HIER_MAXPROTOCOL;
 
-static char hier_prot[HIER_MAXPROTOCOL][7]={"0","tcp","gm","mx","mvapi","openib","sm"};
+/* Commments: need to add ofud, portals and sctp into this list! */
+static char hier_prot[HIER_MAXPROTOCOL][HIER_MAX_PROTNAMELEN]={"0","tcp","udapl","mx","openib","sm"};
 
 static void mca_coll_hierarch_checkfor_component (struct ompi_communicator_t *comm,
 						  int component_level,
 						  char *component_name, 
 						  int *key, int *ncount);
-static void mca_coll_hierarch_dump_struct ( struct mca_coll_base_comm_t *c);
-
-/*
- * Linear set of collective algorithms
- */
-static const mca_coll_base_module_1_0_0_t intra = {
-
-  /* Initialization / finalization functions */
-
-  mca_coll_hierarch_module_init,
-  mca_coll_hierarch_module_finalize,
-
-  /* Collective function pointers */
-  /* function pointers marked with NULL are not yet implemented
-     and will use the functions provided in the basic module */
-  NULL, /* allgather */
-  NULL, /* allgatherv */
-  mca_coll_hierarch_allreduce_intra, 
-  NULL, /* alltoall */
-  NULL, /* alltoallv */
-  NULL, /* alltoallw */
-  mca_coll_hierarch_barrier_intra,
-  mca_coll_hierarch_bcast_intra,
-  NULL,  /* exscan */
-  NULL,  /* gather */
-  NULL,  /* gatherv */
-  mca_coll_hierarch_reduce_intra,
-  NULL,  /* reduce_scatter */
-  NULL,  /* scan */
-  NULL,  /* scatter */
-  NULL   /* scatterv */
-};
-
-
-static const mca_coll_base_module_1_0_0_t null_intra = {
-
-  /* Initialization / finalization functions */
-
-  mca_coll_hierarch_module_init,
-  mca_coll_hierarch_module_finalize,
-
-  /* Collective function pointers */
-  /* function pointers marked with NULL are not yet implemented
-     and will use the functions provided in the basic module */
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL, 
-  NULL, 
-  NULL, 
-  NULL,
-  NULL, 
-  NULL,
-  NULL, 
-  NULL
-};
-
+static void mca_coll_hierarch_checkfor_sm (struct ompi_communicator_t *comm,
+					   int *color,
+					   int *ncount);
+static void mca_coll_hierarch_dump_struct ( mca_coll_hierarch_module_t *c);
 
 
 /*
@@ -117,9 +64,8 @@ static const mca_coll_base_module_1_0_0_t null_intra = {
  * this module to indicate what level of thread support it provides.
  */
 int mca_coll_hierarch_init_query(bool allow_hierarch_user_threads,
-                             bool have_hidden_user_threads)
+                                 bool have_hidden_user_threads)
 {
-
     /* Don't ask. All done */
     return OMPI_SUCCESS;
 }
@@ -130,145 +76,169 @@ int mca_coll_hierarch_init_query(bool allow_hierarch_user_threads,
  * Look at the communicator and decide which set of functions and
  * priority we want to return.
  */
-const mca_coll_base_module_1_0_0_t *
-mca_coll_hierarch_comm_query(struct ompi_communicator_t *comm, int *priority,
-                             struct mca_coll_base_comm_t **data)
+mca_coll_base_module_t *
+mca_coll_hierarch_comm_query(struct ompi_communicator_t *comm, int *priority )
 {
     int size, rank;
-    int color, ncount, maxncount;
-    struct mca_coll_base_comm_t *tdata=NULL;
+    int color, ncount=0, maxncount;
     int level;
     int ret=OMPI_SUCCESS;
     int ignore_sm=0;
-    int symmetric=0;
+    int detection_alg=0;
+    mca_coll_hierarch_module_t *hierarch_module;
 
     /* This module only works for intra-communicators at the moment */
-    if ( OMPI_COMM_IS_INTER(comm) ) {
+    if (OMPI_COMM_IS_INTER(comm)) {
         return NULL;
     }
-
-
-    /* Get the priority level attached to this module. If priority = 0,
-       we assume that we won't be chosen anyway, so we quit and improve
-       therefore the startup time. */
+    
+    /* Get the priority level attached to this module. If priority is less
+     * than or equal to 0, then the module is unavailable. */
     *priority = mca_coll_hierarch_priority_param;
-    if ( 0 >= mca_coll_hierarch_priority_param ) {
+    if (0 >= mca_coll_hierarch_priority_param) {
 	return NULL;
     }
 
+    /* This module only works with ob1 pml respectively btl's. Opt out
+       of we use cm/mtl's. */
+    if ( strcmp ( mca_pml_base_selected_component.pmlm_version.mca_component_name, "ob1")) {
+	return NULL;
+    }
+
+
+    size = ompi_comm_size(comm);
+    if (size < 3) {
+	/* No need for hierarchical collectives for 1 or 2 procs. */
+	return NULL;
+    }
+
+    hierarch_module = OBJ_NEW(mca_coll_hierarch_module_t);
+    if (NULL == hierarch_module) {
+	return NULL;
+    }
+
+    hierarch_module->super.coll_module_enable = mca_coll_hierarch_module_enable;
+    hierarch_module->super.ft_event = mca_coll_hierarch_ft_event;
+    hierarch_module->super.coll_allgather  = NULL;
+    hierarch_module->super.coll_allgatherv = NULL;
+    hierarch_module->super.coll_allreduce  = mca_coll_hierarch_allreduce_intra;
+    hierarch_module->super.coll_alltoall   = NULL;
+    hierarch_module->super.coll_alltoallv  = NULL;
+    hierarch_module->super.coll_alltoallw  = NULL;
+    hierarch_module->super.coll_barrier    = mca_coll_hierarch_barrier_intra;
+    hierarch_module->super.coll_bcast      = mca_coll_hierarch_bcast_intra;
+    hierarch_module->super.coll_exscan     = NULL;
+    hierarch_module->super.coll_gather     = NULL;
+    hierarch_module->super.coll_gatherv    = NULL;
+    hierarch_module->super.coll_reduce     = mca_coll_hierarch_reduce_intra;
+    hierarch_module->super.coll_reduce_scatter = NULL;
+    hierarch_module->super.coll_scan       = NULL;
+    hierarch_module->super.coll_scatter    = NULL;
+    hierarch_module->super.coll_scatterv   = NULL;
+
+
     /* Check whether we should ignore sm. This might be necessary to take advantage
        of the some ib or gm collectives. */
     ignore_sm = mca_coll_hierarch_ignore_sm_param;
 
-
-    /* Check whether we can assume a symmetric configuration. This can save commmunication
-       and improve the startup time, since we can conclude from our configuration onto
-       the configuration of every other process.
-    */
-    symmetric = mca_coll_hierarch_symmetric_param;
-
-    size = ompi_comm_size(comm);
-    
-    if ( size < 3 ) {
-	/* No need for hierarchical collectives for 1 or 2 procs. */
-	return NULL;
-    }
-
     rank = ompi_comm_rank(comm);
 
-    /* allocate the data structure holding all information */
-    tdata = calloc ( 1, sizeof(struct mca_coll_base_comm_t));
-    if ( NULL == tdata ) {
-        *priority = 0;
-        return NULL;
-    }
-    
-    tdata->hier_num_colorarr  = size;
-    tdata->hier_colorarr      = (int *) malloc ( sizeof(int) * size);
-    if ( NULL == tdata->hier_colorarr ) {
+    hierarch_module->hier_num_colorarr  = size;
+    hierarch_module->hier_colorarr      = (int *) malloc ( sizeof(int) * size);
+    if ( NULL == hierarch_module->hier_colorarr ) {
         *priority = 0;
         return NULL;
     }
 
     /* 
      * walk through the list of registered protocols, and check which one
-     * is feasable. 
+     * is feasible. 
      * Later we start with level=0, and introduce the multi-cell check 
-    */
+     */
     if ( ignore_sm ) {
 	mca_coll_hierarch_max_protocol = HIER_MAXPROTOCOL - 1;
     }
-    for ( level = mca_coll_hierarch_max_protocol - 1; level >0 ; level--) {
-        mca_coll_hierarch_checkfor_component ( comm, 
-					       level, 
-					       hier_prot[level], 
-					       &color, 
-					       &ncount);
- 
-        /* This is probably a no-no! but for the moment we agreed with Jeff,
-           that this might be the best solution. These functions emulate an 
-	   allreduce and  an allgather.
-        */
-	if ( symmetric ) {
-	    maxncount = ncount;
+
+    /* if number of levels is not specified, or if it is specified as ALL_LEVELS,
+    * proceed in the usual way
+    */
+
+    detection_alg = mca_coll_hierarch_detection_alg_param;
+    if( TWO_LEVELS == detection_alg ) {
+	mca_coll_hierarch_max_protocol = 2;
+	if ( mca_coll_hierarch_verbose_param ) {
+	    printf("Using two level hierarchy detection\n");
 	}
-	else {
-	    ret = mca_coll_hierarch_allreduce_tmp (&ncount, &maxncount, 1, MPI_INT, 
-						   MPI_MAX, comm );
-	    if ( OMPI_SUCCESS != ret ) {
-		return NULL;
-	    }
+    }
+
+    for ( level = mca_coll_hierarch_max_protocol - 1; level >0 ; level--) {
+	if ( ALL_LEVELS == detection_alg ) {
+	    mca_coll_hierarch_checkfor_component ( comm, 
+						   level, 
+						   hier_prot[level], 
+						   &color, 
+						   &ncount);
+	}
+        else if  (TWO_LEVELS == detection_alg ) {
+            mca_coll_hierarch_checkfor_sm ( comm, &color, &ncount );
 	}
 
-        if ( 0 == maxncount ) {
+        /* This is probably a no-no! but for the moment we agreed with Jeff,
+	** that this might be the best solution. These functions emulate an 
+        ** allreduce and  an allgather.
+	*/
+	ret = mca_coll_hierarch_allreduce_tmp (&ncount, &maxncount, 1, MPI_INT, 
+					       MPI_MAX, comm );
+	if ( OMPI_SUCCESS != ret ) {
+	    return NULL;
+	}
+
+	if ( 0 == maxncount ) {
 	    if ( mca_coll_hierarch_verbose_param ) {
 		printf("%s:%d: nobody talks with %s. Continuing to next level.\n",  
 		       comm->c_name, rank, hier_prot[level]);
-		}
-	    continue;
-        }
-        else if ( maxncount == (size-1) ) {
-	    /* 
-             * everybody can talk to every other process with this protocol, 
-             * no need to continue in the hierarchy tree and for the 
-             * hierarchical component.
-             * Its (size-1) because we do not count ourselves.
-	     * maxncount[1] should be zero.
-             */
-	    if ( mca_coll_hierarch_verbose_param ) {
-		printf("%s:%d: everybody talks with %s. No need to continue\n", 
-		       comm->c_name, rank, hier_prot[level]);
 	    }
-            goto exit;
-        }
-        else {
+	    continue;
+	}
+	else if ( maxncount == (size-1) ) {
+	    /* 
+	     * everybody can talk to every other process with this protocol, 
+	     * no need to continue in the hierarchy tree and for the 
+	     * hierarchical component.
+	     * Its (size-1) because we do not count ourselves.
+	     * maxncount[1] should be zero.
+	     */
+	    if ( mca_coll_hierarch_verbose_param ) {
+		if ( ALL_LEVELS == detection_alg ) {
+		    printf("%s:%d: everybody talks with %s. No need to continue\n", 
+			   comm->c_name, rank, hier_prot[level]);
+		}
+		else if ( TWO_LEVELS == detection_alg ) {
+		    printf("%s:%d: everybody talks with sm. No need to continue\n",
+			   comm->c_name, rank );
+		}
+	    }
+	    goto exit;
+	}
+	else {
 	    if ( mca_coll_hierarch_verbose_param ) {
 		printf("%s:%d: %d procs talk with %s. Use this protocol, key %d\n", 
 		       comm->c_name, rank, maxncount, hier_prot[level], color);
 	    }
-
-            ret = mca_coll_hierarch_allgather_tmp (&color, 1, MPI_INT, 
-						   tdata->hier_colorarr, 1, 
+	    
+	    ret = mca_coll_hierarch_allgather_tmp (&color, 1, MPI_INT, 
+						   hierarch_module->hier_colorarr, 1, 
 						   MPI_INT, comm );
-            if ( OMPI_SUCCESS != ret ) {
-        	return NULL;
-            }
-
-           tdata->hier_level = level;
-           *data = tdata;
-	   return &null_intra;
-        }
+	    if ( OMPI_SUCCESS != ret ) {
+		return NULL;
+	    }
+	    
+	    hierarch_module->hier_level = level;
+	    return &(hierarch_module->super);
+	}
     }
         
  exit:
-    if ( NULL != tdata->hier_colorarr ) {
-        free ( tdata->hier_colorarr ) ;
-    }
-    
-    if ( NULL != tdata ) {
-        free ( tdata );
-    }
-
     *priority = 0;
     return NULL;
 }
@@ -277,23 +247,21 @@ mca_coll_hierarch_comm_query(struct ompi_communicator_t *comm, int *priority,
 /*
  * Init module on the communicator
  */
-const struct mca_coll_base_module_1_0_0_t *
-mca_coll_hierarch_module_init(struct ompi_communicator_t *comm)
+int mca_coll_hierarch_module_enable (mca_coll_base_module_t *module,
+				     struct ompi_communicator_t *comm)
 {
     int color;
     int size, rank, ret=OMPI_SUCCESS;
-    int *llr=NULL;
     
     struct ompi_communicator_t *lcomm=NULL;
     struct ompi_communicator_t *llcomm=NULL;
-    struct mca_coll_base_comm_t *data=NULL;
     struct mca_coll_hierarch_llead_t *llead=NULL;
-    
+    mca_coll_hierarch_module_t *hierarch_module = (mca_coll_hierarch_module_t *) module;
+
     rank = ompi_comm_rank(comm);
     size = ompi_comm_size(comm);
     
-    data = comm->c_coll_selected_data;
-    color = data->hier_colorarr[rank];
+    color = hierarch_module->hier_colorarr[rank];
     
     /* Generate the subcommunicator based on the color returned by
        the previous function. */
@@ -302,11 +270,11 @@ mca_coll_hierarch_module_init(struct ompi_communicator_t *comm)
         goto exit;
     }
     
-    data->hier_comm     = comm;
-    data->hier_lcomm    = lcomm;
-    data->hier_num_reqs = 2 * size;
-    data->hier_reqs     = (ompi_request_t **) malloc (sizeof(ompi_request_t)*size*2);
-    if ( NULL == data->hier_reqs ) {
+    hierarch_module->hier_comm     = comm;
+    hierarch_module->hier_lcomm    = lcomm;
+    hierarch_module->hier_num_reqs = 2 * size;
+    hierarch_module->hier_reqs     = (ompi_request_t **) malloc (sizeof(ompi_request_t)*size*2);
+    if ( NULL == hierarch_module->hier_reqs ) {
         goto exit;
     }
     
@@ -314,7 +282,7 @@ mca_coll_hierarch_module_init(struct ompi_communicator_t *comm)
        information about local leader and the according subcommunicators 
     */
     llead = (struct mca_coll_hierarch_llead_t * ) malloc ( 
-	sizeof(struct mca_coll_hierarch_llead_t));
+                                                          sizeof(struct mca_coll_hierarch_llead_t));
     if ( NULL == llead ) {
         goto exit;
     }
@@ -325,8 +293,8 @@ mca_coll_hierarch_module_init(struct ompi_communicator_t *comm)
      * depending on the offset, and has to be called therefore every time we need 
      * a new llcomm 
      */
-    mca_coll_hierarch_get_llr ( data );
-    mca_coll_hierarch_get_all_lleaders ( rank, data, llead, 1 );        
+    mca_coll_hierarch_get_llr ( hierarch_module );
+    mca_coll_hierarch_get_all_lleaders ( rank, hierarch_module, llead, 1 );        
     
     /* Generate the lleader communicator assuming that all lleaders are the first
        process in the list of processes with the same color. A function generating 
@@ -338,123 +306,63 @@ mca_coll_hierarch_module_init(struct ompi_communicator_t *comm)
     llead->llcomm = llcomm;
     
     /* Store it now on the data structure */
-    OBJ_CONSTRUCT(&(data->hier_llead), ompi_pointer_array_t);
-    ompi_pointer_array_add ( &(data->hier_llead), llead);
+    OBJ_CONSTRUCT(&(hierarch_module->hier_llead), opal_pointer_array_t);
+    opal_pointer_array_add ( &(hierarch_module->hier_llead), llead);
     
     if ( mca_coll_hierarch_verbose_param ) {
-      mca_coll_hierarch_dump_struct (data);
+        mca_coll_hierarch_dump_struct (hierarch_module);
     }
     
  exit:
-    if ( NULL != llr ) {
-	free ( llr );
-    }
     if ( OMPI_SUCCESS != ret ) {
-        ompi_comm_free ( &lcomm );
-        if ( NULL != data ) {
-            if ( NULL != data->hier_reqs ) {
-        	free ( data->hier_reqs);
-            }
-            if ( NULL != data->hier_colorarr ) {
-        	free ( data->hier_colorarr ) ;
-            }
-            if ( NULL != llead->lleaders ) {
-        	free ( llead->lleaders);
-            }
-	    if ( NULL != data->hier_llr ) {
-		free ( data->hier_llr );
-	    }
-            free ( data );
+        if (NULL != llead) {
+            free(llead);
         }
-        return NULL;
+        ompi_comm_free ( &lcomm );
+	return OMPI_ERROR;
     }
-    
-    return &intra;
-}
 
-
-/*
- * Finalize module on the communicator
- */
-int mca_coll_hierarch_module_finalize(struct ompi_communicator_t *comm)
-{
-    int i;
-    int size;
-    struct ompi_communicator_t *lcomm=NULL;
-    struct mca_coll_base_comm_t *data=NULL;
-    struct mca_coll_hierarch_llead_t *current=NULL;
-
-    data   = comm->c_coll_selected_data;
-    lcomm = data->hier_lcomm;
-
-    ompi_comm_free (&lcomm);
-    free ( data->hier_reqs );
-
-    size = ompi_pointer_array_get_size ( &(data->hier_llead));
-    for ( i=0; i<size; i++) {
-	current = (struct mca_coll_hierarch_llead_t *)ompi_pointer_array_get_item ( 
-	    &(data->hier_llead), i ) ;
-        if ( current->lleaders != NULL ) {
-	    ompi_comm_free ( &(current->llcomm));
-	    free ( current->lleaders );
-	}
-	free ( current );
-    }
-    ompi_pointer_array_remove_all ( &(data->hier_llead));
-    OBJ_DESTRUCT (&(data->hier_llead));
-    free ( data->hier_colorarr );
-    free ( data->hier_llr);
-    free ( data );
-    
-    comm->c_coll_selected_data = NULL;
-    return OMPI_SUCCESS;
-}
-
-int mca_coll_hierarch_comm_unquery ( struct ompi_communicator_t *comm,
-				     struct mca_coll_base_comm_t *data )
-{
-    free ( data->hier_colorarr );
-    free ( data );
     return OMPI_SUCCESS;
 }
 
 
-int mca_coll_hierarch_get_all_lleaders ( int rank, struct mca_coll_base_comm_t *data,
-					  struct mca_coll_hierarch_llead_t * llead, 
-					  int offset )
+
+int mca_coll_hierarch_get_all_lleaders ( int rank, mca_coll_hierarch_module_t *hierarch_module,
+					 struct mca_coll_hierarch_llead_t * llead, 
+					 int offset )
 {
     int i, j, ret=OMPI_SUCCESS;
     int *cntarr=NULL;
     int mycolor;
 
-    cntarr = (int *)calloc (1, sizeof (int)* data->hier_num_lleaders );
+    cntarr = (int *)calloc (1, sizeof (int)* hierarch_module->hier_num_lleaders );
     if ( NULL == cntarr ) {
 	return OMPI_ERR_OUT_OF_RESOURCE;
     }
 
-    llead->lleaders = (int *) malloc (sizeof(int) * data->hier_num_lleaders);
+    llead->lleaders = (int *) malloc (sizeof(int) * hierarch_module->hier_num_lleaders);
     if ( NULL == llead->lleaders ) {
 	ret = OMPI_ERR_OUT_OF_RESOURCE;
         goto exit;
     }
     llead->offset = offset;
 
-    for ( i=0; i < data->hier_num_lleaders; i++ )  {
-    	if ( data->hier_llr[i] == MPI_UNDEFINED ) {
+    for ( i=0; i < hierarch_module->hier_num_lleaders; i++ )  {
+    	if ( MPI_UNDEFINED == hierarch_module->hier_llr[i] ) {
 	    cntarr[i] = 1;
 	    llead->lleaders[i] = MPI_UNDEFINED;
 	}
     }
 
-    for ( i=0; i<data->hier_num_colorarr; i++) {
-	if ( data->hier_colorarr[i] == MPI_UNDEFINED ) {
+    for ( i=0; i<hierarch_module->hier_num_colorarr; i++) {
+	if ( MPI_UNDEFINED == hierarch_module->hier_colorarr[i] ) {
 	    continue;
 	}
-	for ( j=0; j<data->hier_num_lleaders; j++) {	    
+	for ( j=0; j<hierarch_module->hier_num_lleaders; j++) {	    
 	    if ( cntarr[j] >= offset ) {
 		continue;
 	    }
-	    if ( data->hier_colorarr[i] == data->hier_llr[j]) {
+	    if ( hierarch_module->hier_colorarr[i] == hierarch_module->hier_llr[j]) {
 		cntarr[j]++;
 		llead->lleaders[j] = i;
 		break;
@@ -462,15 +370,15 @@ int mca_coll_hierarch_get_all_lleaders ( int rank, struct mca_coll_base_comm_t *
 	}
     }
 
-    mycolor = data->hier_colorarr[rank];
-    if ( mycolor == MPI_UNDEFINED ) {
+    mycolor = hierarch_module->hier_colorarr[rank];
+    if ( MPI_UNDEFINED == mycolor ) {
 	llead->am_lleader = 1;
 	llead->my_lleader = MPI_UNDEFINED;
     }
     else {
 	llead->am_lleader = 0;
-	for ( i=0; i< data->hier_num_lleaders; i++ ) {
-	    if ( data->hier_llr[i] == mycolor ) {
+	for ( i=0; i< hierarch_module->hier_num_lleaders; i++ ) {
+	    if ( hierarch_module->hier_llr[i] == mycolor ) {
 		llead->my_lleader = cntarr[i]-1;
 		if ( llead->lleaders[i] == rank ) {
 		    llead->am_lleader = 1; 
@@ -487,39 +395,39 @@ int mca_coll_hierarch_get_all_lleaders ( int rank, struct mca_coll_base_comm_t *
     return ret;
 }
 
-int mca_coll_hierarch_get_llr ( struct mca_coll_base_comm_t *data )
+int mca_coll_hierarch_get_llr ( mca_coll_hierarch_module_t *hierarch_module )
 {
     int i, j, cnt, found;
     int ncount;
 
-    ncount = mca_coll_hierarch_count_lleaders ( data->hier_num_colorarr, 
-						data->hier_colorarr);
-    data->hier_num_lleaders = ncount;
-    data->hier_llr = (int *) malloc ( data->hier_num_lleaders * sizeof(int));
-    data->hier_max_offset = (int *) calloc ( 1, data->hier_num_lleaders * sizeof(int));
-    if ( ( NULL == data->hier_llr) || ( NULL == data->hier_max_offset )) {
+    ncount = mca_coll_hierarch_count_lleaders ( hierarch_module->hier_num_colorarr, 
+						hierarch_module->hier_colorarr);
+    hierarch_module->hier_num_lleaders = ncount;
+    hierarch_module->hier_llr = (int *) malloc ( hierarch_module->hier_num_lleaders * sizeof(int));
+    hierarch_module->hier_max_offset = (int *) calloc ( 1, hierarch_module->hier_num_lleaders * sizeof(int));
+    if ( ( NULL == hierarch_module->hier_llr) || ( NULL == hierarch_module->hier_max_offset )) {
 	return OMPI_ERR_OUT_OF_RESOURCE;
     }
 
-    data->hier_llr[0] = data->hier_colorarr[0];
-    data->hier_max_offset[0]=1;
-    for ( cnt=1, i=1; i<data->hier_num_colorarr; i++ ) {
-	if ( data->hier_colorarr[i] == MPI_UNDEFINED ) { 
-	    data->hier_llr[cnt]     = data->hier_colorarr[i];
-	    data->hier_max_offset[cnt] = 1;
+    hierarch_module->hier_llr[0] = hierarch_module->hier_colorarr[0];
+    hierarch_module->hier_max_offset[0]=1;
+    for ( cnt=1, i=1; i<hierarch_module->hier_num_colorarr; i++ ) {
+	if ( MPI_UNDEFINED == hierarch_module->hier_colorarr[i] ) { 
+	    hierarch_module->hier_llr[cnt]     = hierarch_module->hier_colorarr[i];
+	    hierarch_module->hier_max_offset[cnt] = 1;
 	    cnt++;
 	    continue;
 	}
 	for ( found=0, j=0; j<cnt; j++ ) {
-	    if ( data->hier_llr[j]  == data->hier_colorarr[i]) {
-		data->hier_max_offset[j]++;
+	    if ( hierarch_module->hier_llr[j]  == hierarch_module->hier_colorarr[i]) {
+		hierarch_module->hier_max_offset[j]++;
 		found = 1;
 		break;
 	    }
 	}
 	if ( !found ) {
-	    data->hier_llr[cnt] = data->hier_colorarr[i];
-	    data->hier_max_offset[cnt]++;
+	    hierarch_module->hier_llr[cnt] = hierarch_module->hier_colorarr[i];
+	    hierarch_module->hier_max_offset[cnt]++;
 	    cnt++;
 	}
     }
@@ -529,28 +437,28 @@ int mca_coll_hierarch_get_llr ( struct mca_coll_base_comm_t *data )
 
 
 struct ompi_communicator_t*  mca_coll_hierarch_get_llcomm (int root, 
-						     struct mca_coll_base_comm_t *data,
-						     int* llroot,
-						     int* lroot) 
+                                                           mca_coll_hierarch_module_t *hierarch_module,
+                                                           int* llroot,
+                                                           int* lroot) 
 {
     struct ompi_communicator_t *llcomm=NULL;
     struct ompi_group_t *llgroup=NULL;
     struct ompi_group_t *group=NULL;
     struct mca_coll_hierarch_llead_t *llead=NULL;
     int found, i, rc, num_llead, offset;
-    int rank = ompi_comm_rank (data->hier_comm);
+    int rank = ompi_comm_rank (hierarch_module->hier_comm);
     
     /* determine what our offset of root is in the colorarr */
     offset = mca_coll_hierarch_get_offset ( root, 
-					    data->hier_num_colorarr, 
-					    data->hier_colorarr );
+					    hierarch_module->hier_num_colorarr, 
+					    hierarch_module->hier_colorarr );
     
-    num_llead = ompi_pointer_array_get_size ( &(data->hier_llead) );
+    num_llead = opal_pointer_array_get_size ( &(hierarch_module->hier_llead) );
     for ( found=0, i=0; i < num_llead; i++ ) {
-        llead = (struct mca_coll_hierarch_llead_t *) ompi_pointer_array_get_item (
-	    &(data->hier_llead), i );
-	if ( llead == NULL ) {
-	  continue;
+        llead = (struct mca_coll_hierarch_llead_t *) opal_pointer_array_get_item (
+                                                                                  &(hierarch_module->hier_llead), i );
+	if ( NULL == llead ) {
+            continue;
 	}
 
 	if (llead->offset == offset ) {
@@ -559,10 +467,10 @@ struct ompi_communicator_t*  mca_coll_hierarch_get_llcomm (int root,
 	}
 #if 0
 	else if () {
-	  /* the offset of root = maxoffset of this color and
-	   * the offset on llead is larger then offset of root.
-	   * then we can also use this llead structure 
-	   */
+            /* the offset of root = maxoffset of this color and
+             * the offset on llead is larger then offset of root.
+             * then we can also use this llead structure 
+             */
 	}
 #endif
     }
@@ -570,23 +478,23 @@ struct ompi_communicator_t*  mca_coll_hierarch_get_llcomm (int root,
     if ( !found ) {
 	/* allocate a new llead element */
 	llead = (struct mca_coll_hierarch_llead_t *) malloc (
-	    sizeof(struct mca_coll_hierarch_llead_t));
+                                                             sizeof(struct mca_coll_hierarch_llead_t));
 	if ( NULL == llead ) {
 	    return NULL;
 	}
 	
 	/* generate the list of lleaders with this offset */
-	mca_coll_hierarch_get_all_lleaders ( rank, data, llead, offset );   
+	mca_coll_hierarch_get_all_lleaders ( rank, hierarch_module, llead, offset );   
 	
 	/* create new lleader subcommunicator */
-	rc = ompi_comm_split ( data->hier_comm, llead->am_lleader, root, &llcomm, 0);
+	rc = ompi_comm_split ( hierarch_module->hier_comm, llead->am_lleader, root, &llcomm, 0);
 	if ( OMPI_SUCCESS != rc ) {
 	    return NULL;
 	}
 	llead->llcomm = llcomm;
 
-	/* Store the new element on the data struct */
-	ompi_pointer_array_add ( &(data->hier_llead), llead);
+	/* Store the new element on the hierarch_module struct */
+	opal_pointer_array_add ( &(hierarch_module->hier_llead), llead);
     }
 
     llcomm = llead->llcomm;
@@ -594,7 +502,7 @@ struct ompi_communicator_t*  mca_coll_hierarch_get_llcomm (int root,
     *llroot = MPI_UNDEFINED;
 
     if ( MPI_COMM_NULL != llcomm ) {
-	rc = ompi_comm_group ( data->hier_comm, &group);
+	rc = ompi_comm_group ( hierarch_module->hier_comm, &group);
 	if ( OMPI_SUCCESS != rc ) {
 	    return NULL;
 	}
@@ -619,6 +527,37 @@ struct ompi_communicator_t*  mca_coll_hierarch_get_llcomm (int root,
 /**********************************************************************/
 /**********************************************************************/
 /**********************************************************************/
+static void
+mca_coll_hierarch_checkfor_sm ( struct ompi_communicator_t *comm, int *color,  int *ncount )
+{
+    int i, size;
+    int lncount=0;
+    struct ompi_proc_t** procs=NULL;
+    struct ompi_proc_t* my_proc=NULL;
+
+
+    *color = -1;
+    size = ompi_comm_size(comm);
+    my_proc = ompi_proc_local();
+    procs = comm->c_local_group->grp_proc_pointers;
+    for ( i = 0 ; i < size ; i++) {
+	if ( procs[i]->proc_name.jobid == my_proc->proc_name.jobid &&
+	     ( (procs[i]->proc_flags & OMPI_PROC_FLAG_LOCAL)) ) {
+	    lncount++;
+	    if ( *color == -1){
+		 *color = i;
+	    }
+	}
+    }
+
+    /* we need to decrease ncount in order to make the other allreduce/allgather 
+       operations work */
+    lncount--;
+    *ncount = lncount;
+    return;
+}
+
+
 /* This function checks how many processes are using the component
    'component_name' for communication and returns this count in 
    'ncount'. Furthermore it returns a 'key', which can be used to split
@@ -640,7 +579,6 @@ mca_coll_hierarch_checkfor_component ( struct ompi_communicator_t *comm,
 {
     ompi_bitmap_t reachable;
     ompi_proc_t **procs=NULL;
-    struct mca_bml_base_endpoint_t **bml_endpoints=NULL;
     struct mca_bml_base_btl_array_t *bml_btl_array=NULL;
     mca_bml_base_btl_t *bml_btl=NULL;
     mca_btl_base_component_t *btl=NULL;
@@ -668,14 +606,8 @@ mca_coll_hierarch_checkfor_component ( struct ompi_communicator_t *comm,
         return;
     }
 
-    bml_endpoints = (struct mca_bml_base_endpoint_t **) malloc ( size * 
-                     sizeof(struct mca_bml_base_endpoint_t*));
-    if ( NULL == bml_endpoints ) {
-        return;
-    }
-
     procs = comm->c_local_group->grp_proc_pointers;
-    rc = mca_bml.bml_add_procs ( size, procs, bml_endpoints,  &reachable );
+    rc = mca_bml.bml_add_procs ( size, procs, &reachable );
     if(OMPI_SUCCESS != rc) {
         return;
     }
@@ -687,10 +619,10 @@ mca_coll_hierarch_checkfor_component ( struct ompi_communicator_t *comm,
         }
 	
         if ( use_rdma ) {
-            bml_btl_array = &(bml_endpoints[i]->btl_rdma);
+            bml_btl_array = &(procs[i]->proc_bml->btl_rdma);
         }
         else {
-            bml_btl_array = &(bml_endpoints[i]->btl_send);
+            bml_btl_array = &(procs[i]->proc_bml->btl_send);
         }
         bml_btl = mca_bml_base_btl_array_get_index ( bml_btl_array, 0 );
         btl = bml_btl->btl->btl_component;
@@ -705,7 +637,7 @@ mca_coll_hierarch_checkfor_component ( struct ompi_communicator_t *comm,
         if (! strcmp (btl->btl_version.mca_component_name, component_name)){
             counter++;
 	    if (i<firstproc ) {
-	      firstproc = i;
+                firstproc = i;
 	    }
 	    continue;
 	}	    
@@ -727,10 +659,6 @@ mca_coll_hierarch_checkfor_component ( struct ompi_communicator_t *comm,
 
     *key = firstproc;
 
-    if ( NULL != bml_endpoints ) {
-        free ( bml_endpoints);
-    }
-
     return;
 }
 
@@ -739,7 +667,7 @@ mca_coll_hierarch_checkfor_component ( struct ompi_communicator_t *comm,
 /********************************************************************************/
 /********************************************************************************/
 
-static void mca_coll_hierarch_dump_struct ( struct mca_coll_base_comm_t *c)
+static void mca_coll_hierarch_dump_struct ( mca_coll_hierarch_module_t *c)
 {
     int i, j;
     int rank;
@@ -751,13 +679,13 @@ static void mca_coll_hierarch_dump_struct ( struct mca_coll_base_comm_t *c)
            rank, c->hier_comm->c_name, c->hier_comm->c_contextid);
 
     printf("%d: No of llead communicators: %d No of lleaders: %d\n", 
-	   rank, ompi_pointer_array_get_size ( &(c->hier_llead)),
+	   rank, opal_pointer_array_get_size ( &(c->hier_llead)),
 	   c->hier_num_lleaders );
 
-    for ( i=0; i < ompi_pointer_array_get_size(&(c->hier_llead)); i++ ) {
-	current = ompi_pointer_array_get_item (&(c->hier_llead), i);
+    for ( i=0; i < opal_pointer_array_get_size(&(c->hier_llead)); i++ ) {
+	current = (mca_coll_hierarch_llead_t*)opal_pointer_array_get_item (&(c->hier_llead), i);
 	if ( current == NULL ) {
-	  continue;
+            continue;
 	}
 
 	printf("%d:  my_leader %d am_leader %d\n", rank,
@@ -771,3 +699,22 @@ static void mca_coll_hierarch_dump_struct ( struct mca_coll_base_comm_t *c)
     return;
 }
 
+int mca_coll_hierarch_ft_event(int state) {
+    if(OPAL_CRS_CHECKPOINT == state) {
+        ;
+    }
+    else if(OPAL_CRS_CONTINUE == state) {
+        ;
+    }
+    else if(OPAL_CRS_RESTART == state) {
+        ;
+    }
+    else if(OPAL_CRS_TERM == state ) {
+        ;
+    }
+    else {
+        ;
+    }
+
+    return OMPI_SUCCESS;
+}
