@@ -2,13 +2,14 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -29,6 +30,12 @@
 #include "ompi/mca/mpool/mpool.h"
 #include "opal/threads/mutex.h" 
 
+#if defined(HAVE_MALLOPT) && defined(M_TRIM_THRESHOLD) && defined(M_MMAP_MAX)
+#define OMPI_MPOOL_BASE_HAVE_LINUX_MALLOPT 1
+#else
+#define OMPI_MPOOL_BASE_HAVE_LINUX_MALLOPT 0
+#endif
+
 #if defined(c_plusplus) || defined(__cplusplus)
 extern "C" {
 #endif
@@ -42,11 +49,11 @@ static inline unsigned int my_log2(unsigned long val) {
     return count > 0 ? count-1: 0;
 }
 static inline void *down_align_addr(void* addr, unsigned int shift) {
-    return (void*) (((unsigned long) addr) & (~(unsigned long) 0) << shift); 
+    return (void*) (((intptr_t) addr) & (~(intptr_t) 0) << shift); 
 }
 
 static inline void *up_align_addr(void*addr, unsigned int shift) { 
-    return (void*) ((((unsigned long) addr) | ~((~(unsigned long) 0) << shift))); 
+    return (void*) ((((intptr_t) addr) | ~((~(intptr_t) 0) << shift))); 
 }
 
 struct mca_mpool_base_selected_module_t {
@@ -58,7 +65,7 @@ struct mca_mpool_base_selected_module_t {
 };
 typedef struct mca_mpool_base_selected_module_t mca_mpool_base_selected_module_t;
 
-OBJ_CLASS_DECLARATION(mca_mpool_base_selected_module_t);
+OMPI_DECLSPEC OBJ_CLASS_DECLARATION(mca_mpool_base_selected_module_t);
 
 /*
  * Data structures for the tree of allocated memory
@@ -84,13 +91,17 @@ OMPI_DECLSPEC int mca_mpool_base_module_destroy(mca_mpool_base_module_t *module)
  */
 OMPI_DECLSPEC extern int mca_mpool_base_output;
 OMPI_DECLSPEC extern opal_list_t mca_mpool_base_components;
-OMPI_DECLSPEC extern opal_list_t mca_mpool_base_modules;
+extern opal_list_t mca_mpool_base_modules;
+OMPI_DECLSPEC extern uint32_t mca_mpool_base_page_size;
+OMPI_DECLSPEC extern uint32_t mca_mpool_base_page_size_log;
 
+/* only used within base -- no need to DECLSPEC */
+extern int mca_mpool_base_used_mem_hooks;
+extern int mca_mpool_base_use_mem_hooks_index;
+extern int mca_mpool_base_disable_mallopt_index;
+    
 #if defined(c_plusplus) || defined(__cplusplus)
 }
 #endif
 
 #endif /* MCA_MEM_BASE_H */
-
-
-
