@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2007      Sun Microsystems, Inc.  All rights reserved.
+ * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -25,16 +26,22 @@
 #include <stdlib.h>
 #endif  /* HAVE_STDLIB_H */
 #include <errno.h>
-#include "opal/util/output.h"
+#include "orte/util/show_help.h"
 #include "opal/mca/base/base.h"
 #include "opal/mca/base/mca_base_param.h"
+
+#include "orte/util/show_help.h"
+#include "orte/util/proc_info.h"
+
 #include "ompi/mca/allocator/base/base.h"
 #include "mpool_sm.h"
 #include "ompi/mca/common/sm/common_sm_mmap.h"
-#include "orte/util/proc_info.h"
-#include "orte/util/sys_info.h"
 #include "ompi/proc/proc.h"
 
+#if OPAL_ENABLE_FT    == 1
+#include "opal/runtime/opal_cr.h"
+#endif
+
 /*
  * Local functions
  */
@@ -49,10 +56,7 @@ mca_mpool_sm_component_t mca_mpool_sm_component = {
          information about the component itself */
 
       {
-        /* Indicate that we are a mpool v1.0.0 component (which also
-           implies a specific MCA version) */
-
-        MCA_MPOOL_BASE_VERSION_1_0_0,
+        MCA_MPOOL_BASE_VERSION_2_0_0,
 
         "sm", /* MCA component name */
         OMPI_MAJOR_VERSION,  /* MCA component major version */
@@ -61,12 +65,9 @@ mca_mpool_sm_component_t mca_mpool_sm_component = {
         mca_mpool_sm_open,  /* component open  */
         mca_mpool_sm_close
       },
-
-      /* Next the MCA v1.0.0 component meta data */
-      
       {
-        /* Whether the component is checkpointable or not */
-        false
+          /* The component is checkpoint ready */
+          MCA_BASE_METADATA_PARAM_CHECKPOINT
       },
 
       mca_mpool_sm_init
@@ -87,7 +88,7 @@ static int mca_mpool_sm_open(void)
     default_max = 512*1024*1024;
     default_min = 128*1024*1024;
     default_peer = 32*1024*1024;
-    
+
     /* register SM component parameters */
     mca_base_param_reg_string(&mca_mpool_sm_component.super.mpool_version,
                               "allocator",
@@ -137,7 +138,17 @@ static int mca_mpool_sm_close( void )
 {
     if( NULL != mca_common_sm_mmap ) {
         if( OMPI_SUCCESS == mca_common_sm_mmap_fini( mca_common_sm_mmap ) ) {
+#if OPAL_ENABLE_FT    == 1
+            /* Only unlink the file if we are *not* restarting
+             * If we are restarting the file will be unlinked at a later time.
+             */
+            if(OPAL_CR_STATUS_RESTART_PRE  != opal_cr_checkpointing_state &&
+               OPAL_CR_STATUS_RESTART_POST != opal_cr_checkpointing_state ) {
+                unlink( mca_common_sm_mmap->map_path );
+            }
+#else
             unlink( mca_common_sm_mmap->map_path );
+#endif
         }
         OBJ_RELEASE( mca_common_sm_mmap );
     }
@@ -149,12 +160,12 @@ static mca_mpool_base_module_t* mca_mpool_sm_init(
 {
     char *file_name;
     int len;
-    mca_mpool_sm_module_t* mpool_module; 
+    mca_mpool_sm_module_t* mpool_module;
     mca_allocator_base_component_t* allocator_component;
     long max_size, min_size, peer_size;
     ompi_proc_t **procs;
     size_t num_all_procs, i, num_local_procs = 0;
-    
+
     /* README: this needs to change if procs in different jobs (even
        spawned ones) are to talk using shared memory */
     procs = ompi_proc_world(&num_all_procs);
@@ -163,9 +174,10 @@ static mca_mpool_base_module_t* mca_mpool_sm_init(
             num_local_procs++;
         }
     }
+    free(procs);
 
     /* parse the max, min and peer sizes, and validate them */
-    /* absolutely necessary to reset errno each time */ 
+    /* absolutely necessary to reset errno each time */
     errno = 0;
     max_size  = strtol(max_size_param, (char **)NULL, 10);
     if (errno == ERANGE) {
@@ -175,7 +187,7 @@ static mca_mpool_base_module_t* mca_mpool_sm_init(
         opal_output(0, "mca_mpool_sm_init: invalid max_size entered. set it to (%ld)", default_max);
         max_size = default_max;
     }
-    
+
     errno = 0;
     min_size  = strtol(min_size_param, (char **)NULL, 10);
     if (errno == ERANGE) {
@@ -243,14 +255,15 @@ static mca_mpool_base_module_t* mca_mpool_sm_init(
         }
     }
 
-    mpool_module = (mca_mpool_sm_module_t*)malloc(sizeof(mca_mpool_sm_module_t)); 
-    mca_mpool_sm_module_init(mpool_module); 
+    mpool_module = (mca_mpool_sm_module_t*)malloc(sizeof(mca_mpool_sm_module_t));
+    mca_mpool_sm_module_init(mpool_module);
 
     /* create initial shared memory mapping */
     len = asprintf( &file_name, "%s"OPAL_PATH_SEP"shared_mem_pool.%s",
                     orte_process_info.job_session_dir,
-                    orte_system_info.nodename );
+                    orte_process_info.nodename );
     if ( 0 > len ) {
+        free(mpool_module);
         return NULL;
     }
     
@@ -264,8 +277,10 @@ static mca_mpool_base_module_t* mca_mpool_sm_init(
                  file_name,sizeof(mca_common_sm_mmap_t), 8 )
              )) 
     {
-        opal_output(0, "mca_mpool_sm_init: unable to create shared memory mapping (%s)", file_name);
+        opal_output(mca_mpool_sm_component.verbose, 
+                    "mca_mpool_sm_init: unable to create shared memory mapping (%s)", file_name);
         free(file_name);
+        free(mpool_module);
         return NULL;
     }
     free(file_name);
@@ -275,7 +290,8 @@ static mca_mpool_base_module_t* mca_mpool_sm_init(
       allocator_component->allocator_init(true,
                                           mca_common_sm_mmap_seg_alloc, NULL, NULL);
     if(NULL == mpool_module->sm_allocator) {
-      opal_output(0, "mca_mpool_sm_init: unable to initialize allocator");
+        opal_output(0, "mca_mpool_sm_init: unable to initialize allocator");
+        free(mpool_module);
         return NULL;
     }
    
