@@ -1,11 +1,11 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
@@ -24,6 +24,7 @@
 #include "ompi/op/op.h"
 #include "ompi/datatype/datatype.h"
 #include "ompi/datatype/datatype_internal.h"
+#include "ompi/memchecker.h"
 
 #if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Accumulate = PMPI_Accumulate
@@ -43,6 +44,12 @@ int MPI_Accumulate(void *origin_addr, int origin_count, MPI_Datatype origin_data
     int rc;
     ompi_win_t *ompi_win = (ompi_win_t*) win;
 
+    MEMCHECKER(
+        memchecker_datatype(origin_datatype);
+        memchecker_datatype(target_datatype);
+        memchecker_call(&opal_memchecker_base_isdefined, origin_addr, origin_count, origin_datatype);
+    );
+
     if (MPI_PARAM_CHECK) {
         rc = OMPI_SUCCESS;
 
@@ -63,8 +70,6 @@ int MPI_Accumulate(void *origin_addr, int origin_count, MPI_Datatype origin_data
             rc = MPI_ERR_RMA_SYNC;
         } else if ( target_disp < 0 ) {
             rc = MPI_ERR_DISP;
-        } else if ( (origin_count < 0) || (target_count < 0) ) {
-            rc = MPI_ERR_COUNT;
         } else {
             OMPI_CHECK_DATATYPE_FOR_ONE_SIDED(rc, origin_datatype, origin_count);
             if (OMPI_SUCCESS == rc) {
@@ -83,23 +88,23 @@ int MPI_Accumulate(void *origin_addr, int origin_count, MPI_Datatype origin_data
                     if (ompi_ddt_is_predefined(origin_datatype)) {
                         origin_check_dt = origin_datatype;
                     } else {
-                        int i, index = -1, num_found = 0;
+                        int i, found_index = -1, num_found = 0;
                         uint64_t mask = 1;
 
                         for (i = 0 ; i < DT_MAX_PREDEFINED ; ++i) {
                             if (origin_datatype->bdt_used & mask) {
                                 num_found++;
-                                index = i;
+                                found_index = i;
                             }
                             mask *= 2;
                         }
-                        if (index < 0 || num_found > 1) {
+                        if (found_index < 0 || num_found > 1) {
                             /* this is an erroneous datatype.  Let
                                ompi_op_is_valid tell the user that */
                             OMPI_ERRHANDLER_RETURN(MPI_ERR_TYPE, win, MPI_ERR_TYPE, FUNC_NAME);
                         } else {
                             origin_check_dt = (ompi_datatype_t*)
-                                ompi_ddt_basicDatatypes[index];
+                                ompi_ddt_basicDatatypes[found_index];
                         }
                     }
 
@@ -116,17 +121,17 @@ int MPI_Accumulate(void *origin_addr, int origin_count, MPI_Datatype origin_data
                     if (ompi_ddt_is_predefined(target_datatype)) {
                         op_check_dt = target_datatype;
                     } else {
-                        int i, index = -1, num_found = 0;
+                        int i, found_index = -1, num_found = 0;
                         uint64_t mask = 1;
 
                         for (i = 0 ; i < DT_MAX_PREDEFINED ; ++i) {
                             if (target_datatype->bdt_used & mask) {
                                 num_found++;
-                                index = i;
+                                found_index = i;
                             }
                             mask *= 2;
                         }
-                        if (index < 0 || num_found > 1) {
+                        if (found_index < 0 || num_found > 1) {
                             /* this is an erroneous datatype.  Let
                                ompi_op_is_valid tell the user that */
                             OMPI_ERRHANDLER_RETURN(MPI_ERR_TYPE, win, MPI_ERR_TYPE, FUNC_NAME);
@@ -136,7 +141,7 @@ int MPI_Accumulate(void *origin_addr, int origin_count, MPI_Datatype origin_data
                                op?  Unfortunately have to cast away
                                constness... */
                             op_check_dt = (ompi_datatype_t*)
-                                ompi_ddt_basicDatatypes[index];
+                                ompi_ddt_basicDatatypes[found_index];
                         }
                     }
 
@@ -181,17 +186,17 @@ int MPI_Accumulate(void *origin_addr, int origin_count, MPI_Datatype origin_data
             if (ompi_ddt_is_predefined(target_datatype)) {
                 op_check_dt = target_datatype;
             } else {
-                int i, index = -1, num_found = 0;
+                int i, found_index = -1, num_found = 0;
                 uint64_t mask = 1;
 
                 for (i = 0 ; i < DT_MAX_PREDEFINED ; ++i) {
                     if (target_datatype->bdt_used & mask) {
                         num_found++;
-                        index = i;
+                        found_index = i;
                     }
                     mask *= 2;
                 }
-                if (index < 0 || num_found > 1) {
+                if (found_index < 0 || num_found > 1) {
                     /* this is an erroneous datatype.  Let
                        ompi_op_is_valid tell the user that */
                     op_check_dt = target_datatype;
@@ -201,7 +206,7 @@ int MPI_Accumulate(void *origin_addr, int origin_count, MPI_Datatype origin_data
                        op?  Unfortunately have to cast away
                        constness... */
                     op_check_dt = (ompi_datatype_t*)
-                        ompi_ddt_basicDatatypes[index];
+                        ompi_ddt_basicDatatypes[found_index];
                 }
             }
             if (!ompi_op_is_valid(op, op_check_dt, &msg, FUNC_NAME)) {
@@ -212,7 +217,11 @@ int MPI_Accumulate(void *origin_addr, int origin_count, MPI_Datatype origin_data
         }
     }
 
-    if (MPI_PROC_NULL == target_rank) return MPI_SUCCESS;
+    if (MPI_PROC_NULL == target_rank) {
+        return MPI_SUCCESS;
+    }
+
+    OPAL_CR_ENTER_LIBRARY();
 
     rc = ompi_win->w_osc_module->osc_accumulate(origin_addr, 
                                                 origin_count,
