@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2007      Cisco, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -38,6 +39,7 @@ static const char FUNC_NAME[] = "MPI_Alloc_mem";
 
 int MPI_Alloc_mem(MPI_Aint size, MPI_Info info, void *baseptr)
 {
+
     if (MPI_PARAM_CHECK) {
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
         if (size < 0 || NULL == baseptr) {
@@ -49,14 +51,28 @@ int MPI_Alloc_mem(MPI_Aint size, MPI_Info info, void *baseptr)
         }
     }
     
+    /* Per these threads:
+
+         http://www.open-mpi.org/community/lists/devel/2007/07/1977.php 
+         http://www.open-mpi.org/community/lists/devel/2007/07/1979.php         
+
+       If you call MPI_ALLOC_MEM with a size of 0, you get NULL
+       back .*/
+    if (0 == size) {
+        *((void **) baseptr) = NULL;
+        return MPI_SUCCESS;
+    }
+
+    OPAL_CR_ENTER_LIBRARY();
+
     *((void **) baseptr) = mca_mpool_base_alloc((size_t) size, info);
+    OPAL_CR_EXIT_LIBRARY();
     if (NULL == *((void **) baseptr)) {
         return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_NO_MEM, 
                                       FUNC_NAME);
     }
 
     /* All done */
-
     return MPI_SUCCESS;
 }
 
