@@ -1,11 +1,11 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
@@ -19,9 +19,11 @@
 #include "ompi_config.h"
 #include <stdio.h>
 
-#include "opal/util/show_help.h"
+#include "orte/util/show_help.h"
 #include "ompi/mpi/c/bindings.h"
 #include "ompi/info/info.h"
+#include "ompi/mca/dpm/dpm.h"
+#include "ompi/memchecker.h"
 
 #if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Comm_spawn_multiple = PMPI_Comm_spawn_multiple
@@ -39,13 +41,16 @@ int MPI_Comm_spawn_multiple(int count, char **array_of_commands, char ***array_o
                             int root, MPI_Comm comm, MPI_Comm *intercomm,
                             int *array_of_errcodes) 
 {
-    int i=0, rc=0, rank=0;
+    int i=0, rc=0, rank=0, flag;
     ompi_communicator_t *newcomp=NULL;
-    int send_first=0; /* they are contacting us first */
+    bool send_first=false; /* they are contacting us first */
     char port_name[MPI_MAX_PORT_NAME];
-    char *tmp_port;
-    orte_rml_tag_t tag = 0;
+    bool non_mpi, cumulative = false;
 
+    MEMCHECKER(
+        memchecker_comm(comm);
+    );
+    
     if ( MPI_PARAM_CHECK ) {
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
 
@@ -72,6 +77,24 @@ int MPI_Comm_spawn_multiple(int count, char **array_of_commands, char ***array_o
                 return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_INFO,
                                               FUNC_NAME);
             }
+            /* If ompi_non_mpi is set to true on any info, it must be
+               set to true on all of them.  Note that not setting
+               ompi_non_mpi is the same as setting it to false. */
+            ompi_info_get_bool(array_of_info[i], "ompi_non_mpi", &non_mpi,
+                               &flag);
+            if (flag && 0 == i) {
+                /* If this is the first info, save its ompi_non_mpi value */
+                cumulative = non_mpi;
+            } else if (!flag) {
+                non_mpi = false;
+            }
+            /* If this info's effective value doesn't agree with the
+               rest of them, error */
+            if (cumulative != non_mpi) {
+                return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD,
+                                              MPI_ERR_INFO,
+                                              FUNC_NAME);
+            }
         }
     }
    
@@ -101,21 +124,50 @@ int MPI_Comm_spawn_multiple(int count, char **array_of_commands, char ***array_o
         }
     }
 
-    if ( rank == root ) {
-        /* Open a port. The port_name is passed as an environment variable
-         * to the children. */
-        ompi_open_port (port_name);
-        ompi_comm_start_processes(count, array_of_commands,
-                                  array_of_argv, array_of_maxprocs,
-                                  array_of_info, port_name);
-        tmp_port = ompi_parse_port (port_name, &tag);
-        free(tmp_port);
+    if (MPI_INFO_NULL == array_of_info[0]) {
+        non_mpi = false;
+    } else {
+        ompi_info_get_bool(array_of_info[0], "ompi_non_mpi", &non_mpi,
+                           &flag);
+        if (!flag) {
+            non_mpi = false;
+        }
     }
 
-    rc = ompi_comm_connect_accept (comm, root, NULL, send_first, &newcomp, tag);
+    /* initialize the port name to avoid problems */
+    memset(port_name, 0, MPI_MAX_PORT_NAME);
+    
+    OPAL_CR_ENTER_LIBRARY();
 
-    /* close the port again. Nothing has to be done for that at the moment.*/
+    if ( rank == root ) {
+        if (!non_mpi) {
+            /* Open a port. The port_name is passed as an environment
+               variable to the children. */
+            if (OMPI_SUCCESS != (rc = ompi_dpm.open_port (port_name, OMPI_RML_TAG_INVALID))) {
+                goto error;
+            }
+        }
+        if (OMPI_SUCCESS != (rc = ompi_dpm.spawn(count, array_of_commands,
+                                                 array_of_argv, array_of_maxprocs,
+                                                 array_of_info, port_name))) {
+            goto error;
+        }
+    }
 
+    if (non_mpi) {
+        newcomp = MPI_COMM_NULL;
+    } else {
+        rc = ompi_dpm.connect_accept (comm, root, port_name, send_first, &newcomp);
+    }
+
+error:
+    OPAL_CR_EXIT_LIBRARY();
+    
+    /* close the port */
+    if (rank == root && !non_mpi) {
+        ompi_dpm.close_port(port_name);
+    }
+    
     /* set array of errorcodes */
     if (MPI_ERRCODES_IGNORE != array_of_errcodes) {
         for ( i=0; i < newcomp->c_remote_group->grp_proc_count; i++ ) {
