@@ -1,11 +1,11 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2006 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
@@ -18,6 +18,7 @@
 
 #include "ompi_config.h"
 
+#include "orte/util/show_help.h"
 #include "ompi/mpi/c/bindings.h"
 #include "ompi/constants.h"
 
@@ -36,7 +37,12 @@ int MPI_Init_thread(int *argc, char ***argv, int required,
                     int *provided) 
 {
     int err;
-    MPI_Comm null = NULL;
+
+    if ( MPI_PARAM_CHECK ) {
+        if (required < MPI_THREAD_SINGLE || required > MPI_THREAD_MULTIPLE) {
+            ompi_mpi_errors_are_fatal_comm_handler(NULL, NULL, FUNC_NAME);
+        }
+    }
 
     /*
      *   A thread compliant MPI implementation will be able to return provided
@@ -52,13 +58,22 @@ int MPI_Init_thread(int *argc, char ***argv, int required,
     /* Ensure that we were not already initialized or finalized */
 
     if (ompi_mpi_finalized) {
-        /* JMS show_help */
-        return OMPI_ERRHANDLER_INVOKE(null, MPI_ERR_OTHER, FUNC_NAME);
+        if (0 == ompi_comm_rank(MPI_COMM_WORLD)) {
+            orte_show_help("help-mpi-api.txt", "mpi-function-after-finalize",
+                           true, FUNC_NAME);
+        }
+        return ompi_errhandler_invoke(NULL, NULL, OMPI_ERRHANDLER_TYPE_COMM, 
+                                      MPI_ERR_OTHER, FUNC_NAME);
     } else if (ompi_mpi_initialized) {
-        /* JMS show_help */
+        if (0 == ompi_comm_rank(MPI_COMM_WORLD)) {
+            orte_show_help("help-mpi-api.txt", "mpi-initialize-twice",
+                           true, FUNC_NAME);
+        }
         return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_OTHER, FUNC_NAME);
     }
 
+    OPAL_CR_ENTER_LIBRARY();
+
     /* Call the back-end initialization function (we need to put as
        little in this function as possible so that if it's profiled, we
        don't lose anything) */
@@ -68,6 +83,18 @@ int MPI_Init_thread(int *argc, char ***argv, int required,
     } else {
         err = ompi_mpi_init(0, NULL, required, provided);
     }
-    
-    OMPI_ERRHANDLER_RETURN(err, null, err, FUNC_NAME);
+
+    OPAL_CR_EXIT_LIBRARY();
+
+    /* Since we don't have a communicator to invoke an errorhandler on
+       here, don't use the fancy-schmancy ERRHANDLER macros; they're
+       really designed for real communicator objects.  Just use the
+       back-end function directly. */
+
+    if (MPI_SUCCESS != err) {
+        return ompi_errhandler_invoke(NULL, NULL, OMPI_ERRHANDLER_TYPE_COMM,
+                                      err < 0 ? ompi_errcode_get_mpi_code(err) :
+                                      err, FUNC_NAME);
+    }
+    return MPI_SUCCESS;
 }
