@@ -1,11 +1,11 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
@@ -22,6 +22,7 @@
 
 #include "ompi/mpi/c/bindings.h"
 #include "ompi/datatype/datatype.h"
+#include "ompi/memchecker.h"
 
 #if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Type_create_darray = PMPI_Type_create_darray
@@ -60,6 +61,10 @@ int MPI_Type_create_darray(int size,
     int i, start_loop, end_loop, step;
     int *coords = NULL, rc = OMPI_SUCCESS;
 
+    MEMCHECKER(
+        memchecker_datatype(oldtype);
+    );
+    
     if (MPI_PARAM_CHECK) {
         int prod_psize = 1;
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
@@ -96,12 +101,15 @@ int MPI_Type_create_darray(int size,
             return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_ARG, FUNC_NAME);
     }
 
+    OPAL_CR_ENTER_LIBRARY();
+
     /* speedy corner case */
     if (ndims < 1) {
         /* Don't just return MPI_DATATYPE_NULL as that can't be
            MPI_TYPE_FREE()ed, and that seems bad */
         *newtype = ompi_ddt_create(0);
         ompi_ddt_add(*newtype, &ompi_mpi_datatype_null, 0, 0, 0);
+        OPAL_CR_EXIT_LIBRARY();
         return MPI_SUCCESS;
     }
 
@@ -137,7 +145,7 @@ int MPI_Type_create_darray(int size,
 
     /* Build up array */
     for (i = start_loop ; i != end_loop; i += step) {
-        int nprocs, rank;
+        int nprocs, tmp_rank;
 
         switch(distrib_array[i]) {
         case MPI_DISTRIBUTE_BLOCK:
@@ -153,12 +161,12 @@ int MPI_Type_create_darray(int size,
         case MPI_DISTRIBUTE_NONE:
             /* treat it as a block distribution on 1 process */
             if (order == MPI_ORDER_C) {
-                nprocs = psize_array[i]; rank = coords[i];
+                nprocs = psize_array[i]; tmp_rank = coords[i];
             } else {
-                nprocs = 1; rank = 0;
+                nprocs = 1; tmp_rank = 0;
             }
 
-            rc = block(gsize_array, i, ndims, nprocs, rank,
+            rc = block(gsize_array, i, ndims, nprocs, tmp_rank,
                        MPI_DISTRIBUTE_DFLT_DARG, order, orig_extent, 
                        lastType, newtype, st_offsets+i); 
             break;
@@ -222,6 +230,8 @@ int MPI_Type_create_darray(int size,
     if (NULL != st_offsets) free(st_offsets);
     if (NULL != coords) free(coords);
 
+    OPAL_CR_EXIT_LIBRARY();
+
     OMPI_ERRHANDLER_RETURN(rc, MPI_COMM_WORLD, rc, FUNC_NAME);
 }
 
