@@ -10,7 +10,7 @@
 //                         University of Stuttgart.  All rights reserved.
 // Copyright (c) 2004-2005 The Regents of the University of California.
 //                         All rights reserved.
-// Copyright (c) 2007      Cisco Systems, Inc.  All rights reserved.
+// Copyright (c) 2007-2008 Cisco Systems, Inc.  All rights reserved.
 // $COPYRIGHT$
 // 
 // Additional copyrights may follow
@@ -537,19 +537,21 @@ MPI::Comm::Abort(int errorcode)
 inline MPI::Errhandler
 MPI::Comm::Get_errhandler() const
 {
-  return *my_errhandler;
+    MPI_Errhandler errhandler;
+    MPI_Comm_get_errhandler(mpi_comm, &errhandler);
+    return errhandler;
 }
 
-inline MPI::Errhandler
-MPI::Comm::Create_errhandler(MPI::Comm::_MPI2CPP_ERRHANDLERFN_* function)
+inline void
+MPI::Comm::Set_errhandler(const MPI::Errhandler& errhandler) const
 {
-  MPI_Errhandler errhandler;
-  // $%%@#%# AIX/POE 2.3.0.0 makes us put in this cast here
-  (void)MPI_Comm_create_errhandler((MPI_Handler_function*) ompi_mpi_cxx_comm_errhandler_intercept, 
-			      &errhandler);
-  MPI::Errhandler temp(errhandler);
-  temp.comm_handler_fn = (void(*)(MPI::Comm&, int*, ...))function;
-  return temp;
+    (void)MPI_Comm_set_errhandler(mpi_comm, errhandler);
+}
+
+inline void 
+MPI::Comm::Call_errhandler(int errorcode) const
+{
+  (void) MPI_Comm_call_errhandler(mpi_comm, errorcode);
 }
 
 // 1) original Create_keyval that takes the first 2 arguments as C++
@@ -560,9 +562,11 @@ MPI::Comm::Create_keyval(MPI::Comm::Copy_attr_function* comm_copy_attr_fn,
                          void* extra_state)
 {
     // Back-end function does the heavy lifting
-    return do_create_keyval(NULL, NULL, 
-                            comm_copy_attr_fn, comm_delete_attr_fn,
-                            extra_state);
+    int ret, keyval;
+    ret = do_create_keyval(NULL, NULL, 
+                           comm_copy_attr_fn, comm_delete_attr_fn,
+                           extra_state, keyval);
+    return (MPI_SUCCESS == ret) ? keyval : ret;
 }
 
 // 2) overload Create_keyval to take the first 2 arguments as C
@@ -573,9 +577,11 @@ MPI::Comm::Create_keyval(MPI_Comm_copy_attr_function* comm_copy_attr_fn,
                          void* extra_state)
 {
     // Back-end function does the heavy lifting
-    return do_create_keyval(comm_copy_attr_fn, comm_delete_attr_fn,
-                            NULL, NULL,
-                            extra_state);
+    int ret, keyval;
+    ret = do_create_keyval(comm_copy_attr_fn, comm_delete_attr_fn,
+                           NULL, NULL,
+                           extra_state, keyval);
+    return (MPI_SUCCESS == ret) ? keyval : ret;
 }
 
 // 3) overload Create_keyval to take the first 2 arguments as C++ & C
@@ -586,9 +592,11 @@ MPI::Comm::Create_keyval(MPI::Comm::Copy_attr_function* comm_copy_attr_fn,
                          void* extra_state)
 {
     // Back-end function does the heavy lifting
-    return do_create_keyval(NULL, comm_delete_attr_fn,
-                            comm_copy_attr_fn, NULL,
-                            extra_state);
+    int ret, keyval;
+    ret = do_create_keyval(NULL, comm_delete_attr_fn,
+                           comm_copy_attr_fn, NULL,
+                           extra_state, keyval);
+    return (MPI_SUCCESS == ret) ? keyval : ret;
 }
 
 // 4) overload Create_keyval to take the first 2 arguments as C & C++
@@ -599,9 +607,11 @@ MPI::Comm::Create_keyval(MPI_Comm_copy_attr_function* comm_copy_attr_fn,
                          void* extra_state)
 {
     // Back-end function does the heavy lifting
-    return do_create_keyval(comm_copy_attr_fn, NULL,
-                            NULL, comm_delete_attr_fn,
-                            extra_state);
+    int ret, keyval;
+    ret = do_create_keyval(comm_copy_attr_fn, NULL,
+                           NULL, comm_delete_attr_fn,
+                           extra_state, keyval);
+    return (MPI_SUCCESS == ret) ? keyval : ret;
 }
 
 inline void
@@ -635,42 +645,8 @@ MPI::Comm::NULL_COPY_FN(const MPI::Comm& oldcomm, int comm_keyval,
 			       void* extra_state, void* attribute_val_in,
 			       void* attribute_val_out, bool& flag)
 {
-#if SIZEOF_BOOL != SIZEOF_INT
-  int f = (int)flag;
-  int ret;
-  if (MPI_NULL_COPY_FN != 0) {
-
-    // Portland pgCC 5.0-2 has a bug that we have to workaround here.
-    // MPI_NULL_COPY_FN is actually a #define for ((MPI_Copy_function
-    // *) 0).  If we try to invoke it, such as:
-    //   ret = MPI_NULL_COPY_FN(...);
-    // the preprocessor will resolve this to:
-    //   ret = ((MPI_Copy_function *) 0)(...);
-    // which should be fine.  But unfortunately, pgCC 5.0-2 makes this
-    // into a real symbol that will refuse to link.  The workaround is
-    // to assign this into a temp variable and then invoke through the
-    // function pointer.  This shouldn't be necessary.  :-(
-
-    MPI_Copy_function *stupid_compiler = MPI_NULL_COPY_FN;
-    ret = stupid_compiler(oldcomm, comm_keyval, extra_state, attribute_val_in,
-                          attribute_val_out, &f);
-    flag = OPAL_INT_TO_BOOL(f);
-  } else {
-    ret = MPI_SUCCESS;
-    flag = true;
-  }
-  return ret;
-#else
-  if (MPI_NULL_COPY_FN != 0) {
-
-    // See note above.
-
-    MPI_Copy_function *stupid_compiler = MPI_NULL_COPY_FN;
-    return stupid_compiler(oldcomm, comm_keyval, extra_state, 
-                           attribute_val_in, attribute_val_out, (int*)&flag);
-  } else
+    flag = false;
     return MPI_SUCCESS;
-#endif
 }
 
 inline int
@@ -678,7 +654,7 @@ MPI::Comm::DUP_FN(const MPI::Comm& oldcomm, int comm_keyval,
 			 void* extra_state, void* attribute_val_in,
 			 void* attribute_val_out, bool& flag)
 {
-#if SIZEOF_BOOL != SIZEOF_INT
+#if OMPI_SIZEOF_BOOL != OMPI_SIZEOF_INT
   int f = (int)flag;
   int ret;
   ret = MPI_DUP_FN(oldcomm, comm_keyval, extra_state, attribute_val_in,
@@ -695,13 +671,6 @@ inline int
 MPI::Comm::NULL_DELETE_FN(MPI::Comm& comm, int comm_keyval, void* attribute_val,
 				 void* extra_state)
 {
-  if (MPI_NULL_DELETE_FN != 0) {
-
-    // See note in MPI_NULL_COPY_FN.
-
-    MPI_Delete_function *stupid_compiler = MPI_NULL_DELETE_FN;
-    return stupid_compiler(comm, comm_keyval, attribute_val, extra_state);
-  } else
     return MPI_SUCCESS;
 }
 
