@@ -10,32 +10,30 @@
 //                         University of Stuttgart.  All rights reserved.
 // Copyright (c) 2004-2005 The Regents of the University of California.
 //                         All rights reserved.
+// Copyright (c) 2007-2008 Cisco Systems, Inc.  All rights reserved.
+// Copyright (c) 2007      Sun Microsystems, Inc.  All rights reserved.
 // $COPYRIGHT$
 // 
 // Additional copyrights may follow
 // 
 // $HEADER$
 
-//
-
-#include <stdio.h>
-static const int ompi_stdio_seek_set = SEEK_SET;
-static const int ompi_stdio_seek_cur = SEEK_CUR;
-static const int ompi_stdio_seek_end = SEEK_END;
-
 #include "mpicxx.h"
 
-/* Need to include ompi_config.h after mpicxx.h... */
+/* Need to include ompi_config.h after mpicxx.h so that we get
+   SEEK_SET and friends right */
 #include "ompi_config.h"
 
-#include "ompi/errhandler/errhandler.h"
-
-#if OMPI_WANT_MPI_CXX_SEEK
-
-const int SEEK_SET = ompi_stdio_seek_set;
-const int SEEK_CUR = ompi_stdio_seek_cur;
-const int SEEK_END = ompi_stdio_seek_end;
+#if OMPI_CXX_USE_PRAGMA_IDENT
+#pragma ident OMPI_IDENT_STRING
+#elif OMPI_CXX_USE_IDENT
+#ident OMPI_IDENT_STRING
 #endif
+namespace MPI {
+    const char ompi_libcxx_version_string[] = OMPI_IDENT_STRING;
+}
+
+#include "ompi/errhandler/errhandler.h"
 
 namespace MPI {
 
@@ -44,8 +42,8 @@ int mpi_errno = MPI_SUCCESS;
 #endif
 
 
-const void* BOTTOM = (void*) MPI_BOTTOM;
-const void* IN_PLACE = (void*) MPI_IN_PLACE;
+void* const BOTTOM = (void*) MPI_BOTTOM;
+void* const IN_PLACE = (void*) MPI_IN_PLACE;
 
 // return  codes
 const int SUCCESS = MPI_SUCCESS;
@@ -248,7 +246,9 @@ const Datatype     DATATYPE_NULL = MPI_DATATYPE_NULL;
 Request      REQUEST_NULL = MPI_REQUEST_NULL;
 const Op           OP_NULL = MPI_OP_NULL;
 const Errhandler   ERRHANDLER_NULL;  
+#if OMPI_PROVIDE_MPI_FILE_INTERFACE
 const File FILE_NULL = MPI_FILE_NULL;
+#endif
 
 // constants specifying empty or ignored input
 const char**       ARGV_NULL = (const char**) MPI_ARGV_NULL;
@@ -262,6 +262,7 @@ const int GRAPH = MPI_GRAPH;
 const int CART = MPI_CART;
 
 // MPI-2 IO
+#if OMPI_PROVIDE_MPI_FILE_INTERFACE
 const int MODE_CREATE = MPI_MODE_CREATE;
 const int MODE_RDONLY = MPI_MODE_RDONLY;
 const int MODE_WRONLY = MPI_MODE_WRONLY;
@@ -281,6 +282,7 @@ const int SEEK_END = MPI_SEEK_END;
 #endif
 
 const int MAX_DATAREP_STRING = MPI_MAX_DATAREP_STRING;
+#endif
 
 // one-sided constants
 const int MODE_NOCHECK = MPI_MODE_NOCHECK;
@@ -296,4 +298,4 @@ const int LOCK_SHARED = MPI_LOCK_SHARED;
 const Datatype UB(MPI_UB);
 const Datatype LB(MPI_LB);
 
-}; /* namespace MPI */
+} /* namespace MPI */
