@@ -9,6 +9,9 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2006      Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
+ * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -28,6 +31,7 @@
 #include "opal/util/output.h"
 #include "opal/util/show_help.h"
 #include "opal/mca/base/mca_base_param.h"
+#include "opal/threads/mutex.h"
 
 int opal_register_params(void)
 {
@@ -69,5 +73,23 @@ int opal_register_params(void)
         free(string);
     }
 
+#if OMPI_ENABLE_DEBUG
+
+
+    mca_base_param_reg_int_name("opal", "progress_debug", 
+                                "Set to non-zero to debug progress engine features",
+                                false, false, 0, NULL);
+
+    {
+        int value;
+        mca_base_param_reg_int_name("opal", "debug_locks",
+                                    "Debug mutex usage within Open MPI.  On a "
+                                    "non-threaded build, this enables integer counters and "
+                                    "warning messages when double-locks are detected.",
+                                    false, false, 0, &value);
+        if (value) opal_mutex_check_locks = true;
+    }
+#endif
+
     return OPAL_SUCCESS;
 }
