@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -9,6 +9,10 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2007      Los Alamos National Security, LLC.
+ *                         All rights reserved. 
+ * Copyright (c) 2007      Voltaire. All rights reserved.
+ *
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -31,9 +35,8 @@
 #include <sys/types.h>
 #endif
 
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
+BEGIN_C_DECLS
+
   /**
    * Append a string (by value) to an new or existing NULL-terminated
    * argv array.
@@ -63,7 +66,7 @@ extern "C" {
    * value into the argv array; there is no need to keep the original
    * string (i.e., the arg parameter) after invoking this function.
    */
-OPAL_DECLSPEC  int opal_argv_append(int *argc, char ***argv, const char *arg);
+OPAL_DECLSPEC  int opal_argv_append(int *argc, char ***argv, const char *arg) __opal_attribute_nonnull__(1) __opal_attribute_nonnull__(3);
 
   /**
    * Append to an argv-style array, but ignore the size of the array.
@@ -98,7 +101,8 @@ OPAL_DECLSPEC  int opal_argv_append_nosize(char ***argv, const char *arg);
 OPAL_DECLSPEC  void opal_argv_free(char **argv);
   
   /**
-   * Split a string into a NULL-terminated argv array.
+   * Split a string into a NULL-terminated argv array. Do not include empty 
+   * strings in result array.
    *
    * @param src_string Input string.
    * @param delimiter Delimiter character.
@@ -111,7 +115,24 @@ OPAL_DECLSPEC  void opal_argv_free(char **argv);
    * argument (i.e., it can be freed after calling this function
    * without invalidating the output argv).
    */
-OPAL_DECLSPEC  char **opal_argv_split(const char *src_string, int delimiter);
+OPAL_DECLSPEC  char **opal_argv_split(const char *src_string, int delimiter) __opal_attribute_warn_unused_result__;
+
+  /**
+   * Split a string into a NULL-terminated argv array. Include empty 
+   * strings in result array.
+   *
+   * @param src_string Input string.
+   * @param delimiter Delimiter character.
+   *
+   * @retval argv pointer to new argv array on success
+   * @retval NULL on error
+   *
+   * All strings are insertted into the argv array by value; the
+   * newly-allocated array makes no references to the src_string
+   * argument (i.e., it can be freed after calling this function
+   * without invalidating the output argv).
+   */
+OPAL_DECLSPEC  char **opal_argv_split_with_empty(const char *src_string, int delimiter) __opal_attribute_warn_unused_result__;
 
   /**
    * Return the length of a NULL-terminated argv array.
@@ -141,7 +162,9 @@ OPAL_DECLSPEC  int opal_argv_count(char **argv);
    *
    * It is the callers responsibility to free the returned string.
    */
-OPAL_DECLSPEC  char *opal_argv_join(char **argv, int delimiter);
+OPAL_DECLSPEC  char *opal_argv_join(char **argv, int delimiter) __opal_attribute_malloc__;
+
+OPAL_DECLSPEC char *opal_argv_join_range(char **argv, size_t start, size_t end, int delimiter) __opal_attribute_malloc__;
 
   /**
    * Return the number of bytes consumed by an argv array.
@@ -166,7 +189,7 @@ OPAL_DECLSPEC  size_t opal_argv_len(char **argv);
    * Specifically, the output argv will be an array of the same length
    * as the input argv, and strcmp(argv_in[i], argv_out[i]) will be 0.
    */
-OPAL_DECLSPEC  char **opal_argv_copy(char **argv);
+OPAL_DECLSPEC  char **opal_argv_copy(char **argv) __opal_attribute_malloc__;
 
     /**
      * Delete one or more tokens from the middle of an argv.
@@ -204,7 +227,7 @@ OPAL_DECLSPEC  int opal_argv_delete(int *argc, char ***argv,
      * @param source The argv to copy tokens from
      *
      * @retval OPAL_SUCCESS upon success
-     * @retval OMPI_BAD_PARAM if any parameters are non-sensical
+     * @retval OPAL_BAD_PARAM if any parameters are non-sensical
      *
      * This function takes one arg and inserts it in the middle of
      * another.  The first token in source will be insertted at index
@@ -218,8 +241,7 @@ OPAL_DECLSPEC  int opal_argv_delete(int *argc, char ***argv,
      * target).
      */
 OPAL_DECLSPEC  int opal_argv_insert(char ***target, int start, char **source);
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
+
+END_C_DECLS
 
 #endif /* OPAL_ARGV_H */
