@@ -1,14 +1,15 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2008 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -23,6 +24,7 @@
  */
 
 #include "orte_config.h"
+#include "orte/constants.h"
 
 #include <stdlib.h>
 #ifdef HAVE_UNISTD_H
@@ -53,58 +55,66 @@
 #include <libutil.h>
 #endif
 
-#include "orte/mca/iof/base/iof_base_setup.h"
-
-#include "orte/orte_constants.h"
-#include "opal/util/output.h"
 #include "opal/util/opal_pty.h"
+#include "opal/util/opal_environ.h"
+
+#include "orte/util/show_help.h"
 #include "orte/mca/errmgr/errmgr.h"
+
 #include "orte/mca/iof/iof.h"
-#include "orte/mca/ns/ns.h"
+#include "orte/mca/iof/base/iof_base_setup.h"
 
 int
 orte_iof_base_setup_prefork(orte_iof_base_io_conf_t *opts)
 {
-    int ret;
-
-    /* first check to make sure we can do ptys */
-#if !OMPI_ENABLE_PTY_SUPPORT
-    opts->usepty = 0;
-#endif
+    int ret = -1;
 
     fflush(stdout);
 
+    /* first check to make sure we can do ptys */
 #if OMPI_ENABLE_PTY_SUPPORT
     if (opts->usepty) {
+        /**
+         * It has been reported that on MAC OS X 10.4 and prior one cannot
+         * safely close the writing side of a pty before completly reading
+         * all data inside.
+         * There seems to be two issues: first all pending data is
+         * discarded, and second it randomly generate kernel panics.
+         * Apparently this issue was fixed in 10.5 so by now we use the
+         * pty exactly as we use the pipes.
+         * This comment is here as a reminder.
+         */
         ret = opal_openpty(&(opts->p_stdout[0]), &(opts->p_stdout[1]),
                            (char*)NULL, (struct termios*)NULL, (struct winsize*)NULL);
-    } else {
-        ret = -1;
     }
 #else
-    ret = -1;
+    opts->usepty = 0;
 #endif
 
 #if defined(__WINDOWS__)
     /* Windows doesn't have a 'pipe' function.
      * So we need to do something a bit more complex */
-    /* XXX Implement this properly JJH
+    /* 
      * http://www-106.ibm.com/developerworks/linux/library/l-rt4/?open&t=grl,l=252,p=pipes
      */
 #else
     if (ret < 0) {
         if (pipe(opts->p_stdout) < 0) {
-            ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
-            return ORTE_ERR_OUT_OF_RESOURCE;
+            ORTE_ERROR_LOG(ORTE_ERR_SYS_LIMITS_PIPES);
+            return ORTE_ERR_SYS_LIMITS_PIPES;
         }
     }
-        if (pipe(opts->p_stdin) < 0) {
-            ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
-            return ORTE_ERR_OUT_OF_RESOURCE;
-        }
+    if (pipe(opts->p_stdin) < 0) {
+        ORTE_ERROR_LOG(ORTE_ERR_SYS_LIMITS_PIPES);
+        return ORTE_ERR_SYS_LIMITS_PIPES;
+    }
     if (pipe(opts->p_stderr) < 0) {
-        ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
-        return ORTE_ERR_OUT_OF_RESOURCE;
+        ORTE_ERROR_LOG(ORTE_ERR_SYS_LIMITS_PIPES);
+        return ORTE_ERR_SYS_LIMITS_PIPES;
+    }
+    if (pipe(opts->p_internal) < 0) {
+        ORTE_ERROR_LOG(ORTE_ERR_SYS_LIMITS_PIPES);
+        return ORTE_ERR_SYS_LIMITS_PIPES;
     }
 #endif
 
@@ -113,22 +123,22 @@ orte_iof_base_setup_prefork(orte_iof_base_io_conf_t *opts)
 
 
 int
-orte_iof_base_setup_child(orte_iof_base_io_conf_t *opts)
+orte_iof_base_setup_child(orte_iof_base_io_conf_t *opts, char ***env)
 {
     int ret;
+    char *str;
 
-    if (!opts->usepty) {
-        close(opts->p_stdout[0]);
-    }
     close(opts->p_stdin[1]);
+    close(opts->p_stdout[0]);
     close(opts->p_stderr[0]);
+    close(opts->p_internal[0]);
 
     if (opts->usepty) {
 #ifndef __WINDOWS__
             /* disable echo */
             struct termios term_attrs;
             if (tcgetattr(opts->p_stdout[1], &term_attrs) < 0) {
-                return ORTE_ERROR;
+                return ORTE_ERR_PIPE_SETUP_FAILURE;
             }
             term_attrs.c_lflag &= ~ (ECHO | ECHOE | ECHOK |
                                      ECHOCTL | ECHOKE | ECHONL);
@@ -141,23 +151,23 @@ orte_iof_base_setup_child(orte_iof_base_io_conf_t *opts)
 #endif
                                      ONLCR);
             if (tcsetattr(opts->p_stdout[1], TCSANOW, &term_attrs) == -1) {
-                return ORTE_ERROR;
+                return ORTE_ERR_PIPE_SETUP_FAILURE;
             }
 #endif
         ret = dup2(opts->p_stdout[1], fileno(stdout));
-        if (ret < 0) return ORTE_ERROR;
+        if (ret < 0) return ORTE_ERR_PIPE_SETUP_FAILURE;
 
     } else {
         if(opts->p_stdout[1] != fileno(stdout)) {
             ret = dup2(opts->p_stdout[1], fileno(stdout));
-            if (ret < 0) return ORTE_ERROR;
+            if (ret < 0) return ORTE_ERR_PIPE_SETUP_FAILURE;
             close(opts->p_stdout[1]); 
         }
     }
     if (opts->connect_stdin) {
         if(opts->p_stdin[0] != fileno(stdin)) {
             ret = dup2(opts->p_stdin[0], fileno(stdin));
-            if (ret < 0) return ORTE_ERROR;
+            if (ret < 0) return ORTE_ERR_PIPE_SETUP_FAILURE;
             close(opts->p_stdin[0]); 
         }
     } else {
@@ -173,10 +183,18 @@ orte_iof_base_setup_child(orte_iof_base_io_conf_t *opts)
     }
     if(opts->p_stderr[1] != fileno(stderr)) {
         ret = dup2(opts->p_stderr[1], fileno(stderr));
-        if (ret < 0) return ORTE_ERROR;
+        if (ret < 0) return ORTE_ERR_PIPE_SETUP_FAILURE;
         close(opts->p_stderr[1]);
     }
 
+    /* Set an environment variable that the new child process can use
+       to get the fd of the pipe connected to the INTERNAL IOF tag. */
+    asprintf(&str, "%d", opts->p_internal[1]);
+    if (NULL != str) {
+        opal_setenv("OPAL_OUTPUT_STDERR_FD", str, true, env);
+        free(str);
+    }
+
     return ORTE_SUCCESS;
 }
 
@@ -187,17 +205,15 @@ orte_iof_base_setup_parent(const orte_process_name_t* name,
 {
     int ret;
 
-    if (! opts->usepty) {
-        close(opts->p_stdout[1]);
-    }
     close(opts->p_stdin[0]);
+    close(opts->p_stdout[1]);
     close(opts->p_stderr[1]);
+    close(opts->p_internal[1]);
 
     /* connect stdin endpoint */
     if (opts->connect_stdin) {
         /* and connect the pty to stdin */
-        ret = orte_iof.iof_publish(name, ORTE_IOF_SINK,
-                                   ORTE_IOF_STDIN, opts->p_stdin[1]);
+        ret = orte_iof.pull(name, ORTE_IOF_STDIN, opts->p_stdin[1]);
         if(ORTE_SUCCESS != ret) {
             ORTE_ERROR_LOG(ret);
             return ret;
@@ -206,16 +222,20 @@ orte_iof_base_setup_parent(const orte_process_name_t* name,
         close(opts->p_stdin[1]);
     }
 
-    /* connect read end to IOF */
-    ret = orte_iof.iof_publish(name, ORTE_IOF_SOURCE,
-                              ORTE_IOF_STDOUT, opts->p_stdout[0]);
+    /* connect read ends to IOF */
+    ret = orte_iof.push(name, ORTE_IOF_STDOUT, opts->p_stdout[0]);
     if(ORTE_SUCCESS != ret) {
         ORTE_ERROR_LOG(ret);
         return ret;
     }
 
-    ret = orte_iof.iof_publish(name, ORTE_IOF_SOURCE, 
-                              ORTE_IOF_STDERR, opts->p_stderr[0]);
+    ret = orte_iof.push(name, ORTE_IOF_STDERR, opts->p_stderr[0]);
+    if(ORTE_SUCCESS != ret) {
+        ORTE_ERROR_LOG(ret);
+        return ret;
+    }
+
+    ret = orte_iof.push(name, ORTE_IOF_STDDIAG, opts->p_internal[0]);
     if(ORTE_SUCCESS != ret) {
         ORTE_ERROR_LOG(ret);
         return ret;
